@@ -11,16 +11,18 @@ import (
 	"net/url"
 	"os"
 	"strings"
+	"time"
 
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/cancel"
 	"github.com/lxc/lxd/shared/ioprogress"
+	"github.com/lxc/lxd/shared/units"
 )
 
 // Image handling functions
 
-// GetImages returns a list of available images as Image structs
+// GetImages returns a list of available images as Image structs.
 func (r *ProtocolLXD) GetImages() ([]api.Image, error) {
 	images := []api.Image{}
 
@@ -32,53 +34,68 @@ func (r *ProtocolLXD) GetImages() ([]api.Image, error) {
 	return images, nil
 }
 
-// GetImageFingerprints returns a list of available image fingerprints
-func (r *ProtocolLXD) GetImageFingerprints() ([]string, error) {
-	urls := []string{}
+// GetImagesWithFilter returns a filtered list of available images as Image structs.
+func (r *ProtocolLXD) GetImagesWithFilter(filters []string) ([]api.Image, error) {
+	if !r.HasExtension("api_filtering") {
+		return nil, fmt.Errorf("The server is missing the required \"api_filtering\" API extension")
+	}
 
-	// Fetch the raw value
-	_, err := r.queryStruct("GET", "/images", nil, "", &urls)
+	images := []api.Image{}
+
+	v := url.Values{}
+	v.Set("recursion", "1")
+	v.Set("filter", parseFilters(filters))
+
+	_, err := r.queryStruct("GET", fmt.Sprintf("/images?%s", v.Encode()), nil, "", &images)
 	if err != nil {
 		return nil, err
 	}
 
-	// Parse it
-	fingerprints := []string{}
-	for _, url := range urls {
-		fields := strings.Split(url, "/images/")
-		fingerprints = append(fingerprints, fields[len(fields)-1])
-	}
-
-	return fingerprints, nil
+	return images, nil
 }
 
-// GetImage returns an Image struct for the provided fingerprint
+// GetImageFingerprints returns a list of available image fingerprints.
+func (r *ProtocolLXD) GetImageFingerprints() ([]string, error) {
+	// Fetch the raw URL values.
+	urls := []string{}
+	baseURL := "/images"
+	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
+	if err != nil {
+		return nil, err
+	}
+
+	// Parse it.
+	return urlsToResourceNames(baseURL, urls...)
+}
+
+// GetImage returns an Image struct for the provided fingerprint.
 func (r *ProtocolLXD) GetImage(fingerprint string) (*api.Image, string, error) {
 	return r.GetPrivateImage(fingerprint, "")
 }
 
-// GetImageFile downloads an image from the server, returning an ImageFileRequest struct
+// GetImageFile downloads an image from the server, returning an ImageFileRequest struct.
 func (r *ProtocolLXD) GetImageFile(fingerprint string, req ImageFileRequest) (*ImageFileResponse, error) {
 	return r.GetPrivateImageFile(fingerprint, "", req)
 }
 
-// GetImageSecret is a helper around CreateImageSecret that returns a secret for the image
+// GetImageSecret is a helper around CreateImageSecret that returns a secret for the image.
 func (r *ProtocolLXD) GetImageSecret(fingerprint string) (string, error) {
 	op, err := r.CreateImageSecret(fingerprint)
 	if err != nil {
 		return "", err
 	}
+
 	opAPI := op.Get()
 
 	return opAPI.Metadata["secret"].(string), nil
 }
 
-// GetPrivateImage is similar to GetImage but allows passing a secret download token
+// GetPrivateImage is similar to GetImage but allows passing a secret download token.
 func (r *ProtocolLXD) GetPrivateImage(fingerprint string, secret string) (*api.Image, string, error) {
 	image := api.Image{}
 
 	// Build the API path
-	path := fmt.Sprintf("/images/%s", url.QueryEscape(fingerprint))
+	path := fmt.Sprintf("/images/%s", url.PathEscape(fingerprint))
 	var err error
 	path, err = r.setQueryAttributes(path)
 	if err != nil {
@@ -101,14 +118,14 @@ func (r *ProtocolLXD) GetPrivateImage(fingerprint string, secret string) (*api.I
 	return &image, etag, nil
 }
 
-// GetPrivateImageFile is similar to GetImageFile but allows passing a secret download token
+// GetPrivateImageFile is similar to GetImageFile but allows passing a secret download token.
 func (r *ProtocolLXD) GetPrivateImageFile(fingerprint string, secret string, req ImageFileRequest) (*ImageFileResponse, error) {
-	// Sanity checks
+	// Quick checks.
 	if req.MetaFile == nil && req.RootfsFile == nil {
 		return nil, fmt.Errorf("No file requested")
 	}
 
-	uri := fmt.Sprintf("/1.0/images/%s/export", url.QueryEscape(fingerprint))
+	uri := fmt.Sprintf("/1.0/images/%s/export", url.PathEscape(fingerprint))
 
 	var err error
 	uri, err = r.setQueryAttributes(uri)
@@ -131,7 +148,7 @@ func (r *ProtocolLXD) GetPrivateImageFile(fingerprint string, secret string, req
 	}
 
 	// Build the URL
-	uri = fmt.Sprintf("%s%s", r.httpHost, uri)
+	uri = fmt.Sprintf("%s%s", r.httpBaseURL.String(), uri)
 	if secret != "" {
 		uri, err = setQueryParam(uri, "secret", secret)
 		if err != nil {
@@ -139,7 +156,14 @@ func (r *ProtocolLXD) GetPrivateImageFile(fingerprint string, secret string, req
 		}
 	}
 
-	return lxdDownloadImage(fingerprint, uri, r.httpUserAgent, r.http, req)
+	// Use relatively short response header timeout so as not to hold the image lock open too long.
+	// Deference client and transport in order to clone them so as to not modify timeout of base client.
+	httpClient := *r.http
+	httpTransport := httpClient.Transport.(*http.Transport).Clone()
+	httpTransport.ResponseHeaderTimeout = 30 * time.Second
+	httpClient.Transport = httpTransport
+
+	return lxdDownloadImage(fingerprint, uri, r.httpUserAgent, &httpClient, req)
 }
 
 func lxdDownloadImage(fingerprint string, uri string, userAgent string, client *http.Client, req ImageFileRequest) (*ImageFileResponse, error) {
@@ -161,7 +185,8 @@ func lxdDownloadImage(fingerprint string, uri string, userAgent string, client *
 	if err != nil {
 		return nil, err
 	}
-	defer response.Body.Close()
+
+	defer func() { _ = response.Body.Close() }()
 	defer close(doneCh)
 
 	if response.StatusCode != http.StatusOK {
@@ -179,15 +204,24 @@ func lxdDownloadImage(fingerprint string, uri string, userAgent string, client *
 	// Handle the data
 	body := response.Body
 	if req.ProgressHandler != nil {
-		body = &ioprogress.ProgressReader{
+		reader := &ioprogress.ProgressReader{
 			ReadCloser: response.Body,
 			Tracker: &ioprogress.ProgressTracker{
 				Length: response.ContentLength,
-				Handler: func(percent int64, speed int64) {
-					req.ProgressHandler(ioprogress.ProgressData{Text: fmt.Sprintf("%d%% (%s/s)", percent, shared.GetByteSizeString(speed, 2))})
-				},
 			},
 		}
+
+		if response.ContentLength > 0 {
+			reader.Tracker.Handler = func(percent int64, speed int64) {
+				req.ProgressHandler(ioprogress.ProgressData{Text: fmt.Sprintf("%d%% (%s/s)", percent, units.GetByteSizeString(speed, 2))})
+			}
+		} else {
+			reader.Tracker.Handler = func(received int64, speed int64) {
+				req.ProgressHandler(ioprogress.ProgressData{Text: fmt.Sprintf("%s (%s/s)", units.GetByteSizeString(received, 2), units.GetByteSizeString(speed, 2))})
+			}
+		}
+
+		body = reader
 	}
 
 	// Hashing
@@ -216,6 +250,7 @@ func lxdDownloadImage(fingerprint string, uri string, userAgent string, client *
 		if err != nil {
 			return nil, err
 		}
+
 		resp.MetaSize = size
 		resp.MetaName = part.FileName()
 
@@ -225,7 +260,7 @@ func lxdDownloadImage(fingerprint string, uri string, userAgent string, client *
 			return nil, err
 		}
 
-		if part.FormName() != "rootfs" {
+		if !shared.StringInSlice(part.FormName(), []string{"rootfs", "rootfs.img"}) {
 			return nil, fmt.Errorf("Invalid multipart image")
 		}
 
@@ -233,6 +268,7 @@ func lxdDownloadImage(fingerprint string, uri string, userAgent string, client *
 		if err != nil {
 			return nil, err
 		}
+
 		resp.RootfsSize = size
 		resp.RootfsName = part.FileName()
 
@@ -260,6 +296,7 @@ func lxdDownloadImage(fingerprint string, uri string, userAgent string, client *
 	if err != nil {
 		return nil, err
 	}
+
 	resp.MetaSize = size
 	resp.MetaName = filename
 
@@ -272,7 +309,7 @@ func lxdDownloadImage(fingerprint string, uri string, userAgent string, client *
 	return &resp, nil
 }
 
-// GetImageAliases returns the list of available aliases as ImageAliasesEntry structs
+// GetImageAliases returns the list of available aliases as ImageAliasesEntry structs.
 func (r *ProtocolLXD) GetImageAliases() ([]api.ImageAliasesEntry, error) {
 	aliases := []api.ImageAliasesEntry{}
 
@@ -285,32 +322,26 @@ func (r *ProtocolLXD) GetImageAliases() ([]api.ImageAliasesEntry, error) {
 	return aliases, nil
 }
 
-// GetImageAliasNames returns the list of available alias names
+// GetImageAliasNames returns the list of available alias names.
 func (r *ProtocolLXD) GetImageAliasNames() ([]string, error) {
+	// Fetch the raw URL values.
 	urls := []string{}
-
-	// Fetch the raw value
-	_, err := r.queryStruct("GET", "/images/aliases", nil, "", &urls)
+	baseURL := "/images/aliases"
+	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
 
-	// Parse it
-	names := []string{}
-	for _, url := range urls {
-		fields := strings.Split(url, "/images/aliases/")
-		names = append(names, fields[len(fields)-1])
-	}
-
-	return names, nil
+	// Parse it.
+	return urlsToResourceNames(baseURL, urls...)
 }
 
-// GetImageAlias returns an existing alias as an ImageAliasesEntry struct
+// GetImageAlias returns an existing alias as an ImageAliasesEntry struct.
 func (r *ProtocolLXD) GetImageAlias(name string) (*api.ImageAliasesEntry, string, error) {
 	alias := api.ImageAliasesEntry{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct("GET", fmt.Sprintf("/images/aliases/%s", url.QueryEscape(name)), nil, "", &alias)
+	etag, err := r.queryStruct("GET", fmt.Sprintf("/images/aliases/%s", url.PathEscape(name)), nil, "", &alias)
 	if err != nil {
 		return nil, "", err
 	}
@@ -318,7 +349,42 @@ func (r *ProtocolLXD) GetImageAlias(name string) (*api.ImageAliasesEntry, string
 	return &alias, etag, nil
 }
 
-// CreateImage requests that LXD creates, copies or import a new image
+// GetImageAliasType returns an existing alias as an ImageAliasesEntry struct.
+func (r *ProtocolLXD) GetImageAliasType(imageType string, name string) (*api.ImageAliasesEntry, string, error) {
+	alias, etag, err := r.GetImageAlias(name)
+	if err != nil {
+		return nil, "", err
+	}
+
+	if imageType != "" {
+		if alias.Type == "" {
+			alias.Type = "container"
+		}
+
+		if alias.Type != imageType {
+			return nil, "", fmt.Errorf("Alias doesn't exist for the specified type")
+		}
+	}
+
+	return alias, etag, nil
+}
+
+// GetImageAliasArchitectures returns a map of architectures / targets.
+func (r *ProtocolLXD) GetImageAliasArchitectures(imageType string, name string) (map[string]*api.ImageAliasesEntry, error) {
+	alias, _, err := r.GetImageAliasType(imageType, name)
+	if err != nil {
+		return nil, err
+	}
+
+	img, _, err := r.GetImage(alias.Target)
+	if err != nil {
+		return nil, err
+	}
+
+	return map[string]*api.ImageAliasesEntry{img.Architecture: alias}, nil
+}
+
+// CreateImage requests that LXD creates, copies or import a new image.
 func (r *ProtocolLXD) CreateImage(image api.ImagesPost, args *ImageCreateArgs) (Operation, error) {
 	if image.CompressionAlgorithm != "" {
 		if !r.HasExtension("image_compression_algorithm") {
@@ -355,7 +421,8 @@ func (r *ProtocolLXD) CreateImage(image api.ImagesPost, args *ImageCreateArgs) (
 		if err != nil {
 			return nil, err
 		}
-		defer os.Remove(tmpfile.Name())
+
+		defer func() { _ = os.Remove(tmpfile.Name()) }()
 
 		// Setup the multipart writer
 		w := multipart.NewWriter(tmpfile)
@@ -372,7 +439,12 @@ func (r *ProtocolLXD) CreateImage(image api.ImagesPost, args *ImageCreateArgs) (
 		}
 
 		// Rootfs file
-		fw, err = w.CreateFormFile("rootfs", args.RootfsName)
+		if args.Type == "virtual-machine" {
+			fw, err = w.CreateFormFile("rootfs.img", args.RootfsName)
+		} else {
+			fw, err = w.CreateFormFile("rootfs", args.RootfsName)
+		}
+
 		if err != nil {
 			return nil, err
 		}
@@ -383,7 +455,10 @@ func (r *ProtocolLXD) CreateImage(image api.ImagesPost, args *ImageCreateArgs) (
 		}
 
 		// Done writing to multipart
-		w.Close()
+		err = w.Close()
+		if err != nil {
+			return nil, err
+		}
 
 		// Figure out the size of the whole thing
 		size, err := tmpfile.Seek(0, 2)
@@ -397,21 +472,25 @@ func (r *ProtocolLXD) CreateImage(image api.ImagesPost, args *ImageCreateArgs) (
 		}
 
 		// Setup progress handler
-		body = &ioprogress.ProgressReader{
-			ReadCloser: tmpfile,
-			Tracker: &ioprogress.ProgressTracker{
-				Length: size,
-				Handler: func(percent int64, speed int64) {
-					args.ProgressHandler(ioprogress.ProgressData{Text: fmt.Sprintf("%d%% (%s/s)", percent, shared.GetByteSizeString(speed, 2))})
+		if args.ProgressHandler != nil {
+			body = &ioprogress.ProgressReader{
+				ReadCloser: tmpfile,
+				Tracker: &ioprogress.ProgressTracker{
+					Length: size,
+					Handler: func(percent int64, speed int64) {
+						args.ProgressHandler(ioprogress.ProgressData{Text: fmt.Sprintf("%d%% (%s/s)", percent, units.GetByteSizeString(speed, 2))})
+					},
 				},
-			},
+			}
+		} else {
+			body = tmpfile
 		}
 
 		contentType = w.FormDataContentType()
 	}
 
 	// Prepare the HTTP request
-	reqURL, err := r.setQueryAttributes(fmt.Sprintf("%s/1.0/images", r.httpHost))
+	reqURL, err := r.setQueryAttributes(fmt.Sprintf("%s/1.0/images", r.httpBaseURL.String()))
 	if err != nil {
 		return nil, err
 	}
@@ -441,17 +520,32 @@ func (r *ProtocolLXD) CreateImage(image api.ImagesPost, args *ImageCreateArgs) (
 		req.Header.Set("X-LXD-properties", imgProps.Encode())
 	}
 
+	if len(image.Profiles) > 0 {
+		imgProfiles := url.Values{}
+
+		for _, v := range image.Profiles {
+			imgProfiles.Add("profile", v)
+		}
+
+		req.Header.Set("X-LXD-profiles", imgProfiles.Encode())
+	}
+
 	// Set the user agent
-	if r.httpUserAgent != "" {
-		req.Header.Set("User-Agent", r.httpUserAgent)
+	if image.Source != nil && image.Source.Fingerprint != "" && image.Source.Secret != "" && image.Source.Mode == "push" {
+		// Set fingerprint
+		req.Header.Set("X-LXD-fingerprint", image.Source.Fingerprint)
+
+		// Set secret
+		req.Header.Set("X-LXD-secret", image.Source.Secret)
 	}
 
 	// Send the request
-	resp, err := r.do(req)
+	resp, err := r.DoHTTP(req)
 	if err != nil {
 		return nil, err
 	}
-	defer resp.Body.Close()
+
+	defer func() { _ = resp.Body.Close() }()
 
 	// Handle errors
 	response, _, err := lxdParseResponse(resp)
@@ -475,7 +569,7 @@ func (r *ProtocolLXD) CreateImage(image api.ImagesPost, args *ImageCreateArgs) (
 	return &op, nil
 }
 
-// tryCopyImage iterates through the source server URLs until one lets it download the image
+// tryCopyImage iterates through the source server URLs until one lets it download the image.
 func (r *ProtocolLXD) tryCopyImage(req api.ImagesPost, urls []string) (RemoteOperation, error) {
 	if len(urls) == 0 {
 		return nil, fmt.Errorf("The source server isn't listening on the network")
@@ -498,9 +592,13 @@ func (r *ProtocolLXD) tryCopyImage(req api.ImagesPost, urls []string) (RemoteOpe
 				return
 			}
 
+			var errors []remoteOperationResult
+
 			// Get the operation data
 			op, err := rop.GetTarget()
 			if err != nil {
+				errors = append(errors, remoteOperationResult{Error: err})
+				rop.err = remoteOperationError("Failed to get operation data", errors)
 				return
 			}
 
@@ -513,7 +611,12 @@ func (r *ProtocolLXD) tryCopyImage(req api.ImagesPost, urls []string) (RemoteOpe
 				alias.Name = entry.Name
 				alias.Target = fingerprint
 
-				r.CreateImageAlias(alias)
+				err := r.CreateImageAlias(alias)
+				if err != nil {
+					errors = append(errors, remoteOperationResult{Error: err})
+					rop.err = remoteOperationError("Failed to create image alias", errors)
+					return
+				}
 			}
 		}()
 	}
@@ -521,26 +624,33 @@ func (r *ProtocolLXD) tryCopyImage(req api.ImagesPost, urls []string) (RemoteOpe
 	// Forward targetOp to remote op
 	go func() {
 		success := false
-		errors := map[string]error{}
+		var errors []remoteOperationResult
 		for _, serverURL := range urls {
 			req.Source.Server = serverURL
 
 			op, err := r.CreateImage(req, nil)
 			if err != nil {
-				errors[serverURL] = err
+				errors = append(errors, remoteOperationResult{URL: serverURL, Error: err})
 				continue
 			}
 
+			rop.handlerLock.Lock()
 			rop.targetOp = op
+			rop.handlerLock.Unlock()
 
 			for _, handler := range rop.handlers {
-				rop.targetOp.AddHandler(handler)
+				_, _ = rop.targetOp.AddHandler(handler)
 			}
 
 			err = rop.targetOp.Wait()
 			if err != nil {
-				errors[serverURL] = err
-				continue
+				errors = append(errors, remoteOperationResult{URL: serverURL, Error: err})
+
+				if shared.IsConnectionError(err) {
+					continue
+				}
+
+				break
 			}
 
 			success = true
@@ -557,19 +667,191 @@ func (r *ProtocolLXD) tryCopyImage(req api.ImagesPost, urls []string) (RemoteOpe
 	return &rop, nil
 }
 
-// CopyImage copies an image from a remote server. Additional options can be passed using ImageCopyArgs
+// CopyImage copies an image from a remote server. Additional options can be passed using ImageCopyArgs.
 func (r *ProtocolLXD) CopyImage(source ImageServer, image api.Image, args *ImageCopyArgs) (RemoteOperation, error) {
-	// Sanity checks
-	if r == source {
+	// Quick checks.
+	if r.isSameServer(source) {
 		return nil, fmt.Errorf("The source and target servers must be different")
 	}
 
+	// Handle profile list overrides.
+	if args.Profiles != nil {
+		if !r.HasExtension("image_copy_profile") {
+			return nil, fmt.Errorf("The server is missing the required \"image_copy_profile\" API extension")
+		}
+
+		image.Profiles = args.Profiles
+	} else {
+		// If profiles aren't provided, clear the list on the source to
+		// avoid requiring the destination to have them all.
+		image.Profiles = nil
+	}
+
 	// Get source server connection information
 	info, err := source.GetConnectionInfo()
 	if err != nil {
 		return nil, err
 	}
 
+	// Push mode
+	if args != nil && args.Mode == "push" {
+		// Get certificate and URL
+		info, err := r.GetConnectionInfo()
+		if err != nil {
+			return nil, err
+		}
+
+		imagesPost := api.ImagesPost{
+			Source: &api.ImagesPostSource{
+				Fingerprint: image.Fingerprint,
+				Mode:        args.Mode,
+			},
+		}
+
+		if args.CopyAliases {
+			imagesPost.Aliases = image.Aliases
+		}
+
+		imagesPost.ExpiresAt = image.ExpiresAt
+		imagesPost.Properties = image.Properties
+		imagesPost.Public = args.Public
+
+		// Receive token from target server. This token is later passed to the source which will use
+		// it, together with the URL and certificate, to connect to the target.
+		tokenOp, err := r.CreateImage(imagesPost, nil)
+		if err != nil {
+			return nil, err
+		}
+
+		opAPI := tokenOp.Get()
+
+		secret, ok := opAPI.Metadata["secret"]
+		if !ok {
+			return nil, fmt.Errorf("No token provided")
+		}
+
+		req := api.ImageExportPost{
+			Target:      info.URL,
+			Certificate: info.Certificate,
+			Secret:      secret.(string),
+			Aliases:     image.Aliases,
+			Project:     info.Project,
+			Profiles:    image.Profiles,
+		}
+
+		exportOp, err := source.ExportImage(image.Fingerprint, req)
+		if err != nil {
+			_ = tokenOp.Cancel()
+			return nil, err
+		}
+
+		rop := remoteOperation{
+			targetOp: exportOp,
+			chDone:   make(chan bool),
+		}
+
+		// Forward targetOp to remote op
+		go func() {
+			rop.err = rop.targetOp.Wait()
+			_ = tokenOp.Cancel()
+			close(rop.chDone)
+		}()
+
+		return &rop, nil
+	}
+
+	// Relay mode
+	if args != nil && args.Mode == "relay" {
+		metaFile, err := ioutil.TempFile("", "lxc_image_")
+		if err != nil {
+			return nil, err
+		}
+
+		defer func() { _ = os.Remove(metaFile.Name()) }()
+
+		rootfsFile, err := ioutil.TempFile("", "lxc_image_")
+		if err != nil {
+			return nil, err
+		}
+
+		defer func() { _ = os.Remove(rootfsFile.Name()) }()
+
+		// Import image
+		req := ImageFileRequest{
+			MetaFile:   metaFile,
+			RootfsFile: rootfsFile,
+		}
+
+		resp, err := source.GetImageFile(image.Fingerprint, req)
+		if err != nil {
+			return nil, err
+		}
+
+		// Export image
+		_, err = metaFile.Seek(0, 0)
+		if err != nil {
+			return nil, err
+		}
+
+		_, err = rootfsFile.Seek(0, 0)
+		if err != nil {
+			return nil, err
+		}
+
+		imagePost := api.ImagesPost{}
+		imagePost.Public = args.Public
+		imagePost.Profiles = image.Profiles
+
+		if args.CopyAliases {
+			imagePost.Aliases = image.Aliases
+			if args.Aliases != nil {
+				imagePost.Aliases = append(imagePost.Aliases, args.Aliases...)
+			}
+		}
+
+		createArgs := &ImageCreateArgs{
+			MetaFile: metaFile,
+			MetaName: image.Filename,
+			Type:     image.Type,
+		}
+
+		if resp.RootfsName != "" {
+			// Deal with split images
+			createArgs.RootfsFile = rootfsFile
+			createArgs.RootfsName = image.Filename
+		}
+
+		rop := remoteOperation{
+			chDone: make(chan bool),
+		}
+
+		go func() {
+			defer close(rop.chDone)
+
+			op, err := r.CreateImage(imagePost, createArgs)
+			if err != nil {
+				rop.err = remoteOperationError("Failed to copy image", nil)
+				return
+			}
+
+			rop.handlerLock.Lock()
+			rop.targetOp = op
+			rop.handlerLock.Unlock()
+
+			for _, handler := range rop.handlers {
+				_, _ = rop.targetOp.AddHandler(handler)
+			}
+
+			err = rop.targetOp.Wait()
+			if err != nil {
+				rop.err = remoteOperationError("Failed to copy image", nil)
+				return
+			}
+		}()
+
+		return &rop, nil
+	}
+
 	// Prepare the copy request
 	req := api.ImagesPost{
 		Source: &api.ImagesPostSource{
@@ -580,7 +862,15 @@ func (r *ProtocolLXD) CopyImage(source ImageServer, image api.Image, args *Image
 			Fingerprint: image.Fingerprint,
 			Mode:        "pull",
 			Type:        "image",
+			Project:     info.Project,
 		},
+		ImagePut: api.ImagePut{
+			Profiles: image.Profiles,
+		},
+	}
+
+	if args != nil {
+		req.Source.ImageType = args.Type
 	}
 
 	// Generate secret token if needed
@@ -610,10 +900,10 @@ func (r *ProtocolLXD) CopyImage(source ImageServer, image api.Image, args *Image
 	return r.tryCopyImage(req, info.Addresses)
 }
 
-// UpdateImage updates the image definition
+// UpdateImage updates the image definition.
 func (r *ProtocolLXD) UpdateImage(fingerprint string, image api.ImagePut, ETag string) error {
 	// Send the request
-	_, _, err := r.query("PUT", fmt.Sprintf("/images/%s", url.QueryEscape(fingerprint)), image, ETag)
+	_, _, err := r.query("PUT", fmt.Sprintf("/images/%s", url.PathEscape(fingerprint)), image, ETag)
 	if err != nil {
 		return err
 	}
@@ -621,10 +911,10 @@ func (r *ProtocolLXD) UpdateImage(fingerprint string, image api.ImagePut, ETag s
 	return nil
 }
 
-// DeleteImage requests that LXD removes an image from the store
+// DeleteImage requests that LXD removes an image from the store.
 func (r *ProtocolLXD) DeleteImage(fingerprint string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation("DELETE", fmt.Sprintf("/images/%s", url.QueryEscape(fingerprint)), nil, "")
+	op, _, err := r.queryOperation("DELETE", fmt.Sprintf("/images/%s", url.PathEscape(fingerprint)), nil, "")
 	if err != nil {
 		return nil, err
 	}
@@ -632,14 +922,14 @@ func (r *ProtocolLXD) DeleteImage(fingerprint string) (Operation, error) {
 	return op, nil
 }
 
-// RefreshImage requests that LXD issues an image refresh
+// RefreshImage requests that LXD issues an image refresh.
 func (r *ProtocolLXD) RefreshImage(fingerprint string) (Operation, error) {
 	if !r.HasExtension("image_force_refresh") {
 		return nil, fmt.Errorf("The server is missing the required \"image_force_refresh\" API extension")
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/images/%s/refresh", url.QueryEscape(fingerprint)), nil, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/images/%s/refresh", url.PathEscape(fingerprint)), nil, "")
 	if err != nil {
 		return nil, err
 	}
@@ -647,10 +937,10 @@ func (r *ProtocolLXD) RefreshImage(fingerprint string) (Operation, error) {
 	return op, nil
 }
 
-// CreateImageSecret requests that LXD issues a temporary image secret
+// CreateImageSecret requests that LXD issues a temporary image secret.
 func (r *ProtocolLXD) CreateImageSecret(fingerprint string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation("POST", fmt.Sprintf("/images/%s/secret", url.QueryEscape(fingerprint)), nil, "")
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/images/%s/secret", url.PathEscape(fingerprint)), nil, "")
 	if err != nil {
 		return nil, err
 	}
@@ -658,7 +948,7 @@ func (r *ProtocolLXD) CreateImageSecret(fingerprint string) (Operation, error) {
 	return op, nil
 }
 
-// CreateImageAlias sets up a new image alias
+// CreateImageAlias sets up a new image alias.
 func (r *ProtocolLXD) CreateImageAlias(alias api.ImageAliasesPost) error {
 	// Send the request
 	_, _, err := r.query("POST", "/images/aliases", alias, "")
@@ -669,10 +959,10 @@ func (r *ProtocolLXD) CreateImageAlias(alias api.ImageAliasesPost) error {
 	return nil
 }
 
-// UpdateImageAlias updates the image alias definition
+// UpdateImageAlias updates the image alias definition.
 func (r *ProtocolLXD) UpdateImageAlias(name string, alias api.ImageAliasesEntryPut, ETag string) error {
 	// Send the request
-	_, _, err := r.query("PUT", fmt.Sprintf("/images/aliases/%s", url.QueryEscape(name)), alias, ETag)
+	_, _, err := r.query("PUT", fmt.Sprintf("/images/aliases/%s", url.PathEscape(name)), alias, ETag)
 	if err != nil {
 		return err
 	}
@@ -680,10 +970,10 @@ func (r *ProtocolLXD) UpdateImageAlias(name string, alias api.ImageAliasesEntryP
 	return nil
 }
 
-// RenameImageAlias renames an existing image alias
+// RenameImageAlias renames an existing image alias.
 func (r *ProtocolLXD) RenameImageAlias(name string, alias api.ImageAliasesEntryPost) error {
 	// Send the request
-	_, _, err := r.query("POST", fmt.Sprintf("/images/aliases/%s", url.QueryEscape(name)), alias, "")
+	_, _, err := r.query("POST", fmt.Sprintf("/images/aliases/%s", url.PathEscape(name)), alias, "")
 	if err != nil {
 		return err
 	}
@@ -691,13 +981,28 @@ func (r *ProtocolLXD) RenameImageAlias(name string, alias api.ImageAliasesEntryP
 	return nil
 }
 
-// DeleteImageAlias removes an alias from the LXD image store
+// DeleteImageAlias removes an alias from the LXD image store.
 func (r *ProtocolLXD) DeleteImageAlias(name string) error {
 	// Send the request
-	_, _, err := r.query("DELETE", fmt.Sprintf("/images/aliases/%s", url.QueryEscape(name)), nil, "")
+	_, _, err := r.query("DELETE", fmt.Sprintf("/images/aliases/%s", url.PathEscape(name)), nil, "")
 	if err != nil {
 		return err
 	}
 
 	return nil
 }
+
+// ExportImage exports (copies) an image to a remote server.
+func (r *ProtocolLXD) ExportImage(fingerprint string, image api.ImageExportPost) (Operation, error) {
+	if !r.HasExtension("images_push_relay") {
+		return nil, fmt.Errorf("The server is missing the required \"images_push_relay\" API extension")
+	}
+
+	// Send the request
+	op, _, err := r.queryOperation("POST", fmt.Sprintf("/images/%s/export", url.PathEscape(fingerprint)), &image, "")
+	if err != nil {
+		return nil, err
+	}
+
+	return op, nil
+}
