@@ -2,13 +2,13 @@ package config
 
 import (
 	"fmt"
+	"os"
 	"path/filepath"
 
 	"github.com/juju/persistent-cookiejar"
-	"gopkg.in/macaroon-bakery.v2/httpbakery"
 )
 
-// Config holds settings to be used by a client or daemon
+// Config holds settings to be used by a client or daemon.
 type Config struct {
 	// DefaultRemote holds the remote daemon name from the Remotes map
 	// that the client should communicate with by default
@@ -30,15 +30,27 @@ type Config struct {
 	// PromptPassword is a helper function used when encountering an encrypted key
 	PromptPassword func(filename string) (string, error) `yaml:"-"`
 
-	// ProjectOverride allows overriding the default project for container queries
+	// ProjectOverride allows overriding the default project
 	ProjectOverride string `yaml:"-"`
 
-	authInteractor []httpbakery.Interactor
-
-	cookiejar *cookiejar.Jar
+	// Cookie jars
+	cookieJars map[string]*cookiejar.Jar
 }
 
-// ConfigPath returns a joined path of the configuration directory and passed arguments
+// GlobalConfigPath returns a joined path of the global configuration directory and passed arguments.
+func (c *Config) GlobalConfigPath(paths ...string) string {
+	configDir := "/etc/lxd"
+	if os.Getenv("LXD_GLOBAL_CONF") != "" {
+		configDir = os.Getenv("LXD_GLOBAL_CONF")
+	}
+
+	path := []string{configDir}
+	path = append(path, paths...)
+
+	return filepath.Join(path...)
+}
+
+// ConfigPath returns a joined path of the configuration directory and passed arguments.
 func (c *Config) ConfigPath(paths ...string) string {
 	path := []string{c.ConfigDir}
 	path = append(path, paths...)
@@ -46,20 +58,24 @@ func (c *Config) ConfigPath(paths ...string) string {
 	return filepath.Join(path...)
 }
 
-// ServerCertPath returns the path for the remote's server certificate
+// CookiesPath returns the path for the remote's cookie jar.
+func (c *Config) CookiesPath(remote string) string {
+	return c.ConfigPath("jars", remote)
+}
+
+// ServerCertPath returns the path for the remote's server certificate.
 func (c *Config) ServerCertPath(remote string) string {
+	if c.Remotes[remote].Global {
+		return c.GlobalConfigPath("servercerts", fmt.Sprintf("%s.crt", remote))
+	}
+
 	return c.ConfigPath("servercerts", fmt.Sprintf("%s.crt", remote))
 }
 
-// SetAuthInteractor sets the interactor for macaroon-based authorization
-func (c *Config) SetAuthInteractor(interactor []httpbakery.Interactor) {
-	c.authInteractor = interactor
-}
-
-// SaveCookies saves cookies to file
+// SaveCookies saves cookies to file.
 func (c *Config) SaveCookies() {
-	if c.cookiejar != nil {
-		c.cookiejar.Save()
+	for _, jar := range c.cookieJars {
+		_ = jar.Save()
 	}
 }
 
@@ -67,14 +83,8 @@ func (c *Config) SaveCookies() {
 func NewConfig(configDir string, defaults bool) *Config {
 	config := &Config{ConfigDir: configDir}
 	if defaults {
-		config.Remotes = DefaultRemotes
-		config.DefaultRemote = "local"
-	}
-
-	if configDir != "" {
-		config.cookiejar, _ = cookiejar.New(
-			&cookiejar.Options{
-				Filename: filepath.Join(configDir, "cookies")})
+		config.Remotes = DefaultConfig.Remotes
+		config.DefaultRemote = DefaultConfig.DefaultRemote
 	}
 
 	return config
