@@ -1,38 +1,37 @@
 package main
 
 import (
-	"encoding/csv"
-	"encoding/json"
 	"fmt"
 	"io/ioutil"
 	"os"
 	"sort"
 	"strings"
-	"syscall"
 
-	"github.com/olekukonko/tablewriter"
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
+	"github.com/lxc/lxd/lxc/utils"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 	cli "github.com/lxc/lxd/shared/cmd"
 	"github.com/lxc/lxd/shared/i18n"
 	"github.com/lxc/lxd/shared/termios"
+	"github.com/lxc/lxd/shared/units"
 )
 
 type cmdNetwork struct {
 	global *cmdGlobal
 
 	flagTarget string
+	flagType   string
 }
 
 func (c *cmdNetwork) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("network")
-	cmd.Short = i18n.G("Manage and attach containers to networks")
+	cmd.Use = usage("network")
+	cmd.Short = i18n.G("Manage and attach instances to networks")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Manage and attach containers to networks`))
+		`Manage and attach instances to networks`))
 
 	// Attach
 	networkAttachCmd := cmdNetworkAttach{global: c.global, network: c}
@@ -66,6 +65,10 @@ func (c *cmdNetwork) Command() *cobra.Command {
 	networkGetCmd := cmdNetworkGet{global: c.global, network: c}
 	cmd.AddCommand(networkGetCmd.Command())
 
+	// Info
+	networkInfoCmd := cmdNetworkInfo{global: c.global, network: c}
+	cmd.AddCommand(networkInfoCmd.Command())
+
 	// List
 	networkListCmd := cmdNetworkList{global: c.global, network: c}
 	cmd.AddCommand(networkListCmd.Command())
@@ -90,10 +93,29 @@ func (c *cmdNetwork) Command() *cobra.Command {
 	networkUnsetCmd := cmdNetworkUnset{global: c.global, network: c, networkSet: &networkSetCmd}
 	cmd.AddCommand(networkUnsetCmd.Command())
 
+	// ACL
+	networkACLCmd := cmdNetworkACL{global: c.global}
+	cmd.AddCommand(networkACLCmd.Command())
+
+	// Forward
+	networkForwardCmd := cmdNetworkForward{global: c.global}
+	cmd.AddCommand(networkForwardCmd.Command())
+
+	// Peer
+	networkPeerCmd := cmdNetworkPeer{global: c.global}
+	cmd.AddCommand(networkPeerCmd.Command())
+
+	// Zone
+	networkZoneCmd := cmdNetworkZone{global: c.global}
+	cmd.AddCommand(networkZoneCmd.Command())
+
+	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
+	cmd.Args = cobra.NoArgs
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
-// Attach
+// Attach.
 type cmdNetworkAttach struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -101,10 +123,10 @@ type cmdNetworkAttach struct {
 
 func (c *cmdNetworkAttach) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("attach [<remote>:]<network> <container> [<device name>] [<interface name>]")
-	cmd.Short = i18n.G("Attach network interfaces to containers")
+	cmd.Use = usage("attach", i18n.G("[<remote>:]<network> <instance> [<device name>] [<interface name>]"))
+	cmd.Short = i18n.G("Attach network interfaces to instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Attach new network interfaces to containers`))
+		`Attach new network interfaces to instances`))
 
 	cmd.RunE = c.Run
 
@@ -112,7 +134,7 @@ func (c *cmdNetworkAttach) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkAttach) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 4)
 	if exit {
 		return err
@@ -142,23 +164,35 @@ func (c *cmdNetworkAttach) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	// Prepare the container's device entry
-	device := map[string]string{
-		"type":    "nic",
-		"nictype": "macvlan",
-		"parent":  resource.name,
-	}
+	// Prepare the instance's device entry
+	var device map[string]string
+	if network.Managed && resource.server.HasExtension("instance_nic_network") {
+		// If network is managed, use the network property rather than nictype, so that the network's
+		// inherited properties are loaded into the NIC when started.
+		device = map[string]string{
+			"type":    "nic",
+			"network": network.Name,
+		}
+	} else {
+		// If network is unmanaged default to using a macvlan connected to the specified interface.
+		device = map[string]string{
+			"type":    "nic",
+			"nictype": "macvlan",
+			"parent":  resource.name,
+		}
 
-	if network.Type == "bridge" {
-		device["nictype"] = "bridged"
+		if network.Type == "bridge" {
+			// If the network type is an unmanaged bridge, use bridged NIC type.
+			device["nictype"] = "bridged"
+		}
 	}
 
 	if len(args) > 3 {
 		device["name"] = args[3]
 	}
 
-	// Add the device to the container
-	err = containerDeviceAdd(resource.server, args[1], devName, device)
+	// Add the device to the instance
+	err = instanceDeviceAdd(resource.server, args[1], devName, device)
 	if err != nil {
 		return err
 	}
@@ -166,7 +200,7 @@ func (c *cmdNetworkAttach) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Attach profile
+// Attach profile.
 type cmdNetworkAttachProfile struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -174,7 +208,7 @@ type cmdNetworkAttachProfile struct {
 
 func (c *cmdNetworkAttachProfile) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("attach-profile [<remote>:]<network> <profile> [<device name>] [<interface name>]")
+	cmd.Use = usage("attach-profile", i18n.G("[<remote>:]<network> <profile> [<device name>] [<interface name>]"))
 	cmd.Short = i18n.G("Attach network interfaces to profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Attach network interfaces to profiles`))
@@ -185,7 +219,7 @@ func (c *cmdNetworkAttachProfile) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkAttachProfile) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 4)
 	if exit {
 		return err
@@ -239,7 +273,7 @@ func (c *cmdNetworkAttachProfile) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Create
+// Create.
 type cmdNetworkCreate struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -247,19 +281,20 @@ type cmdNetworkCreate struct {
 
 func (c *cmdNetworkCreate) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("create [<remote>:]<network> [key=value...]")
+	cmd.Use = usage("create", i18n.G("[<remote>:]<network> [key=value...]"))
 	cmd.Short = i18n.G("Create new networks")
-	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Create new networks`))
+	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`Create new networks`))
 
 	cmd.Flags().StringVar(&c.network.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.Flags().StringVarP(&c.network.flagType, "type", "t", "", i18n.G("Network type")+"``")
+
 	cmd.RunE = c.Run
 
 	return cmd
 }
 
 func (c *cmdNetworkCreate) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, -1)
 	if exit {
 		return err
@@ -278,6 +313,7 @@ func (c *cmdNetworkCreate) Run(cmd *cobra.Command, args []string) error {
 	network := api.NetworksPost{}
 	network.Name = resource.name
 	network.Config = map[string]string{}
+	network.Type = c.network.flagType
 
 	for i := 1; i < len(args); i++ {
 		entry := strings.SplitN(args[i], "=", 2)
@@ -299,16 +335,18 @@ func (c *cmdNetworkCreate) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	if c.network.flagTarget != "" {
-		fmt.Printf(i18n.G("Network %s pending on member %s")+"\n", resource.name, c.network.flagTarget)
-	} else {
-		fmt.Printf(i18n.G("Network %s created")+"\n", resource.name)
+	if !c.global.flagQuiet {
+		if c.network.flagTarget != "" {
+			fmt.Printf(i18n.G("Network %s pending on member %s")+"\n", resource.name, c.network.flagTarget)
+		} else {
+			fmt.Printf(i18n.G("Network %s created")+"\n", resource.name)
+		}
 	}
 
 	return nil
 }
 
-// Delete
+// Delete.
 type cmdNetworkDelete struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -316,7 +354,7 @@ type cmdNetworkDelete struct {
 
 func (c *cmdNetworkDelete) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("delete [<remote>:]<network>")
+	cmd.Use = usage("delete", i18n.G("[<remote>:]<network>"))
 	cmd.Aliases = []string{"rm"}
 	cmd.Short = i18n.G("Delete networks")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -328,7 +366,7 @@ func (c *cmdNetworkDelete) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkDelete) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -352,11 +390,14 @@ func (c *cmdNetworkDelete) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf(i18n.G("Network %s deleted")+"\n", resource.name)
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Network %s deleted")+"\n", resource.name)
+	}
+
 	return nil
 }
 
-// Detach
+// Detach.
 type cmdNetworkDetach struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -364,10 +405,10 @@ type cmdNetworkDetach struct {
 
 func (c *cmdNetworkDetach) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("detach [<remote>:]<network> <container> [<device name>]")
-	cmd.Short = i18n.G("Detach network interfaces from containers")
+	cmd.Use = usage("detach", i18n.G("[<remote>:]<network> <instance> [<device name>]"))
+	cmd.Short = i18n.G("Detach network interfaces from instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Detach network interfaces from containers`))
+		`Detach network interfaces from instances`))
 
 	cmd.RunE = c.Run
 
@@ -375,7 +416,7 @@ func (c *cmdNetworkDetach) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkDetach) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 3)
 	if exit {
 		return err
@@ -399,16 +440,16 @@ func (c *cmdNetworkDetach) Run(cmd *cobra.Command, args []string) error {
 		devName = args[2]
 	}
 
-	// Get the container entry
-	container, etag, err := resource.server.GetContainer(args[1])
+	// Get the instance entry
+	inst, etag, err := resource.server.GetInstance(args[1])
 	if err != nil {
 		return err
 	}
 
 	// Find the device
 	if devName == "" {
-		for n, d := range container.Devices {
-			if d["type"] == "nic" && d["parent"] == resource.name {
+		for n, d := range inst.Devices {
+			if d["type"] == "nic" && (d["parent"] == resource.name || d["network"] == resource.name) {
 				if devName != "" {
 					return fmt.Errorf(i18n.G("More than one device matches, specify the device name"))
 				}
@@ -422,18 +463,18 @@ func (c *cmdNetworkDetach) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("No device found for this network"))
 	}
 
-	device, ok := container.Devices[devName]
+	device, ok := inst.Devices[devName]
 	if !ok {
 		return fmt.Errorf(i18n.G("The specified device doesn't exist"))
 	}
 
-	if device["type"] != "nic" || device["parent"] != resource.name {
+	if device["type"] != "nic" || (device["parent"] != resource.name && device["network"] != resource.name) {
 		return fmt.Errorf(i18n.G("The specified device doesn't match the network"))
 	}
 
 	// Remove the device
-	delete(container.Devices, devName)
-	op, err := resource.server.UpdateContainer(args[1], container.Writable(), etag)
+	delete(inst.Devices, devName)
+	op, err := resource.server.UpdateInstance(args[1], inst.Writable(), etag)
 	if err != nil {
 		return err
 	}
@@ -441,7 +482,7 @@ func (c *cmdNetworkDetach) Run(cmd *cobra.Command, args []string) error {
 	return op.Wait()
 }
 
-// Detach profile
+// Detach profile.
 type cmdNetworkDetachProfile struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -449,7 +490,7 @@ type cmdNetworkDetachProfile struct {
 
 func (c *cmdNetworkDetachProfile) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("detach-profile [<remote>:]<network> <container> [<device name>]")
+	cmd.Use = usage("detach-profile", i18n.G("[<remote>:]<network> <profile> [<device name>]"))
 	cmd.Short = i18n.G("Detach network interfaces from profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Detach network interfaces from profiles`))
@@ -460,7 +501,7 @@ func (c *cmdNetworkDetachProfile) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkDetachProfile) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 3)
 	if exit {
 		return err
@@ -493,7 +534,7 @@ func (c *cmdNetworkDetachProfile) Run(cmd *cobra.Command, args []string) error {
 	// Find the device
 	if devName == "" {
 		for n, d := range profile.Devices {
-			if d["type"] == "nic" && d["parent"] == resource.name {
+			if d["type"] == "nic" && (d["parent"] == resource.name || d["network"] == resource.name) {
 				if devName != "" {
 					return fmt.Errorf(i18n.G("More than one device matches, specify the device name"))
 				}
@@ -512,7 +553,7 @@ func (c *cmdNetworkDetachProfile) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("The specified device doesn't exist"))
 	}
 
-	if device["type"] != "nic" || device["parent"] != resource.name {
+	if device["type"] != "nic" || (device["parent"] != resource.name && device["network"] != resource.name) {
 		return fmt.Errorf(i18n.G("The specified device doesn't match the network"))
 	}
 
@@ -526,7 +567,7 @@ func (c *cmdNetworkDetachProfile) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Edit
+// Edit.
 type cmdNetworkEdit struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -534,7 +575,7 @@ type cmdNetworkEdit struct {
 
 func (c *cmdNetworkEdit) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("edit [<remote>:]<network>")
+	cmd.Use = usage("edit", i18n.G("[<remote>:]<network>"))
 	cmd.Short = i18n.G("Edit network configurations as YAML")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Edit network configurations as YAML`))
@@ -546,7 +587,7 @@ func (c *cmdNetworkEdit) Command() *cobra.Command {
 
 func (c *cmdNetworkEdit) helpTemplate() string {
 	return i18n.G(
-		`### This is a yaml representation of the network.
+		`### This is a YAML representation of the network.
 ### Any line starting with a '# will be ignored.
 ###
 ### A network consists of a set of configuration items.
@@ -565,7 +606,7 @@ func (c *cmdNetworkEdit) helpTemplate() string {
 }
 
 func (c *cmdNetworkEdit) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -584,7 +625,7 @@ func (c *cmdNetworkEdit) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// If stdin isn't a terminal, read text from it
-	if !termios.IsTerminal(int(syscall.Stdin)) {
+	if !termios.IsTerminal(getStdinFd()) {
 		contents, err := ioutil.ReadAll(os.Stdin)
 		if err != nil {
 			return err
@@ -631,7 +672,7 @@ func (c *cmdNetworkEdit) Run(cmd *cobra.Command, args []string) error {
 		// Respawn the editor
 		if err != nil {
 			fmt.Fprintf(os.Stderr, i18n.G("Config parsing error: %s")+"\n", err)
-			fmt.Println(i18n.G("Press enter to open the editor again"))
+			fmt.Println(i18n.G("Press enter to open the editor again or ctrl+c to abort change"))
 
 			_, err := os.Stdin.Read(make([]byte, 1))
 			if err != nil {
@@ -642,14 +683,17 @@ func (c *cmdNetworkEdit) Run(cmd *cobra.Command, args []string) error {
 			if err != nil {
 				return err
 			}
+
 			continue
 		}
+
 		break
 	}
+
 	return nil
 }
 
-// Get
+// Get.
 type cmdNetworkGet struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -657,7 +701,7 @@ type cmdNetworkGet struct {
 
 func (c *cmdNetworkGet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("get [<remote>:]<network> <key>")
+	cmd.Use = usage("get", i18n.G("[<remote>:]<network> <key>"))
 	cmd.Short = i18n.G("Get values for network configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Get values for network configuration keys`))
@@ -669,7 +713,7 @@ func (c *cmdNetworkGet) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkGet) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -707,7 +751,127 @@ func (c *cmdNetworkGet) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// List
+// Info.
+type cmdNetworkInfo struct {
+	global  *cmdGlobal
+	network *cmdNetwork
+}
+
+func (c *cmdNetworkInfo) Command() *cobra.Command {
+	cmd := &cobra.Command{}
+	cmd.Use = usage("info", i18n.G("[<remote>:]<network>"))
+	cmd.Short = i18n.G("Get runtime information on networks")
+	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
+		`Get runtime information on networks`))
+
+	cmd.Flags().StringVar(&c.network.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
+	cmd.RunE = c.Run
+
+	return cmd
+}
+
+func (c *cmdNetworkInfo) Run(cmd *cobra.Command, args []string) error {
+	// Quick checks.
+	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
+	if exit {
+		return err
+	}
+
+	// Parse remote.
+	resources, err := c.global.ParseServers(args[0])
+	if err != nil {
+		return err
+	}
+
+	resource := resources[0]
+	client := resource.server
+
+	if resource.name == "" {
+		return fmt.Errorf(i18n.G("Missing network name"))
+	}
+
+	// Targeting.
+	if c.network.flagTarget != "" {
+		if !client.IsClustered() {
+			return fmt.Errorf(i18n.G("To use --target, the destination remote must be a cluster"))
+		}
+
+		client = client.UseTarget(c.network.flagTarget)
+	}
+
+	state, err := client.GetNetworkState(resource.name)
+	if err != nil {
+		return err
+	}
+
+	// Interface information.
+	fmt.Printf(i18n.G("Name: %s")+"\n", resource.name)
+	fmt.Printf(i18n.G("MAC address: %s")+"\n", state.Hwaddr)
+	fmt.Printf(i18n.G("MTU: %d")+"\n", state.Mtu)
+	fmt.Printf(i18n.G("State: %s")+"\n", state.State)
+	fmt.Printf(i18n.G("Type: %s")+"\n", state.Type)
+
+	// IP addresses.
+	if state.Addresses != nil && len(state.Addresses) > 0 {
+		fmt.Println("")
+		fmt.Println(i18n.G("IP addresses:"))
+		for _, addr := range state.Addresses {
+			fmt.Printf("  %s\t%s/%s (%s)\n", addr.Family, addr.Address, addr.Netmask, addr.Scope)
+		}
+	}
+
+	// Network usage.
+	fmt.Println("")
+	fmt.Println(i18n.G("Network usage:"))
+	fmt.Printf("  %s: %s\n", i18n.G("Bytes received"), units.GetByteSizeString(state.Counters.BytesReceived, 2))
+	fmt.Printf("  %s: %s\n", i18n.G("Bytes sent"), units.GetByteSizeString(state.Counters.BytesSent, 2))
+	fmt.Printf("  %s: %d\n", i18n.G("Packets received"), state.Counters.PacketsReceived)
+	fmt.Printf("  %s: %d\n", i18n.G("Packets sent"), state.Counters.PacketsSent)
+
+	// Bond information.
+	if state.Bond != nil {
+		fmt.Println("")
+		fmt.Println(i18n.G("Bond:"))
+		fmt.Printf("  %s: %s\n", i18n.G("Mode"), state.Bond.Mode)
+		fmt.Printf("  %s: %s\n", i18n.G("Transmit policy"), state.Bond.TransmitPolicy)
+		fmt.Printf("  %s: %d\n", i18n.G("Up delay"), state.Bond.UpDelay)
+		fmt.Printf("  %s: %d\n", i18n.G("Down delay"), state.Bond.DownDelay)
+		fmt.Printf("  %s: %d\n", i18n.G("MII Frequency"), state.Bond.MIIFrequency)
+		fmt.Printf("  %s: %s\n", i18n.G("MII state"), state.Bond.MIIState)
+		fmt.Printf("  %s: %s\n", i18n.G("Lower devices"), strings.Join(state.Bond.LowerDevices, ", "))
+	}
+
+	// Bridge information.
+	if state.Bridge != nil {
+		fmt.Println("")
+		fmt.Println(i18n.G("Bridge:"))
+		fmt.Printf("  %s: %s\n", i18n.G("ID"), state.Bridge.ID)
+		fmt.Printf("  %s: %v\n", i18n.G("STP"), state.Bridge.STP)
+		fmt.Printf("  %s: %d\n", i18n.G("Forward delay"), state.Bridge.ForwardDelay)
+		fmt.Printf("  %s: %d\n", i18n.G("Default VLAN ID"), state.Bridge.VLANDefault)
+		fmt.Printf("  %s: %v\n", i18n.G("VLAN filtering"), state.Bridge.VLANFiltering)
+		fmt.Printf("  %s: %s\n", i18n.G("Upper devices"), strings.Join(state.Bridge.UpperDevices, ", "))
+	}
+
+	// VLAN information.
+	if state.VLAN != nil {
+		fmt.Println("")
+		fmt.Println(i18n.G("VLAN:"))
+		fmt.Printf("  %s: %s\n", i18n.G("Lower device"), state.VLAN.LowerDevice)
+		fmt.Printf("  %s: %d\n", i18n.G("VLAN ID"), state.VLAN.VID)
+	}
+
+	// OVN information.
+	if state.OVN != nil {
+		fmt.Println("")
+		fmt.Println(i18n.G("OVN:"))
+		fmt.Printf("  %s: %s\n", i18n.G("Chassis"), state.OVN.Chassis)
+	}
+
+	return nil
+}
+
+// List.
 type cmdNetworkList struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -717,20 +881,20 @@ type cmdNetworkList struct {
 
 func (c *cmdNetworkList) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("list [<remote>:]")
+	cmd.Use = usage("list", i18n.G("[<remote>:]"))
 	cmd.Aliases = []string{"ls"}
 	cmd.Short = i18n.G("List available networks")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`List available networks`))
 
 	cmd.RunE = c.Run
-	cmd.Flags().StringVar(&c.flagFormat, "format", "table", i18n.G("Format (csv|json|table|yaml)")+"``")
+	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
 
 	return cmd
 }
 
 func (c *cmdNetworkList) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 1)
 	if exit {
 		return err
@@ -771,77 +935,51 @@ func (c *cmdNetworkList) Run(cmd *cobra.Command, args []string) error {
 		}
 
 		strUsedBy := fmt.Sprintf("%d", len(network.UsedBy))
-		details := []string{network.Name, network.Type, strManaged, network.Description, strUsedBy}
-		if resource.server.IsClustered() {
-			details = append(details, strings.ToUpper(network.Status))
+		details := []string{
+			network.Name,
+			network.Type,
+			strManaged,
+			network.Config["ipv4.address"],
+			network.Config["ipv6.address"],
+			network.Description,
+			strUsedBy,
+			strings.ToUpper(network.Status),
 		}
+
 		data = append(data, details)
 	}
 
+	sort.Sort(utils.ByName(data))
+
 	header := []string{
 		i18n.G("NAME"),
 		i18n.G("TYPE"),
 		i18n.G("MANAGED"),
+		i18n.G("IPV4"),
+		i18n.G("IPV6"),
 		i18n.G("DESCRIPTION"),
 		i18n.G("USED BY"),
-	}
-	if resource.server.IsClustered() {
-		header = append(header, i18n.G("STATE"))
+		i18n.G("STATE"),
 	}
 
-	switch c.flagFormat {
-	case listFormatTable:
-		table := tablewriter.NewWriter(os.Stdout)
-		table.SetAutoWrapText(false)
-		table.SetAlignment(tablewriter.ALIGN_LEFT)
-		table.SetRowLine(true)
-		table.SetHeader(header)
-		sort.Sort(byName(data))
-		table.AppendBulk(data)
-		table.Render()
-	case listFormatCSV:
-		sort.Sort(byName(data))
-		data = append(data, []string{})
-		copy(data[1:], data[0:])
-		data[0] = header
-		w := csv.NewWriter(os.Stdout)
-		w.WriteAll(data)
-		if err := w.Error(); err != nil {
-			return err
-		}
-	case listFormatJSON:
-		data := networks
-		enc := json.NewEncoder(os.Stdout)
-		err := enc.Encode(data)
-		if err != nil {
-			return err
-		}
-	case listFormatYAML:
-		data := networks
-		out, err := yaml.Marshal(data)
-		if err != nil {
-			return err
-		}
-		fmt.Printf("%s", out)
-	default:
-		return fmt.Errorf(i18n.G("Invalid format %q"), c.flagFormat)
-	}
-
-	return nil
+	return utils.RenderTable(c.flagFormat, header, data, networks)
 }
 
-// List leases
+// List leases.
 type cmdNetworkListLeases struct {
 	global  *cmdGlobal
 	network *cmdNetwork
+
+	flagFormat string
 }
 
 func (c *cmdNetworkListLeases) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("list-leases [<remote>:]<network>")
+	cmd.Use = usage("list-leases", i18n.G("[<remote>:]<network>"))
 	cmd.Short = i18n.G("List DHCP leases")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`List DHCP leases`))
+	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
 
 	cmd.RunE = c.Run
 
@@ -849,7 +987,7 @@ func (c *cmdNetworkListLeases) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkListLeases) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -875,26 +1013,31 @@ func (c *cmdNetworkListLeases) Run(cmd *cobra.Command, args []string) error {
 
 	data := [][]string{}
 	for _, lease := range leases {
-		data = append(data, []string{lease.Hostname, lease.Hwaddr, lease.Address, strings.ToUpper(lease.Type)})
+		entry := []string{lease.Hostname, lease.Hwaddr, lease.Address, strings.ToUpper(lease.Type)}
+		if resource.server.IsClustered() {
+			entry = append(entry, lease.Location)
+		}
+
+		data = append(data, entry)
 	}
 
-	table := tablewriter.NewWriter(os.Stdout)
-	table.SetAutoWrapText(false)
-	table.SetAlignment(tablewriter.ALIGN_LEFT)
-	table.SetRowLine(true)
-	table.SetHeader([]string{
+	sort.Sort(utils.ByName(data))
+
+	header := []string{
 		i18n.G("HOSTNAME"),
 		i18n.G("MAC ADDRESS"),
 		i18n.G("IP ADDRESS"),
-		i18n.G("TYPE")})
-	sort.Sort(byName(data))
-	table.AppendBulk(data)
-	table.Render()
+		i18n.G("TYPE"),
+	}
 
-	return nil
+	if resource.server.IsClustered() {
+		header = append(header, i18n.G("LOCATION"))
+	}
+
+	return utils.RenderTable(c.flagFormat, header, data, leases)
 }
 
-// Rename
+// Rename.
 type cmdNetworkRename struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -902,7 +1045,7 @@ type cmdNetworkRename struct {
 
 func (c *cmdNetworkRename) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("rename [<remote>:]<network> <new-name>")
+	cmd.Use = usage("rename", i18n.G("[<remote>:]<network> <new-name>"))
 	cmd.Aliases = []string{"mv"}
 	cmd.Short = i18n.G("Rename networks")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -914,7 +1057,7 @@ func (c *cmdNetworkRename) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkRename) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -938,11 +1081,14 @@ func (c *cmdNetworkRename) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf(i18n.G("Network %s renamed to %s")+"\n", resource.name, args[1])
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Network %s renamed to %s")+"\n", resource.name, args[1])
+	}
+
 	return nil
 }
 
-// Set
+// Set.
 type cmdNetworkSet struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -950,10 +1096,13 @@ type cmdNetworkSet struct {
 
 func (c *cmdNetworkSet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("set [<remote>:]<network> <key> <value>")
+	cmd.Use = usage("set", i18n.G("[<remote>:]<network> <key>=<value>..."))
 	cmd.Short = i18n.G("Set network configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Set network configuration keys`))
+		`Set network configuration keys
+
+For backward compatibility, a single configuration key may still be set with:
+    lxc network set [<remote>:]<network> <key> <value>`))
 
 	cmd.Flags().StringVar(&c.network.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
@@ -962,8 +1111,8 @@ func (c *cmdNetworkSet) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkSet) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
-	exit, err := c.global.CheckArgs(cmd, args, 3, 3)
+	// Quick checks.
+	exit, err := c.global.CheckArgs(cmd, args, 2, -1)
 	if exit {
 		return err
 	}
@@ -981,11 +1130,12 @@ func (c *cmdNetworkSet) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("Missing network name"))
 	}
 
-	// Set the config key
+	// Handle targeting
 	if c.network.flagTarget != "" {
 		client = client.UseTarget(c.network.flagTarget)
 	}
 
+	// Get the network
 	network, etag, err := client.GetNetwork(resource.name)
 	if err != nil {
 		return err
@@ -995,23 +1145,20 @@ func (c *cmdNetworkSet) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("Only managed networks can be modified"))
 	}
 
-	key := args[1]
-	value := args[2]
-
-	if !termios.IsTerminal(int(syscall.Stdin)) && value == "-" {
-		buf, err := ioutil.ReadAll(os.Stdin)
-		if err != nil {
-			return fmt.Errorf(i18n.G("Can't read from stdin: %s"), err)
-		}
-		value = string(buf[:])
+	// Set the keys
+	keys, err := getConfig(args[1:]...)
+	if err != nil {
+		return err
 	}
 
-	network.Config[key] = value
+	for k, v := range keys {
+		network.Config[k] = v
+	}
 
 	return client.UpdateNetwork(resource.name, network.Writable(), etag)
 }
 
-// Show
+// Show.
 type cmdNetworkShow struct {
 	global  *cmdGlobal
 	network *cmdNetwork
@@ -1019,7 +1166,7 @@ type cmdNetworkShow struct {
 
 func (c *cmdNetworkShow) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("show [<remote>:]<network>")
+	cmd.Use = usage("show", i18n.G("[<remote>:]<network>"))
 	cmd.Short = i18n.G("Show network configurations")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show network configurations`))
@@ -1031,7 +1178,7 @@ func (c *cmdNetworkShow) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkShow) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -1072,7 +1219,7 @@ func (c *cmdNetworkShow) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Unset
+// Unset.
 type cmdNetworkUnset struct {
 	global     *cmdGlobal
 	network    *cmdNetwork
@@ -1081,7 +1228,7 @@ type cmdNetworkUnset struct {
 
 func (c *cmdNetworkUnset) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("unset [<remote>:]<network> <key>")
+	cmd.Use = usage("unset", i18n.G("[<remote>:]<network> <key>"))
 	cmd.Short = i18n.G("Unset network configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Unset network configuration keys`))
@@ -1093,7 +1240,7 @@ func (c *cmdNetworkUnset) Command() *cobra.Command {
 }
 
 func (c *cmdNetworkUnset) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
