@@ -6,12 +6,11 @@ import (
 	"os"
 	"sort"
 	"strings"
-	"syscall"
 
-	"github.com/olekukonko/tablewriter"
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
+	"github.com/lxc/lxd/lxc/utils"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 	cli "github.com/lxc/lxd/shared/cmd"
@@ -25,7 +24,7 @@ type cmdProfile struct {
 
 func (c *cmdProfile) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("profile")
+	cmd.Use = usage("profile")
 	cmd.Short = i18n.G("Manage profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Manage profiles`))
@@ -86,10 +85,13 @@ func (c *cmdProfile) Command() *cobra.Command {
 	profileUnsetCmd := cmdProfileUnset{global: c.global, profile: c, profileSet: &profileSetCmd}
 	cmd.AddCommand(profileUnsetCmd.Command())
 
+	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
+	cmd.Args = cobra.NoArgs
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
-// Add
+// Add.
 type cmdProfileAdd struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -97,10 +99,10 @@ type cmdProfileAdd struct {
 
 func (c *cmdProfileAdd) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("add [<remote>:]<container> <profile>")
-	cmd.Short = i18n.G("Add profiles to containers")
+	cmd.Use = usage("add", i18n.G("[<remote>:]<instance> <profile>"))
+	cmd.Short = i18n.G("Add profiles to instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Add profiles to containers`))
+		`Add profiles to instances`))
 
 	cmd.RunE = c.Run
 
@@ -108,7 +110,7 @@ func (c *cmdProfileAdd) Command() *cobra.Command {
 }
 
 func (c *cmdProfileAdd) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -123,18 +125,18 @@ func (c *cmdProfileAdd) Run(cmd *cobra.Command, args []string) error {
 	resource := resources[0]
 
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing container.name name"))
+		return fmt.Errorf(i18n.G("Missing instance name"))
 	}
 
 	// Add the profile
-	container, etag, err := resource.server.GetContainer(resource.name)
+	inst, etag, err := resource.server.GetInstance(resource.name)
 	if err != nil {
 		return err
 	}
 
-	container.Profiles = append(container.Profiles, args[1])
+	inst.Profiles = append(inst.Profiles, args[1])
 
-	op, err := resource.server.UpdateContainer(resource.name, container.Writable(), etag)
+	op, err := resource.server.UpdateInstance(resource.name, inst.Writable(), etag)
 	if err != nil {
 		return err
 	}
@@ -144,12 +146,14 @@ func (c *cmdProfileAdd) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf(i18n.G("Profile %s added to %s")+"\n", args[1], resource.name)
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Profile %s added to %s")+"\n", args[1], resource.name)
+	}
 
 	return nil
 }
 
-// Assign
+// Assign.
 type cmdProfileAssign struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -157,11 +161,11 @@ type cmdProfileAssign struct {
 
 func (c *cmdProfileAssign) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("assign [<remote>:]<container> <profiles>")
+	cmd.Use = usage("assign", i18n.G("[<remote>:]<instance> <profiles>"))
 	cmd.Aliases = []string{"apply"}
-	cmd.Short = i18n.G("Assign sets of profiles to containers")
+	cmd.Short = i18n.G("Assign sets of profiles to instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Assign sets of profiles to containers`))
+		`Assign sets of profiles to instances`))
 	cmd.Example = cli.FormatSection("", i18n.G(
 		`lxc profile assign foo default,bar
     Set the profiles for "foo" to "default" and "bar".
@@ -178,7 +182,7 @@ lxc profile assign foo ''
 }
 
 func (c *cmdProfileAssign) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -194,21 +198,21 @@ func (c *cmdProfileAssign) Run(cmd *cobra.Command, args []string) error {
 
 	// Assign the profiles
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing container name"))
+		return fmt.Errorf(i18n.G("Missing instance name"))
 	}
 
-	container, etag, err := resource.server.GetContainer(resource.name)
+	inst, etag, err := resource.server.GetInstance(resource.name)
 	if err != nil {
 		return err
 	}
 
 	if args[1] != "" {
-		container.Profiles = strings.Split(args[1], ",")
+		inst.Profiles = strings.Split(args[1], ",")
 	} else {
-		container.Profiles = nil
+		inst.Profiles = nil
 	}
 
-	op, err := resource.server.UpdateContainer(resource.name, container.Writable(), etag)
+	op, err := resource.server.UpdateInstance(resource.name, inst.Writable(), etag)
 	if err != nil {
 		return err
 	}
@@ -222,24 +226,29 @@ func (c *cmdProfileAssign) Run(cmd *cobra.Command, args []string) error {
 		args[1] = i18n.G("(none)")
 	}
 
-	fmt.Printf(i18n.G("Profiles %s applied to %s")+"\n", args[1], resource.name)
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Profiles %s applied to %s")+"\n", args[1], resource.name)
+	}
 
 	return nil
 }
 
-// Copy
+// Copy.
 type cmdProfileCopy struct {
 	global  *cmdGlobal
 	profile *cmdProfile
+
+	flagTargetProject string
 }
 
 func (c *cmdProfileCopy) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("copy [<remote>:]<profile> [<remote>:]<profile>")
+	cmd.Use = usage("copy", i18n.G("[<remote>:]<profile> [<remote>:]<profile>"))
 	cmd.Aliases = []string{"cp"}
 	cmd.Short = i18n.G("Copy profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Copy profiles`))
+	cmd.Flags().StringVar(&c.flagTargetProject, "target-project", "", i18n.G("Copy to a project different from the source")+"``")
 
 	cmd.RunE = c.Run
 
@@ -247,7 +256,7 @@ func (c *cmdProfileCopy) Command() *cobra.Command {
 }
 
 func (c *cmdProfileCopy) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -281,10 +290,14 @@ func (c *cmdProfileCopy) Run(cmd *cobra.Command, args []string) error {
 		Name:       dest.name,
 	}
 
+	if c.flagTargetProject != "" {
+		dest.server = dest.server.UseProject(c.flagTargetProject)
+	}
+
 	return dest.server.CreateProfile(newProfile)
 }
 
-// Create
+// Create.
 type cmdProfileCreate struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -292,7 +305,7 @@ type cmdProfileCreate struct {
 
 func (c *cmdProfileCreate) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("create [<remote>:]<profile>")
+	cmd.Use = usage("create", i18n.G("[<remote>:]<profile>"))
 	cmd.Short = i18n.G("Create profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Create profiles`))
@@ -303,7 +316,7 @@ func (c *cmdProfileCreate) Command() *cobra.Command {
 }
 
 func (c *cmdProfileCreate) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -330,11 +343,14 @@ func (c *cmdProfileCreate) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf(i18n.G("Profile %s created")+"\n", resource.name)
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Profile %s created")+"\n", resource.name)
+	}
+
 	return nil
 }
 
-// Delete
+// Delete.
 type cmdProfileDelete struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -342,7 +358,7 @@ type cmdProfileDelete struct {
 
 func (c *cmdProfileDelete) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("delete [<remote>:]<profile>")
+	cmd.Use = usage("delete", i18n.G("[<remote>:]<profile>"))
 	cmd.Aliases = []string{"rm"}
 	cmd.Short = i18n.G("Delete profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -354,7 +370,7 @@ func (c *cmdProfileDelete) Command() *cobra.Command {
 }
 
 func (c *cmdProfileDelete) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -378,12 +394,14 @@ func (c *cmdProfileDelete) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf(i18n.G("Profile %s deleted")+"\n", resource.name)
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Profile %s deleted")+"\n", resource.name)
+	}
 
 	return nil
 }
 
-// Edit
+// Edit.
 type cmdProfileEdit struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -391,7 +409,7 @@ type cmdProfileEdit struct {
 
 func (c *cmdProfileEdit) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("edit [<remote>:]<profile>")
+	cmd.Use = usage("edit", i18n.G("[<remote>:]<profile>"))
 	cmd.Short = i18n.G("Edit profile configurations as YAML")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Edit profile configurations as YAML`))
@@ -406,7 +424,7 @@ func (c *cmdProfileEdit) Command() *cobra.Command {
 
 func (c *cmdProfileEdit) helpTemplate() string {
 	return i18n.G(
-		`### This is a yaml representation of the profile.
+		`### This is a YAML representation of the profile.
 ### Any line starting with a '# will be ignored.
 ###
 ### A profile consists of a set of configuration items followed by a set of
@@ -426,7 +444,7 @@ func (c *cmdProfileEdit) helpTemplate() string {
 }
 
 func (c *cmdProfileEdit) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -445,7 +463,7 @@ func (c *cmdProfileEdit) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// If stdin isn't a terminal, read text from it
-	if !termios.IsTerminal(int(syscall.Stdin)) {
+	if !termios.IsTerminal(getStdinFd()) {
 		contents, err := ioutil.ReadAll(os.Stdin)
 		if err != nil {
 			return err
@@ -488,7 +506,7 @@ func (c *cmdProfileEdit) Run(cmd *cobra.Command, args []string) error {
 		// Respawn the editor
 		if err != nil {
 			fmt.Fprintf(os.Stderr, i18n.G("Config parsing error: %s")+"\n", err)
-			fmt.Println(i18n.G("Press enter to open the editor again"))
+			fmt.Println(i18n.G("Press enter to open the editor again or ctrl+c to abort change"))
 
 			_, err := os.Stdin.Read(make([]byte, 1))
 			if err != nil {
@@ -499,15 +517,17 @@ func (c *cmdProfileEdit) Run(cmd *cobra.Command, args []string) error {
 			if err != nil {
 				return err
 			}
+
 			continue
 		}
+
 		break
 	}
 
 	return nil
 }
 
-// Get
+// Get.
 type cmdProfileGet struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -515,7 +535,7 @@ type cmdProfileGet struct {
 
 func (c *cmdProfileGet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("get [<remote>:]<profile> <key>")
+	cmd.Use = usage("get", i18n.G("[<remote>:]<profile> <key>"))
 	cmd.Short = i18n.G("Get values for profile configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Get values for profile configuration keys`))
@@ -526,7 +546,7 @@ func (c *cmdProfileGet) Command() *cobra.Command {
 }
 
 func (c *cmdProfileGet) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -554,27 +574,29 @@ func (c *cmdProfileGet) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// List
+// List.
 type cmdProfileList struct {
-	global  *cmdGlobal
-	profile *cmdProfile
+	global     *cmdGlobal
+	profile    *cmdProfile
+	flagFormat string
 }
 
 func (c *cmdProfileList) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("list [<remote>:]")
+	cmd.Use = usage("list", i18n.G("[<remote>:]"))
 	cmd.Aliases = []string{"ls"}
 	cmd.Short = i18n.G("List profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`List profiles`))
 
 	cmd.RunE = c.Run
+	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
 
 	return cmd
 }
 
 func (c *cmdProfileList) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 1)
 	if exit {
 		return err
@@ -602,24 +624,20 @@ func (c *cmdProfileList) Run(cmd *cobra.Command, args []string) error {
 	data := [][]string{}
 	for _, profile := range profiles {
 		strUsedBy := fmt.Sprintf("%d", len(profile.UsedBy))
-		data = append(data, []string{profile.Name, strUsedBy})
+		data = append(data, []string{profile.Name, profile.Description, strUsedBy})
 	}
 
-	table := tablewriter.NewWriter(os.Stdout)
-	table.SetAutoWrapText(false)
-	table.SetAlignment(tablewriter.ALIGN_LEFT)
-	table.SetRowLine(true)
-	table.SetHeader([]string{
-		i18n.G("NAME"),
-		i18n.G("USED BY")})
-	sort.Sort(byName(data))
-	table.AppendBulk(data)
-	table.Render()
+	sort.Sort(utils.ByName(data))
 
-	return nil
+	header := []string{
+		i18n.G("NAME"),
+		i18n.G("DESCRIPTION"),
+		i18n.G("USED BY")}
+
+	return utils.RenderTable(c.flagFormat, header, data, profiles)
 }
 
-// Remove
+// Remove.
 type cmdProfileRemove struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -627,10 +645,10 @@ type cmdProfileRemove struct {
 
 func (c *cmdProfileRemove) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("remove [<remote>:]<container> <profile>")
-	cmd.Short = i18n.G("Remove profiles from containers")
+	cmd.Use = usage("remove", i18n.G("[<remote>:]<instance> <profile>"))
+	cmd.Short = i18n.G("Remove profiles from instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Remove profiles from containers`))
+		`Remove profiles from instances`))
 
 	cmd.RunE = c.Run
 
@@ -638,7 +656,7 @@ func (c *cmdProfileRemove) Command() *cobra.Command {
 }
 
 func (c *cmdProfileRemove) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -653,21 +671,21 @@ func (c *cmdProfileRemove) Run(cmd *cobra.Command, args []string) error {
 	resource := resources[0]
 
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing container name"))
+		return fmt.Errorf(i18n.G("Missing instance name"))
 	}
 
 	// Remove the profile
-	container, etag, err := resource.server.GetContainer(resource.name)
+	inst, etag, err := resource.server.GetInstance(resource.name)
 	if err != nil {
 		return err
 	}
 
-	if !shared.StringInSlice(args[1], container.Profiles) {
+	if !shared.StringInSlice(args[1], inst.Profiles) {
 		return fmt.Errorf(i18n.G("Profile %s isn't currently applied to %s"), args[1], resource.name)
 	}
 
 	profiles := []string{}
-	for _, profile := range container.Profiles {
+	for _, profile := range inst.Profiles {
 		if profile == args[1] {
 			continue
 		}
@@ -675,9 +693,9 @@ func (c *cmdProfileRemove) Run(cmd *cobra.Command, args []string) error {
 		profiles = append(profiles, profile)
 	}
 
-	container.Profiles = profiles
+	inst.Profiles = profiles
 
-	op, err := resource.server.UpdateContainer(resource.name, container.Writable(), etag)
+	op, err := resource.server.UpdateInstance(resource.name, inst.Writable(), etag)
 	if err != nil {
 		return err
 	}
@@ -687,11 +705,14 @@ func (c *cmdProfileRemove) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf(i18n.G("Profile %s removed from %s")+"\n", args[1], resource.name)
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Profile %s removed from %s")+"\n", args[1], resource.name)
+	}
+
 	return nil
 }
 
-// Rename
+// Rename.
 type cmdProfileRename struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -699,7 +720,7 @@ type cmdProfileRename struct {
 
 func (c *cmdProfileRename) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("rename [<remote>:]<profile> <new-name>")
+	cmd.Use = usage("rename", i18n.G("[<remote>:]<profile> <new-name>"))
 	cmd.Aliases = []string{"mv"}
 	cmd.Short = i18n.G("Rename profiles")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -711,7 +732,7 @@ func (c *cmdProfileRename) Command() *cobra.Command {
 }
 
 func (c *cmdProfileRename) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -735,12 +756,14 @@ func (c *cmdProfileRename) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf(i18n.G("Profile %s renamed to %s")+"\n", resource.name, args[1])
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Profile %s renamed to %s")+"\n", resource.name, args[1])
+	}
 
 	return nil
 }
 
-// Set
+// Set.
 type cmdProfileSet struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -748,10 +771,13 @@ type cmdProfileSet struct {
 
 func (c *cmdProfileSet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("set [<remote>:]<profile> <key> <value>")
+	cmd.Use = usage("set", i18n.G("[<remote>:]<profile> <key><value>..."))
 	cmd.Short = i18n.G("Set profile configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Set profile configuration keys`))
+		`Set profile configuration keys
+
+For backward compatibility, a single configuration key may still be set with:
+    lxc profile set [<remote>:]<profile> <key> <value>`))
 
 	cmd.RunE = c.Run
 
@@ -759,8 +785,8 @@ func (c *cmdProfileSet) Command() *cobra.Command {
 }
 
 func (c *cmdProfileSet) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
-	exit, err := c.global.CheckArgs(cmd, args, 3, 3)
+	// Quick checks.
+	exit, err := c.global.CheckArgs(cmd, args, 2, -1)
 	if exit {
 		return err
 	}
@@ -777,29 +803,26 @@ func (c *cmdProfileSet) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("Missing profile name"))
 	}
 
-	// Set the configuration key
-	key := args[1]
-	value := args[2]
-
-	if !termios.IsTerminal(int(syscall.Stdin)) && value == "-" {
-		buf, err := ioutil.ReadAll(os.Stdin)
-		if err != nil {
-			return fmt.Errorf(i18n.G("Can't read from stdin: %s"), err)
-		}
-		value = string(buf[:])
-	}
-
+	// Get the profile
 	profile, etag, err := resource.server.GetProfile(resource.name)
 	if err != nil {
 		return err
 	}
 
-	profile.Config[key] = value
+	// Set the configuration key
+	keys, err := getConfig(args[1:]...)
+	if err != nil {
+		return err
+	}
+
+	for k, v := range keys {
+		profile.Config[k] = v
+	}
 
 	return resource.server.UpdateProfile(resource.name, profile.Writable(), etag)
 }
 
-// Show
+// Show.
 type cmdProfileShow struct {
 	global  *cmdGlobal
 	profile *cmdProfile
@@ -807,7 +830,7 @@ type cmdProfileShow struct {
 
 func (c *cmdProfileShow) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("show [<remote>:]<profile>")
+	cmd.Use = usage("show", i18n.G("[<remote>:]<profile>"))
 	cmd.Short = i18n.G("Show profile configurations")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show profile configurations`))
@@ -818,7 +841,7 @@ func (c *cmdProfileShow) Command() *cobra.Command {
 }
 
 func (c *cmdProfileShow) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -852,7 +875,7 @@ func (c *cmdProfileShow) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Unset
+// Unset.
 type cmdProfileUnset struct {
 	global     *cmdGlobal
 	profile    *cmdProfile
@@ -861,7 +884,7 @@ type cmdProfileUnset struct {
 
 func (c *cmdProfileUnset) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("unset [<remote>:]<profile> <key>")
+	cmd.Use = usage("unset", i18n.G("[<remote>:]<profile> <key>"))
 	cmd.Short = i18n.G("Unset profile configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Unset profile configuration keys`))
@@ -872,7 +895,7 @@ func (c *cmdProfileUnset) Command() *cobra.Command {
 }
 
 func (c *cmdProfileUnset) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
