@@ -3,21 +3,22 @@ package main
 import (
 	"fmt"
 	"io/ioutil"
+	"net/url"
 	"os"
 	"sort"
 	"strconv"
 	"strings"
-	"syscall"
 
-	"github.com/olekukonko/tablewriter"
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
+	"github.com/lxc/lxd/lxc/utils"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 	cli "github.com/lxc/lxd/shared/cmd"
 	"github.com/lxc/lxd/shared/i18n"
 	"github.com/lxc/lxd/shared/termios"
+	"github.com/lxc/lxd/shared/units"
 )
 
 type cmdStorage struct {
@@ -28,7 +29,7 @@ type cmdStorage struct {
 
 func (c *cmdStorage) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("storage")
+	cmd.Use = usage("storage")
 	cmd.Short = i18n.G("Manage storage pools and volumes")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Manage storage pools and volumes`))
@@ -73,10 +74,13 @@ func (c *cmdStorage) Command() *cobra.Command {
 	storageVolumeCmd := cmdStorageVolume{global: c.global, storage: c}
 	cmd.AddCommand(storageVolumeCmd.Command())
 
+	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
+	cmd.Args = cobra.NoArgs
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
-// Create
+// Create.
 type cmdStorageCreate struct {
 	global  *cmdGlobal
 	storage *cmdStorage
@@ -84,7 +88,7 @@ type cmdStorageCreate struct {
 
 func (c *cmdStorageCreate) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("create [<remote>:]<pool> <driver> [key=value...]")
+	cmd.Use = usage("create", i18n.G("[<remote>:]<pool> <driver> [key=value...]"))
 	cmd.Short = i18n.G("Create storage pools")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Create storage pools`))
@@ -96,7 +100,7 @@ func (c *cmdStorageCreate) Command() *cobra.Command {
 }
 
 func (c *cmdStorageCreate) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, -1)
 	if exit {
 		return err
@@ -138,16 +142,18 @@ func (c *cmdStorageCreate) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	if c.storage.flagTarget != "" {
-		fmt.Printf(i18n.G("Storage pool %s pending on member %s")+"\n", resource.name, c.storage.flagTarget)
-	} else {
-		fmt.Printf(i18n.G("Storage pool %s created")+"\n", resource.name)
+	if !c.global.flagQuiet {
+		if c.storage.flagTarget != "" {
+			fmt.Printf(i18n.G("Storage pool %s pending on member %s")+"\n", resource.name, c.storage.flagTarget)
+		} else {
+			fmt.Printf(i18n.G("Storage pool %s created")+"\n", resource.name)
+		}
 	}
 
 	return nil
 }
 
-// Delete
+// Delete.
 type cmdStorageDelete struct {
 	global  *cmdGlobal
 	storage *cmdStorage
@@ -155,7 +161,7 @@ type cmdStorageDelete struct {
 
 func (c *cmdStorageDelete) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("delete [<remote>:]<pool>")
+	cmd.Use = usage("delete", i18n.G("[<remote>:]<pool>"))
 	cmd.Aliases = []string{"rm"}
 	cmd.Short = i18n.G("Delete storage pools")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -167,7 +173,7 @@ func (c *cmdStorageDelete) Command() *cobra.Command {
 }
 
 func (c *cmdStorageDelete) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -191,11 +197,14 @@ func (c *cmdStorageDelete) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf(i18n.G("Storage pool %s deleted")+"\n", resource.name)
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Storage pool %s deleted")+"\n", resource.name)
+	}
+
 	return nil
 }
 
-// Edit
+// Edit.
 type cmdStorageEdit struct {
 	global  *cmdGlobal
 	storage *cmdStorage
@@ -203,7 +212,7 @@ type cmdStorageEdit struct {
 
 func (c *cmdStorageEdit) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("edit [<remote>:]<pool>")
+	cmd.Use = usage("edit", i18n.G("[<remote>:]<pool>"))
 	cmd.Short = i18n.G("Edit storage pool configurations as YAML")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Edit storage pool configurations as YAML`))
@@ -218,7 +227,7 @@ func (c *cmdStorageEdit) Command() *cobra.Command {
 
 func (c *cmdStorageEdit) helpTemplate() string {
 	return i18n.G(
-		`### This is a yaml representation of a storage pool.
+		`### This is a YAML representation of a storage pool.
 ### Any line starting with a '#' will be ignored.
 ###
 ### A storage pool consists of a set of configuration items.
@@ -234,7 +243,7 @@ func (c *cmdStorageEdit) helpTemplate() string {
 }
 
 func (c *cmdStorageEdit) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -253,7 +262,7 @@ func (c *cmdStorageEdit) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	// If stdin isn't a terminal, read text from it
-	if !termios.IsTerminal(int(syscall.Stdin)) {
+	if !termios.IsTerminal(getStdinFd()) {
 		contents, err := ioutil.ReadAll(os.Stdin)
 		if err != nil {
 			return err
@@ -296,7 +305,7 @@ func (c *cmdStorageEdit) Run(cmd *cobra.Command, args []string) error {
 		// Respawn the editor
 		if err != nil {
 			fmt.Fprintf(os.Stderr, i18n.G("Config parsing error: %s")+"\n", err)
-			fmt.Println(i18n.G("Press enter to open the editor again"))
+			fmt.Println(i18n.G("Press enter to open the editor again or ctrl+c to abort change"))
 
 			_, err := os.Stdin.Read(make([]byte, 1))
 			if err != nil {
@@ -307,15 +316,17 @@ func (c *cmdStorageEdit) Run(cmd *cobra.Command, args []string) error {
 			if err != nil {
 				return err
 			}
+
 			continue
 		}
+
 		break
 	}
 
 	return nil
 }
 
-// Get
+// Get.
 type cmdStorageGet struct {
 	global  *cmdGlobal
 	storage *cmdStorage
@@ -323,7 +334,7 @@ type cmdStorageGet struct {
 
 func (c *cmdStorageGet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("get [<remote>:]<pool> <key>")
+	cmd.Use = usage("get", i18n.G("[<remote>:]<pool> <key>"))
 	cmd.Short = i18n.G("Get values for storage pool configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Get values for storage pool configuration keys`))
@@ -335,7 +346,7 @@ func (c *cmdStorageGet) Command() *cobra.Command {
 }
 
 func (c *cmdStorageGet) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -353,6 +364,11 @@ func (c *cmdStorageGet) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("Missing pool name"))
 	}
 
+	// If a target member was specified, we return also member-specific config values.
+	if c.storage.flagTarget != "" {
+		resource.server = resource.server.UseTarget(c.storage.flagTarget)
+	}
+
 	// Get the property
 	resp, _, err := resource.server.GetStoragePool(resource.name)
 	if err != nil {
@@ -368,7 +384,7 @@ func (c *cmdStorageGet) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Info
+// Info.
 type cmdStorageInfo struct {
 	global  *cmdGlobal
 	storage *cmdStorage
@@ -378,19 +394,20 @@ type cmdStorageInfo struct {
 
 func (c *cmdStorageInfo) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("info [<remote>:]<pool>")
+	cmd.Use = usage("info", i18n.G("[<remote>:]<pool>"))
 	cmd.Short = i18n.G("Show useful information about storage pools")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show useful information about storage pools`))
 
 	cmd.Flags().BoolVar(&c.flagBytes, "bytes", false, i18n.G("Show the used and free space in bytes"))
+	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
 
 	return cmd
 }
 
 func (c *cmdStorageInfo) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -408,6 +425,15 @@ func (c *cmdStorageInfo) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("Missing pool name"))
 	}
 
+	// Targeting
+	if c.storage.flagTarget != "" {
+		if !resource.server.IsClustered() {
+			return fmt.Errorf(i18n.G("To use --target, the destination remote must be a cluster"))
+		}
+
+		resource.server = resource.server.UseTarget(c.storage.flagTarget)
+	}
+
 	// Get the pool information
 	pool, _, err := resource.server.GetStoragePool(resource.name)
 	if err != nil {
@@ -436,12 +462,48 @@ func (c *cmdStorageInfo) Run(cmd *cobra.Command, args []string) error {
 	poolusedby[usedbystring] = map[string][]string{}
 
 	/* Build up the usedby map
-	/1.0/{containers,images,profiles}/storagepoolname
+	/1.0/{instances,images,profiles}/storagepoolname and
+	/1.0/storage-pools/<poolname>/<type>/<volname>
 	remove the /1.0/ and build the map based on the resources name as key
 	and resources details as value */
 	for _, v := range pool.UsedBy {
-		bytype := string(strings.Split(v[5:], "/")[0])
-		bywhat := string(strings.Split(v[5:], "/")[1])
+		u, err := url.Parse(v)
+		if err != nil {
+			continue
+		}
+
+		fields := strings.Split(u.Path[5:], "/")
+		bytype := fields[0]
+		bywhat := fields[1]
+
+		if bytype == "storage-pools" {
+			bytype = "volumes"
+			bywhat = fields[4]
+		}
+
+		var info string
+
+		// Show info regarding the project and target if present.
+		values := u.Query()
+
+		proj := values.Get("project")
+		target := values.Get("target")
+
+		if proj != "" {
+			info = fmt.Sprintf("project %q", proj)
+		}
+
+		if target != "" {
+			if info == "" {
+				info = fmt.Sprintf("target %q", target)
+			} else {
+				info = fmt.Sprintf("%s, target %q", info, target)
+			}
+		}
+
+		if info != "" {
+			bywhat = fmt.Sprintf("%s (%s)", bywhat, info)
+		}
 
 		poolusedby[usedbystring][bytype] = append(poolusedby[usedbystring][bytype], bywhat)
 	}
@@ -457,8 +519,8 @@ func (c *cmdStorageInfo) Run(cmd *cobra.Command, args []string) error {
 		poolinfo[infostring][totalspacestring] = strconv.FormatUint(res.Space.Total, 10)
 		poolinfo[infostring][spaceusedstring] = strconv.FormatUint(res.Space.Used, 10)
 	} else {
-		poolinfo[infostring][totalspacestring] = shared.GetByteSizeString(int64(res.Space.Total), 2)
-		poolinfo[infostring][spaceusedstring] = shared.GetByteSizeString(int64(res.Space.Used), 2)
+		poolinfo[infostring][totalspacestring] = units.GetByteSizeStringIEC(int64(res.Space.Total), 2)
+		poolinfo[infostring][spaceusedstring] = units.GetByteSizeStringIEC(int64(res.Space.Used), 2)
 	}
 
 	poolinfodata, err := yaml.Marshal(poolinfo)
@@ -477,19 +539,22 @@ func (c *cmdStorageInfo) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// List
+// List.
 type cmdStorageList struct {
 	global  *cmdGlobal
 	storage *cmdStorage
+
+	flagFormat string
 }
 
 func (c *cmdStorageList) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("list [<remote>:]")
+	cmd.Use = usage("list", i18n.G("[<remote>:]"))
 	cmd.Aliases = []string{"ls"}
 	cmd.Short = i18n.G("List available storage pools")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`List available storage pools`))
+	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
 
 	cmd.RunE = c.Run
 
@@ -497,7 +562,7 @@ func (c *cmdStorageList) Command() *cobra.Command {
 }
 
 func (c *cmdStorageList) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 1)
 	if exit {
 		return err
@@ -525,42 +590,36 @@ func (c *cmdStorageList) Run(cmd *cobra.Command, args []string) error {
 	data := [][]string{}
 	for _, pool := range pools {
 		usedby := strconv.Itoa(len(pool.UsedBy))
-		details := []string{pool.Name, pool.Description, pool.Driver}
-		if resource.server.IsClustered() {
-			details = append(details, strings.ToUpper(pool.Status))
-		} else {
+		details := []string{pool.Name, pool.Driver}
+		if !resource.server.IsClustered() {
 			details = append(details, pool.Config["source"])
 		}
+
+		details = append(details, pool.Description)
 		details = append(details, usedby)
+		details = append(details, strings.ToUpper(pool.Status))
 		data = append(data, details)
 	}
 
-	table := tablewriter.NewWriter(os.Stdout)
-	table.SetAutoWrapText(false)
-	table.SetAlignment(tablewriter.ALIGN_LEFT)
-	table.SetRowLine(true)
+	sort.Sort(utils.ByName(data))
 
 	header := []string{
 		i18n.G("NAME"),
-		i18n.G("DESCRIPTION"),
 		i18n.G("DRIVER"),
 	}
-	if resource.server.IsClustered() {
-		header = append(header, i18n.G("STATE"))
-	} else {
+
+	if !resource.server.IsClustered() {
 		header = append(header, i18n.G("SOURCE"))
 	}
+
+	header = append(header, i18n.G("DESCRIPTION"))
 	header = append(header, i18n.G("USED BY"))
-	table.SetHeader(header)
+	header = append(header, i18n.G("STATE"))
 
-	sort.Sort(byName(data))
-	table.AppendBulk(data)
-	table.Render()
-
-	return nil
+	return utils.RenderTable(c.flagFormat, header, data, pools)
 }
 
-// Set
+// Set.
 type cmdStorageSet struct {
 	global  *cmdGlobal
 	storage *cmdStorage
@@ -568,10 +627,13 @@ type cmdStorageSet struct {
 
 func (c *cmdStorageSet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("set [<remote>:]<pool> <key> <value>")
+	cmd.Use = usage("set", i18n.G("[<remote>:]<pool> <key> <value>"))
 	cmd.Short = i18n.G("Set storage pool configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Set storage pool configuration keys`))
+		`Set storage pool configuration keys
+
+For backward compatibility, a single configuration key may still be set with:
+    lxc storage set [<remote>:]<pool> <key> <value>`))
 
 	cmd.Flags().StringVar(&c.storage.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
@@ -580,17 +642,14 @@ func (c *cmdStorageSet) Command() *cobra.Command {
 }
 
 func (c *cmdStorageSet) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
-	exit, err := c.global.CheckArgs(cmd, args, 3, 3)
+	// Quick checks.
+	exit, err := c.global.CheckArgs(cmd, args, 2, -1)
 	if exit {
 		return err
 	}
 
 	// Parse remote
-	remote := ""
-	if len(args) > 0 {
-		remote = args[0]
-	}
+	remote := args[0]
 
 	resources, err := c.global.ParseServers(remote)
 	if err != nil {
@@ -598,31 +657,33 @@ func (c *cmdStorageSet) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	resource := resources[0]
-
 	if resource.name == "" {
 		return fmt.Errorf(i18n.G("Missing pool name"))
 	}
 
+	client := resource.server
+	if c.storage.flagTarget != "" {
+		client = client.UseTarget(c.storage.flagTarget)
+	}
+
 	// Get the pool entry
-	pool, etag, err := resource.server.GetStoragePool(resource.name)
+	pool, etag, err := client.GetStoragePool(resource.name)
 	if err != nil {
 		return err
 	}
 
-	// Read the value
-	value := args[2]
-	if !termios.IsTerminal(int(syscall.Stdin)) && value == "-" {
-		buf, err := ioutil.ReadAll(os.Stdin)
-		if err != nil {
-			return fmt.Errorf(i18n.G("Can't read from stdin: %s"), err)
-		}
-		value = string(buf[:])
+	// Parse key/values
+	keys, err := getConfig(args[1:]...)
+	if err != nil {
+		return err
 	}
 
 	// Update the pool
-	pool.Config[args[1]] = value
+	for k, v := range keys {
+		pool.Config[k] = v
+	}
 
-	err = resource.server.UpdateStoragePool(resource.name, pool.Writable(), etag)
+	err = client.UpdateStoragePool(resource.name, pool.Writable(), etag)
 	if err != nil {
 		return err
 	}
@@ -630,7 +691,7 @@ func (c *cmdStorageSet) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Show
+// Show.
 type cmdStorageShow struct {
 	global  *cmdGlobal
 	storage *cmdStorage
@@ -640,7 +701,7 @@ type cmdStorageShow struct {
 
 func (c *cmdStorageShow) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("show [<remote>:]<pool>")
+	cmd.Use = usage("show", i18n.G("[<remote>:]<pool>"))
 	cmd.Short = i18n.G("Show storage pool configurations and resources")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show storage pool configurations and resources`))
@@ -653,7 +714,7 @@ func (c *cmdStorageShow) Command() *cobra.Command {
 }
 
 func (c *cmdStorageShow) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -715,7 +776,7 @@ func (c *cmdStorageShow) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Unset
+// Unset.
 type cmdStorageUnset struct {
 	global     *cmdGlobal
 	storage    *cmdStorage
@@ -724,7 +785,7 @@ type cmdStorageUnset struct {
 
 func (c *cmdStorageUnset) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("unset [<remote>:]<pool> <key>")
+	cmd.Use = usage("unset", i18n.G("[<remote>:]<pool> <key>"))
 	cmd.Short = i18n.G("Unset storage pool configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Unset storage pool configuration keys`))
@@ -736,7 +797,7 @@ func (c *cmdStorageUnset) Command() *cobra.Command {
 }
 
 func (c *cmdStorageUnset) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
