@@ -1,21 +1,25 @@
 package cluster_test
 
 import (
+	"context"
+	"crypto/x509"
 	"net/http"
 	"net/http/httptest"
 	"testing"
 	"time"
 
-	"github.com/CanonicalLtd/go-dqlite"
-	"github.com/hashicorp/raft"
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/go-dqlite/driver"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-	"golang.org/x/net/context"
+
+	"github.com/lxc/lxd/lxd/cluster"
+	clusterConfig "github.com/lxc/lxd/lxd/cluster/config"
+	"github.com/lxc/lxd/lxd/db"
+	clusterDB "github.com/lxc/lxd/lxd/db/cluster"
+	"github.com/lxc/lxd/lxd/state"
+	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/osarch"
+	"github.com/lxc/lxd/shared/version"
 )
 
 // After a heartbeat request is completed, the leader updates the heartbeat
@@ -28,85 +32,42 @@ func TestHeartbeat(t *testing.T) {
 	f.Grow()
 	f.Grow()
 
+	time.Sleep(1 * time.Second) // Wait for join notifiation triggered heartbeats to complete.
+
 	leader := f.Leader()
 	leaderState := f.State(leader)
 
 	// Artificially mark all nodes as down
-	err := leaderState.Cluster.Transaction(func(tx *db.ClusterTx) error {
-		nodes, err := tx.Nodes()
+	err := leaderState.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		nodes, err := tx.GetNodes()
 		require.NoError(t, err)
 		for _, node := range nodes {
-			err := tx.NodeHeartbeat(node.Address, time.Now().Add(-time.Minute))
+			err := tx.SetNodeHeartbeat(node.Address, time.Now().Add(-time.Minute))
 			require.NoError(t, err)
 		}
+
 		return nil
 	})
 	require.NoError(t, err)
 
 	// Perform the heartbeat requests.
-	heartbeat, _ := cluster.Heartbeat(leader, leaderState.Cluster)
+	leader.Cluster = leaderState.DB.Cluster
+	heartbeat, _ := cluster.HeartbeatTask(leader)
 	ctx := context.Background()
 	heartbeat(ctx)
 
 	// The heartbeat timestamps of all nodes got updated
-	err = leaderState.Cluster.Transaction(func(tx *db.ClusterTx) error {
-		nodes, err := tx.Nodes()
+	err = leaderState.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		nodes, err := tx.GetNodes()
 		require.NoError(t, err)
 
-		offlineThreshold, err := tx.NodeOfflineThreshold()
+		offlineThreshold, err := tx.GetNodeOfflineThreshold()
 		require.NoError(t, err)
 
 		for _, node := range nodes {
 			assert.False(t, node.IsOffline(offlineThreshold))
 		}
-		return nil
-	})
-	require.NoError(t, err)
-}
 
-// If a certain node does not successfully respond to the heartbeat, its
-// timestamp does not get updated.
-func DISABLE_TestHeartbeat_MarkAsDown(t *testing.T) {
-	f := heartbeatFixture{t: t}
-	defer f.Cleanup()
-
-	f.Bootstrap()
-	f.Grow()
-
-	leader := f.Leader()
-	leaderState := f.State(leader)
-
-	// Artificially mark all nodes as down
-	t.Logf("marking all nodes as down")
-	err := leaderState.Cluster.Transaction(func(tx *db.ClusterTx) error {
-		nodes, err := tx.Nodes()
-		require.NoError(t, err)
-		for _, node := range nodes {
-			err := tx.NodeHeartbeat(node.Address, time.Now().Add(-time.Minute))
-			require.NoError(t, err)
-		}
-		return nil
-	})
-	require.NoError(t, err)
-
-	follower := f.Follower()
-
-	// Shutdown the follower node and perform the heartbeat requests.
-	f.Server(follower).Close()
-	heartbeat, _ := cluster.Heartbeat(leader, leaderState.Cluster)
-	ctx := context.Background()
-	heartbeat(ctx)
-
-	// The heartbeat timestamp of the second node did not get updated
-	err = leaderState.Cluster.Transaction(func(tx *db.ClusterTx) error {
-		nodes, err := tx.Nodes()
-		require.NoError(t, err)
-
-		offlineThreshold, err := tx.NodeOfflineThreshold()
-		require.NoError(t, err)
-
-		i := f.Index(follower)
-		assert.True(t, nodes[i].IsOffline(offlineThreshold))
 		return nil
 	})
 	require.NoError(t, err)
@@ -143,10 +104,10 @@ func (f *heartbeatFixture) Grow() *cluster.Gateway {
 	name := address
 
 	nodes, err := cluster.Accept(
-		targetState, target, name, address, cluster.SchemaVersion, len(version.APIExtensions))
+		targetState, target, name, address, cluster.SchemaVersion, len(version.APIExtensions), osarch.ARCH_64BIT_INTEL_X86)
 	require.NoError(f.t, err)
 
-	err = cluster.Join(state, gateway, target.Cert(), name, nodes)
+	err = cluster.Join(state, gateway, target.NetworkCert(), target.ServerCert(), name, nodes)
 	require.NoError(f.t, err)
 
 	return gateway
@@ -160,14 +121,19 @@ func (f *heartbeatFixture) Leader() *cluster.Gateway {
 
 	for {
 		for _, gateway := range f.gateways {
-			if gateway.Raft().State() == raft.Leader {
+			isLeader, err := gateway.IsLeader()
+			if err != nil {
+				f.t.Errorf("failed to check leadership: %v", err)
+			}
+
+			if isLeader {
 				return gateway
 			}
 		}
 
 		select {
 		case <-ctx.Done():
-			f.t.Fatalf("no leader was elected within %s", timeout)
+			f.t.Errorf("no leader was elected within %s", timeout)
 		default:
 		}
 
@@ -184,14 +150,19 @@ func (f *heartbeatFixture) Follower() *cluster.Gateway {
 
 	for {
 		for _, gateway := range f.gateways {
-			if gateway.Raft().State() == raft.Follower {
+			isLeader, err := gateway.IsLeader()
+			if err != nil {
+				f.t.Errorf("failed to check leadership: %v", err)
+			}
+
+			if !isLeader {
 				return gateway
 			}
 		}
 
 		select {
 		case <-ctx.Done():
-			f.t.Fatalf("no node running as follower")
+			f.t.Errorf("no node running as follower")
 		default:
 		}
 
@@ -233,27 +204,47 @@ func (f *heartbeatFixture) node() (*state.State, *cluster.Gateway, string) {
 	state, cleanup := state.NewTestState(f.t)
 	f.cleanups = append(f.cleanups, cleanup)
 
-	cert := shared.TestingKeyPair()
-	gateway := newGateway(f.t, state.Node, cert)
-	f.cleanups = append(f.cleanups, func() { gateway.Shutdown() })
+	serverCert := shared.TestingKeyPair()
+	state.ServerCert = func() *shared.CertInfo { return serverCert }
+	gateway := newGateway(f.t, state.DB.Node, serverCert, serverCert)
+	f.cleanups = append(f.cleanups, func() { _ = gateway.Shutdown() })
 
 	mux := http.NewServeMux()
-	server := newServer(cert, mux)
+	server := newServer(serverCert, mux)
 
-	for path, handler := range gateway.HandlerFuncs() {
+	trustedCerts := func() map[clusterDB.CertificateType]map[string]x509.Certificate {
+		return nil
+	}
+
+	for path, handler := range gateway.HandlerFuncs(nil, trustedCerts) {
 		mux.HandleFunc(path, handler)
 	}
 
 	address := server.Listener.Addr().String()
 	mf := &membershipFixtures{t: f.t, state: state}
-	mf.NetworkAddress(address)
+	mf.ClusterAddress(address)
 
 	var err error
-	require.NoError(f.t, state.Cluster.Close())
-	store := gateway.ServerStore()
+	require.NoError(f.t, state.DB.Cluster.Close())
+	store := gateway.NodeStore()
 	dial := gateway.DialFunc()
-	state.Cluster, err = db.OpenCluster(
-		"db.bin", store, address, "/unused/db/dir", 5*time.Second, dqlite.WithDialFunc(dial))
+	state.DB.Cluster, err = db.OpenCluster(context.Background(), "db.bin", store, address, "/unused/db/dir", 5*time.Second, nil, driver.WithDialFunc(dial))
+	require.NoError(f.t, err)
+
+	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		state.GlobalConfig, err = clusterConfig.Load(tx)
+		if err != nil {
+			return err
+		}
+
+		// Get the local node (will be used if clustered).
+		state.ServerName, err = tx.GetLocalNodeName()
+		if err != nil {
+			return err
+		}
+
+		return nil
+	})
 	require.NoError(f.t, err)
 
 	f.gateways[len(f.gateways)] = gateway
@@ -268,6 +259,7 @@ func (f *heartbeatFixture) Cleanup() {
 	for i := len(f.cleanups) - 1; i >= 0; i-- {
 		f.cleanups[i]()
 	}
+
 	for _, server := range f.servers {
 		server.Close()
 	}
