@@ -1,6 +1,8 @@
 package cluster_test
 
 import (
+	"context"
+	"crypto/x509"
 	"fmt"
 	"io/ioutil"
 	"net/http"
@@ -8,15 +10,19 @@ import (
 	"testing"
 	"time"
 
-	"github.com/CanonicalLtd/go-dqlite"
-	"github.com/lxc/lxd/lxd/cluster"
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/version"
-	"github.com/mpvl/subtest"
+	"github.com/canonical/go-dqlite/driver"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/lxc/lxd/lxd/cluster"
+	clusterConfig "github.com/lxc/lxd/lxd/cluster/config"
+	"github.com/lxc/lxd/lxd/db"
+	clusterDB "github.com/lxc/lxd/lxd/db/cluster"
+	"github.com/lxc/lxd/lxd/project"
+	"github.com/lxc/lxd/lxd/state"
+	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/osarch"
+	"github.com/lxc/lxd/shared/version"
 )
 
 func TestBootstrap_UnmetPreconditions(t *testing.T) {
@@ -26,48 +32,50 @@ func TestBootstrap_UnmetPreconditions(t *testing.T) {
 	}{
 		{
 			func(f *membershipFixtures) {
-				f.NetworkAddress("1.2.3.4:666")
+				f.ClusterAddress("1.2.3.4:666")
+				f.RaftNode("5.6.7.8:666")
 				filename := filepath.Join(f.state.OS.VarDir, "cluster.crt")
-				ioutil.WriteFile(filename, []byte{}, 0644)
+				_ = ioutil.WriteFile(filename, []byte{}, 0644)
 			},
-			"inconsistent state: found leftover cluster certificate",
+			"Inconsistent state: found leftover cluster certificate",
 		},
 		{
 			func(*membershipFixtures) {},
-			"no core.https_address config is set on this node",
+			"No cluster.https_address config is set on this member",
 		},
 		{
 			func(f *membershipFixtures) {
-				f.NetworkAddress("1.2.3.4:666")
+				f.ClusterAddress("1.2.3.4:666")
 				f.RaftNode("5.6.7.8:666")
 			},
-			"the node is already part of a cluster",
+			"The member is already part of a cluster",
 		},
 		{
 			func(f *membershipFixtures) {
 				f.RaftNode("5.6.7.8:666")
 			},
-			"inconsistent state: found leftover entries in raft_nodes",
+			"Inconsistent state: found leftover entries in raft_nodes",
 		},
 		{
 			func(f *membershipFixtures) {
-				f.NetworkAddress("1.2.3.4:666")
+				f.ClusterAddress("1.2.3.4:666")
 				f.ClusterNode("5.6.7.8:666")
 			},
-			"inconsistent state: found leftover entries in nodes",
+			"Inconsistent state: found leftover entries in nodes",
 		},
 	}
 
 	for _, c := range cases {
-		subtest.Run(t, c.error, func(t *testing.T) {
+		t.Run(c.error, func(t *testing.T) {
 			state, cleanup := state.NewTestState(t)
 			defer cleanup()
 
 			c.setup(&membershipFixtures{t: t, state: state})
 
-			cert := shared.TestingKeyPair()
-			gateway := newGateway(t, state.Node, cert)
-			defer gateway.Shutdown()
+			serverCert := shared.TestingKeyPair()
+			state.ServerCert = func() *shared.CertInfo { return serverCert }
+			gateway := newGateway(t, state.DB.Node, serverCert, serverCert)
+			defer func() { _ = gateway.Shutdown() }()
 
 			err := cluster.Bootstrap(state, gateway, "buzz")
 			assert.EqualError(t, err, c.error)
@@ -79,35 +87,36 @@ func TestBootstrap(t *testing.T) {
 	state, cleanup := state.NewTestState(t)
 	defer cleanup()
 
-	cert := shared.TestingKeyPair()
-	gateway := newGateway(t, state.Node, cert)
-	defer gateway.Shutdown()
+	serverCert := shared.TestingKeyPair()
+	gateway := newGateway(t, state.DB.Node, serverCert, serverCert)
+	state.ServerCert = func() *shared.CertInfo { return serverCert }
+	defer func() { _ = gateway.Shutdown() }()
 
 	mux := http.NewServeMux()
-	server := newServer(cert, mux)
+	server := newServer(serverCert, mux)
 	defer server.Close()
 
 	address := server.Listener.Addr().String()
 	f := &membershipFixtures{t: t, state: state}
-	f.NetworkAddress(address)
+	f.ClusterAddress(address)
 
 	err := cluster.Bootstrap(state, gateway, "buzz")
 	require.NoError(t, err)
 
 	// The node-local database has now an entry in the raft_nodes table
-	err = state.Node.Transaction(func(tx *db.NodeTx) error {
-		nodes, err := tx.RaftNodes()
+	err = state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+		nodes, err := tx.GetRaftNodes()
 		require.NoError(t, err)
 		require.Len(t, nodes, 1)
-		assert.Equal(t, int64(1), nodes[0].ID)
+		assert.Equal(t, uint64(1), nodes[0].ID)
 		assert.Equal(t, address, nodes[0].Address)
 		return nil
 	})
 	require.NoError(t, err)
 
 	// The cluster database has now an entry in the nodes table
-	err = state.Cluster.Transaction(func(tx *db.ClusterTx) error {
-		nodes, err := tx.Nodes()
+	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		nodes, err := tx.GetNodes()
 		require.NoError(t, err)
 		require.Len(t, nodes, 1)
 		assert.Equal(t, "buzz", nodes[0].Name)
@@ -119,8 +128,12 @@ func TestBootstrap(t *testing.T) {
 	// The cluster certificate is in place.
 	assert.True(t, shared.PathExists(filepath.Join(state.OS.VarDir, "cluster.crt")))
 
+	trustedCerts := func() map[clusterDB.CertificateType]map[string]x509.Certificate {
+		return nil
+	}
+
 	// The dqlite driver is now exposed over the network.
-	for path, handler := range gateway.HandlerFuncs() {
+	for path, handler := range gateway.HandlerFuncs(nil, trustedCerts) {
 		mux.HandleFunc(path, handler)
 	}
 
@@ -128,7 +141,7 @@ func TestBootstrap(t *testing.T) {
 	require.NoError(t, err)
 	assert.Equal(t, 1, count)
 
-	enabled, err := cluster.Enabled(state.Node)
+	enabled, err := cluster.Enabled(state.DB.Node)
 	require.NoError(t, err)
 	assert.True(t, enabled)
 }
@@ -149,7 +162,7 @@ func TestAccept_UnmetPreconditions(t *testing.T) {
 			cluster.SchemaVersion,
 			len(version.APIExtensions),
 			func(f *membershipFixtures) {},
-			"clustering not enabled",
+			"Clustering isn't enabled",
 		},
 		{
 			"rusp",
@@ -159,7 +172,7 @@ func TestAccept_UnmetPreconditions(t *testing.T) {
 			func(f *membershipFixtures) {
 				f.ClusterNode("5.6.7.8:666")
 			},
-			"cluster already has node with name rusp",
+			"The cluster already has a member with name: rusp",
 		},
 		{
 			"buzz",
@@ -169,7 +182,7 @@ func TestAccept_UnmetPreconditions(t *testing.T) {
 			func(f *membershipFixtures) {
 				f.ClusterNode("5.6.7.8:666")
 			},
-			"cluster already has node with address 5.6.7.8:666",
+			"The cluster already has a member with address: 5.6.7.8:666",
 		},
 		{
 			"buzz",
@@ -179,7 +192,7 @@ func TestAccept_UnmetPreconditions(t *testing.T) {
 			func(f *membershipFixtures) {
 				f.ClusterNode("5.6.7.8:666")
 			},
-			fmt.Sprintf("schema version mismatch: cluster has %d", cluster.SchemaVersion),
+			fmt.Sprintf("The joining server version doesn't match (expected %s with DB schema %d)", version.Version, cluster.SchemaVersion-1),
 		},
 		{
 			"buzz",
@@ -189,21 +202,22 @@ func TestAccept_UnmetPreconditions(t *testing.T) {
 			func(f *membershipFixtures) {
 				f.ClusterNode("5.6.7.8:666")
 			},
-			fmt.Sprintf("API version mismatch: cluster has %d", len(version.APIExtensions)),
+			fmt.Sprintf("The joining server version doesn't match (expected %s with API count %d)", version.Version, len(version.APIExtensions)-1),
 		},
 	}
+
 	for _, c := range cases {
-		subtest.Run(t, c.error, func(t *testing.T) {
+		t.Run(c.error, func(t *testing.T) {
 			state, cleanup := state.NewTestState(t)
 			defer cleanup()
 
-			cert := shared.TestingKeyPair()
-			gateway := newGateway(t, state.Node, cert)
-			defer gateway.Shutdown()
+			serverCert := shared.TestingKeyPair()
+			gateway := newGateway(t, state.DB.Node, serverCert, serverCert)
+			defer func() { _ = gateway.Shutdown() }()
 
 			c.setup(&membershipFixtures{t: t, state: state})
 
-			_, err := cluster.Accept(state, gateway, c.name, c.address, c.schema, c.api)
+			_, err := cluster.Accept(state, gateway, c.name, c.address, c.schema, c.api, osarch.ARCH_64BIT_INTEL_X86)
 			assert.EqualError(t, err, c.error)
 		})
 	}
@@ -214,20 +228,38 @@ func TestAccept(t *testing.T) {
 	state, cleanup := state.NewTestState(t)
 	defer cleanup()
 
-	cert := shared.TestingKeyPair()
-	gateway := newGateway(t, state.Node, cert)
-	defer gateway.Shutdown()
+	serverCert := shared.TestingKeyPair()
+	gateway := newGateway(t, state.DB.Node, serverCert, serverCert)
+	defer func() { _ = gateway.Shutdown() }()
 
 	f := &membershipFixtures{t: t, state: state}
 	f.RaftNode("1.2.3.4:666")
 	f.ClusterNode("1.2.3.4:666")
 
+	err := state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		state.GlobalConfig, err = clusterConfig.Load(tx)
+		if err != nil {
+			return err
+		}
+
+		// Get the local node (will be used if clustered).
+		state.ServerName, err = tx.GetLocalNodeName()
+		if err != nil {
+			return err
+		}
+
+		return nil
+	})
+	require.NoError(t, err)
+
 	nodes, err := cluster.Accept(
-		state, gateway, "buzz", "5.6.7.8:666", cluster.SchemaVersion, len(version.APIExtensions))
+		state, gateway, "buzz", "5.6.7.8:666", cluster.SchemaVersion, len(version.APIExtensions), osarch.ARCH_64BIT_INTEL_X86)
 	assert.NoError(t, err)
 	assert.Len(t, nodes, 2)
-	assert.Equal(t, int64(1), nodes[0].ID)
-	assert.Equal(t, int64(2), nodes[1].ID)
+	assert.Equal(t, uint64(1), nodes[0].ID)
+	assert.Equal(t, uint64(3), nodes[1].ID)
 	assert.Equal(t, "1.2.3.4:666", nodes[0].Address)
 	assert.Equal(t, "5.6.7.8:666", nodes[1].Address)
 }
@@ -242,33 +274,61 @@ func TestJoin(t *testing.T) {
 	targetState, cleanup := state.NewTestState(t)
 	defer cleanup()
 
-	targetGateway := newGateway(t, targetState.Node, targetCert)
-	defer targetGateway.Shutdown()
+	targetGateway := newGateway(t, targetState.DB.Node, targetCert, targetCert)
+	defer func() { _ = targetGateway.Shutdown() }()
 
-	for path, handler := range targetGateway.HandlerFuncs() {
+	altServerCert := shared.TestingAltKeyPair()
+	trustedAltServerCert, _ := x509.ParseCertificate(altServerCert.KeyPair().Certificate[0])
+
+	trustedCerts := func() map[clusterDB.CertificateType]map[string]x509.Certificate {
+		return map[clusterDB.CertificateType]map[string]x509.Certificate{
+			clusterDB.CertificateTypeServer: {
+				altServerCert.Fingerprint(): *trustedAltServerCert,
+			},
+		}
+	}
+
+	for path, handler := range targetGateway.HandlerFuncs(nil, trustedCerts) {
 		targetMux.HandleFunc(path, handler)
 	}
 
 	targetAddress := targetServer.Listener.Addr().String()
 
-	require.NoError(t, targetState.Cluster.Close())
+	require.NoError(t, targetState.DB.Cluster.Close())
 
-	targetStore := targetGateway.ServerStore()
+	targetStore := targetGateway.NodeStore()
 	targetDialFunc := targetGateway.DialFunc()
 
 	var err error
-	targetState.Cluster, err = db.OpenCluster(
-		"db.bin", targetStore, targetAddress, "/unused/db/dir",
-		10*time.Second,
-		dqlite.WithDialFunc(targetDialFunc))
+	targetState.DB.Cluster, err = db.OpenCluster(context.Background(), "db.bin", targetStore, targetAddress, "/unused/db/dir", 10*time.Second, nil, driver.WithDialFunc(targetDialFunc))
+	targetState.ServerCert = func() *shared.CertInfo { return targetCert }
 	require.NoError(t, err)
 
+	err = targetState.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		targetState.GlobalConfig, err = clusterConfig.Load(tx)
+		if err != nil {
+			return err
+		}
+
+		// Get the local node (will be used if clustered).
+		targetState.ServerName, err = tx.GetLocalNodeName()
+		if err != nil {
+			return err
+		}
+
+		return nil
+	})
+	require.NoError(t, err)
+
+	// PreparedStmts is a global variable and will be overwritten by the OpenCluster call below, so save it here.
+	targetStmts := clusterDB.PreparedStmts
+
 	targetF := &membershipFixtures{t: t, state: targetState}
-	targetF.NetworkAddress(targetAddress)
+	targetF.ClusterAddress(targetAddress)
 
 	err = cluster.Bootstrap(targetState, targetGateway, "buzz")
 	require.NoError(t, err)
-	_, err = targetState.Cluster.Networks()
+	_, err = targetState.DB.Cluster.GetNetworks(project.Default)
 	require.NoError(t, err)
 
 	// Setup a joining node
@@ -279,55 +339,68 @@ func TestJoin(t *testing.T) {
 	state, cleanup := state.NewTestState(t)
 	defer cleanup()
 
-	cert := shared.TestingAltKeyPair()
-	gateway := newGateway(t, state.Node, cert)
+	gateway := newGateway(t, state.DB.Node, targetCert, altServerCert)
 
-	defer gateway.Shutdown()
+	defer func() { _ = gateway.Shutdown() }()
 
-	for path, handler := range gateway.HandlerFuncs() {
+	for path, handler := range gateway.HandlerFuncs(nil, trustedCerts) {
 		mux.HandleFunc(path, handler)
 	}
 
 	address := server.Listener.Addr().String()
 
-	require.NoError(t, state.Cluster.Close())
+	require.NoError(t, state.DB.Cluster.Close())
 
-	store := gateway.ServerStore()
+	store := gateway.NodeStore()
 	dialFunc := gateway.DialFunc()
 
-	state.Cluster, err = db.OpenCluster(
-		"db.bin", store, address, "/unused/db/dir", 5*time.Second, dqlite.WithDialFunc(dialFunc))
+	state.DB.Cluster, err = db.OpenCluster(context.Background(), "db.bin", store, address, "/unused/db/dir", 5*time.Second, nil, driver.WithDialFunc(dialFunc))
 	require.NoError(t, err)
 
+	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		state.GlobalConfig, err = clusterConfig.Load(tx)
+		if err != nil {
+			return err
+		}
+
+		// Get the local node (will be used if clustered).
+		state.ServerName, err = tx.GetLocalNodeName()
+		if err != nil {
+			return err
+		}
+
+		return nil
+	})
+	require.NoError(t, err)
+
+	// Save the other instance of PreparedStmts here.
+	sourceStmts := clusterDB.PreparedStmts
+
 	f := &membershipFixtures{t: t, state: state}
-	f.NetworkAddress(address)
+	f.ClusterAddress(address)
 
 	// Accept the joining node.
+	clusterDB.PreparedStmts = targetStmts
 	raftNodes, err := cluster.Accept(
-		targetState, targetGateway, "rusp", address, cluster.SchemaVersion, len(version.APIExtensions))
+		targetState, targetGateway, "rusp", address, cluster.SchemaVersion, len(version.APIExtensions), osarch.ARCH_64BIT_INTEL_X86)
 	require.NoError(t, err)
 
 	// Actually join the cluster.
-	err = cluster.Join(state, gateway, targetCert, "rusp", raftNodes)
+	clusterDB.PreparedStmts = sourceStmts
+	err = cluster.Join(state, gateway, targetCert, altServerCert, "rusp", raftNodes)
 	require.NoError(t, err)
 
 	// The leader now returns an updated list of raft nodes.
+	// The new node is not included to ensure distributed consensus.
 	raftNodes, err = targetGateway.RaftNodes()
 	require.NoError(t, err)
 	assert.Len(t, raftNodes, 2)
-	assert.Equal(t, int64(1), raftNodes[0].ID)
+	assert.Equal(t, uint64(1), raftNodes[0].ID)
 	assert.Equal(t, targetAddress, raftNodes[0].Address)
-	assert.Equal(t, int64(2), raftNodes[1].ID)
+	assert.Equal(t, db.RaftVoter, raftNodes[0].Role)
+	assert.Equal(t, uint64(2), raftNodes[1].ID)
 	assert.Equal(t, address, raftNodes[1].Address)
-
-	// The List function returns all nodes in the cluster.
-	nodes, err := cluster.List(state)
-	require.NoError(t, err)
-	assert.Len(t, nodes, 2)
-	assert.Equal(t, "Online", nodes[0].Status)
-	assert.Equal(t, "Online", nodes[1].Status)
-	assert.True(t, nodes[0].Database)
-	assert.True(t, nodes[1].Database)
+	assert.Equal(t, db.RaftStandBy, raftNodes[1].Role)
 
 	// The Count function returns the number of nodes.
 	count, err := cluster.Count(state)
@@ -335,15 +408,16 @@ func TestJoin(t *testing.T) {
 	assert.Equal(t, 2, count)
 
 	// Leave the cluster.
-	leaving, err := cluster.Leave(state, gateway, "rusp", false /* force */)
+	leaving, err := cluster.Leave(state, targetGateway, "rusp", false /* force */)
 	require.NoError(t, err)
 	assert.Equal(t, address, leaving)
-	err = cluster.Purge(state.Cluster, "rusp")
+	clusterDB.PreparedStmts = targetStmts
+	err = cluster.Purge(targetState.DB.Cluster, "rusp")
 	require.NoError(t, err)
 
 	// The node has gone from the cluster db.
-	err = targetState.Cluster.Transaction(func(tx *db.ClusterTx) error {
-		nodes, err := tx.Nodes()
+	err = targetState.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		nodes, err := tx.GetNodes()
 		require.NoError(t, err)
 		assert.Len(t, nodes, 1)
 		return nil
@@ -351,91 +425,9 @@ func TestJoin(t *testing.T) {
 	require.NoError(t, err)
 
 	// The node has gone from the raft cluster.
-	raft := targetGateway.Raft()
-	future := raft.GetConfiguration()
-	require.NoError(t, future.Error())
-	assert.Len(t, future.Configuration().Servers, 1)
-
-	count, err = cluster.Count(state)
+	members, err := targetGateway.RaftNodes()
 	require.NoError(t, err)
-	assert.Equal(t, 1, count)
-}
-
-func FLAKY_TestPromote(t *testing.T) {
-	// Setup a target node running as leader of a cluster.
-	targetCert := shared.TestingKeyPair()
-	targetMux := http.NewServeMux()
-	targetServer := newServer(targetCert, targetMux)
-	defer targetServer.Close()
-
-	targetState, cleanup := state.NewTestState(t)
-	defer cleanup()
-
-	targetGateway := newGateway(t, targetState.Node, targetCert)
-	defer targetGateway.Shutdown()
-
-	for path, handler := range targetGateway.HandlerFuncs() {
-		targetMux.HandleFunc(path, handler)
-	}
-
-	targetAddress := targetServer.Listener.Addr().String()
-	var err error
-	require.NoError(t, targetState.Cluster.Close())
-	store := targetGateway.ServerStore()
-	dialFunc := targetGateway.DialFunc()
-	targetState.Cluster, err = db.OpenCluster(
-		"db.bin", store, targetAddress, "/unused/db/dir", 5*time.Second, dqlite.WithDialFunc(dialFunc))
-	require.NoError(t, err)
-	targetF := &membershipFixtures{t: t, state: targetState}
-	targetF.NetworkAddress(targetAddress)
-
-	err = cluster.Bootstrap(targetState, targetGateway, "buzz")
-	require.NoError(t, err)
-
-	// Setup a node to be promoted.
-	mux := http.NewServeMux()
-	server := newServer(targetCert, mux) // Use the same cert, as we're already part of the cluster
-	defer server.Close()
-
-	state, cleanup := state.NewTestState(t)
-	defer cleanup()
-
-	address := server.Listener.Addr().String()
-	targetF.ClusterNode(address) // Add the non database node to the cluster database
-	f := &membershipFixtures{t: t, state: state}
-	f.NetworkAddress(address)
-	f.RaftNode(targetAddress) // Insert the leader in our local list of database nodes
-
-	gateway := newGateway(t, state.Node, targetCert)
-	defer gateway.Shutdown()
-
-	for path, handler := range gateway.HandlerFuncs() {
-		mux.HandleFunc(path, handler)
-	}
-
-	// Promote the node.
-	targetF.RaftNode(address) // Add the address of the node to be promoted in the leader's db
-	raftNodes := targetF.RaftNodes()
-	err = cluster.Promote(state, gateway, raftNodes)
-	require.NoError(t, err)
-
-	// The leader now returns an updated list of raft nodes.
-	raftNodes, err = targetGateway.RaftNodes()
-	require.NoError(t, err)
-	assert.Len(t, raftNodes, 2)
-	assert.Equal(t, int64(1), raftNodes[0].ID)
-	assert.Equal(t, targetAddress, raftNodes[0].Address)
-	assert.Equal(t, int64(2), raftNodes[1].ID)
-	assert.Equal(t, address, raftNodes[1].Address)
-
-	// The List function returns all nodes in the cluster.
-	nodes, err := cluster.List(state)
-	require.NoError(t, err)
-	assert.Len(t, nodes, 2)
-	assert.Equal(t, "Online", nodes[0].Status)
-	assert.Equal(t, "Online", nodes[1].Status)
-	assert.True(t, nodes[0].Database)
-	assert.True(t, nodes[1].Database)
+	assert.Len(t, members, 1)
 }
 
 // Helper for setting fixtures for Bootstrap tests.
@@ -445,11 +437,24 @@ type membershipFixtures struct {
 }
 
 // Set core.https_address to the given value.
-func (h *membershipFixtures) NetworkAddress(address string) {
-	err := h.state.Node.Transaction(func(tx *db.NodeTx) error {
+func (h *membershipFixtures) CoreAddress(address string) {
+	err := h.state.DB.Node.Transaction(func(tx *db.NodeTx) error {
 		config := map[string]string{
 			"core.https_address": address,
 		}
+
+		return tx.UpdateConfig(config)
+	})
+	require.NoError(h.t, err)
+}
+
+// Set cluster.https_address to the given value.
+func (h *membershipFixtures) ClusterAddress(address string) {
+	err := h.state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+		config := map[string]string{
+			"cluster.https_address": address,
+		}
+
 		return tx.UpdateConfig(config)
 	})
 	require.NoError(h.t, err)
@@ -457,8 +462,8 @@ func (h *membershipFixtures) NetworkAddress(address string) {
 
 // Add the given address to the raft_nodes table.
 func (h *membershipFixtures) RaftNode(address string) {
-	err := h.state.Node.Transaction(func(tx *db.NodeTx) error {
-		_, err := tx.RaftNodeAdd(address)
+	err := h.state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+		_, err := tx.CreateRaftNode(address, "rusp")
 		return err
 	})
 	require.NoError(h.t, err)
@@ -467,9 +472,9 @@ func (h *membershipFixtures) RaftNode(address string) {
 // Get the current list of the raft nodes in the raft_nodes table.
 func (h *membershipFixtures) RaftNodes() []db.RaftNode {
 	var nodes []db.RaftNode
-	err := h.state.Node.Transaction(func(tx *db.NodeTx) error {
+	err := h.state.DB.Node.Transaction(func(tx *db.NodeTx) error {
 		var err error
-		nodes, err = tx.RaftNodes()
+		nodes, err = tx.GetRaftNodes()
 		return err
 	})
 	require.NoError(h.t, err)
@@ -478,8 +483,8 @@ func (h *membershipFixtures) RaftNodes() []db.RaftNode {
 
 // Add the given address to the nodes table of the cluster database.
 func (h *membershipFixtures) ClusterNode(address string) {
-	err := h.state.Cluster.Transaction(func(tx *db.ClusterTx) error {
-		_, err := tx.NodeAdd("rusp", address)
+	err := h.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		_, err := tx.CreateNode("rusp", address)
 		return err
 	})
 	require.NoError(h.t, err)
