@@ -1,9 +1,11 @@
 package cluster
 
 import (
+	"context"
 	"fmt"
 
 	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/response"
 )
 
 // ResolveTarget is a convenience for handling the value ?targetNode query
@@ -11,8 +13,8 @@ import (
 // the given node is the local one.
 func ResolveTarget(cluster *db.Cluster, target string) (string, error) {
 	address := ""
-	err := cluster.Transaction(func(tx *db.ClusterTx) error {
-		name, err := tx.NodeName()
+	err := cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		name, err := tx.GetLocalNodeName()
 		if err != nil {
 			return err
 		}
@@ -21,9 +23,9 @@ func ResolveTarget(cluster *db.Cluster, target string) (string, error) {
 			return nil
 		}
 
-		node, err := tx.NodeByName(target)
+		node, err := tx.GetNodeByName(target)
 		if err != nil {
-			if err == db.ErrNoSuchObject {
+			if response.IsNotFoundError(err) {
 				return fmt.Errorf("No cluster member called '%s'", target)
 			}
 
