@@ -1,39 +1,32 @@
 package main
 
 import (
-	"github.com/lxc/lxd/lxd/cluster"
+	clusterConfig "github.com/lxc/lxd/lxd/cluster/config"
 	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/lxd/node"
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared"
 )
 
-func daemonConfigRender(state *state.State) (map[string]interface{}, error) {
-	config := map[string]interface{}{}
+func daemonConfigRender(state *state.State) (map[string]any, error) {
+	config := map[string]any{}
 
-	// Turn the config into a JSON-compatible map
-	err := state.Cluster.Transaction(func(tx *db.ClusterTx) error {
-		clusterConfig, err := cluster.ConfigLoad(tx)
-		if err != nil {
-			return err
-		}
-		for key, value := range clusterConfig.Dump() {
-			config[key] = value
-		}
-		return nil
-	})
-	if err != nil {
-		return nil, err
+	// Turn the config into a JSON-compatible map.
+	for key, value := range state.GlobalConfig.Dump() {
+		config[key] = value
 	}
 
-	err = state.Node.Transaction(func(tx *db.NodeTx) error {
+	// Apply the local config.
+	err := state.DB.Node.Transaction(func(tx *db.NodeTx) error {
 		nodeConfig, err := node.ConfigLoad(tx)
 		if err != nil {
 			return err
 		}
+
 		for key, value := range nodeConfig.Dump() {
 			config[key] = value
 		}
+
 		return nil
 	})
 	if err != nil {
@@ -43,18 +36,11 @@ func daemonConfigRender(state *state.State) (map[string]interface{}, error) {
 	return config, nil
 }
 
-func daemonConfigSetProxy(d *Daemon, config *cluster.Config) {
+func daemonConfigSetProxy(d *Daemon, config *clusterConfig.Config) {
 	// Update the cached proxy function
 	d.proxy = shared.ProxyFromConfig(
 		config.ProxyHTTPS(),
 		config.ProxyHTTP(),
 		config.ProxyIgnoreHosts(),
 	)
-
-	// Clear the simplestreams cache as it's tied to the old proxy config
-	imageStreamCacheLock.Lock()
-	for k := range imageStreamCache {
-		delete(imageStreamCache, k)
-	}
-	imageStreamCacheLock.Unlock()
 }
