@@ -1,19 +1,22 @@
 package cluster
 
 import (
+	"context"
 	"database/sql"
 	"fmt"
 	"path/filepath"
+	"strings"
 	"sync/atomic"
 
-	"github.com/CanonicalLtd/go-dqlite"
+	driver "github.com/canonical/go-dqlite/driver"
+
 	"github.com/lxc/lxd/lxd/db/query"
 	"github.com/lxc/lxd/lxd/db/schema"
 	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/osarch"
 	"github.com/lxc/lxd/shared/version"
-	"github.com/pkg/errors"
 )
 
 // Open the cluster database object.
@@ -23,10 +26,10 @@ import (
 //
 // The dialer argument is a function that returns a gRPC dialer that can be
 // used to connect to a database node using the gRPC SQL package.
-func Open(name string, store dqlite.ServerStore, options ...dqlite.DriverOption) (*sql.DB, error) {
-	driver, err := dqlite.NewDriver(store, options...)
+func Open(name string, store driver.NodeStore, options ...driver.Option) (*sql.DB, error) {
+	driver, err := driver.New(store, options...)
 	if err != nil {
-		return nil, errors.Wrap(err, "Failed to create dqlite driver")
+		return nil, fmt.Errorf("Failed to create dqlite driver: %w", err)
 	}
 
 	driverName := dqliteDriverName()
@@ -38,9 +41,10 @@ func Open(name string, store dqlite.ServerStore, options ...dqlite.DriverOption)
 	if name == "" {
 		name = "db.bin"
 	}
+
 	db, err := sql.Open(driverName, name)
 	if err != nil {
-		return nil, fmt.Errorf("cannot open cluster database: %v", err)
+		return nil, fmt.Errorf("cannot open cluster database: %w", err)
 	}
 
 	return db, nil
@@ -63,7 +67,7 @@ func EnsureSchema(db *sql.DB, address string, dir string) (bool, error) {
 		// Check if this is a fresh instance.
 		isUpdate, err := schema.DoesSchemaTableExist(tx)
 		if err != nil {
-			return errors.Wrap(err, "failed to check if schema table exists")
+			return fmt.Errorf("failed to check if schema table exists: %w", err)
 		}
 
 		if !isUpdate {
@@ -74,11 +78,11 @@ func EnsureSchema(db *sql.DB, address string, dir string) (bool, error) {
 		clustered := true
 		n, err := selectUnclusteredNodesCount(tx)
 		if err != nil {
-			return errors.Wrap(err, "failed to fetch unclustered nodes count")
+			return fmt.Errorf("failed to fetch unclustered nodes count: %w", err)
 		}
+
 		if n > 1 {
-			// This should never happen, since we only add nodes
-			// with valid addresses, but check it for sanity.
+			// This should never happen, since we only add nodes with valid addresses..
 			return fmt.Errorf("found more than one unclustered nodes")
 		} else if n == 1 {
 			clustered = false
@@ -95,8 +99,9 @@ func EnsureSchema(db *sql.DB, address string, dir string) (bool, error) {
 				filepath.Join(dir, "global.bak"),
 			)
 			if err != nil {
-				return errors.Wrap(err, "failed to backup global database")
+				return fmt.Errorf("failed to backup global database: %w", err)
 			}
+
 			backupDone = true
 		}
 
@@ -119,11 +124,11 @@ func EnsureSchema(db *sql.DB, address string, dir string) (bool, error) {
 		// Check if we're clustered
 		n, err := selectUnclusteredNodesCount(tx)
 		if err != nil {
-			return errors.Wrap(err, "failed to fetch unclustered nodes count")
+			return fmt.Errorf("failed to fetch unclustered nodes count: %w", err)
 		}
+
 		if n > 1 {
-			// This should never happen, since we only add nodes
-			// with valid addresses, but check it for sanity.
+			// This should never happen, since we only add nodes with valid addresses.
 			return fmt.Errorf("found more than one unclustered nodes")
 		} else if n == 1 {
 			address = "0.0.0.0" // We're not clustered
@@ -132,7 +137,7 @@ func EnsureSchema(db *sql.DB, address string, dir string) (bool, error) {
 		// Update the schema and api_extension columns of ourselves.
 		err = updateNodeVersion(tx, address, apiExtensions)
 		if err != nil {
-			return errors.Wrap(err, "failed to update node version info")
+			return fmt.Errorf("failed to update node version info: %w", err)
 		}
 
 		err = checkClusterIsUpgradable(tx, [2]int{len(updates), apiExtensions})
@@ -140,6 +145,7 @@ func EnsureSchema(db *sql.DB, address string, dir string) (bool, error) {
 			someNodesAreBehind = true
 			return schema.ErrGracefulAbort
 		}
+
 		return err
 	}
 
@@ -157,6 +163,7 @@ func EnsureSchema(db *sql.DB, address string, dir string) (bool, error) {
 	if someNodesAreBehind {
 		return false, nil
 	}
+
 	if err != nil {
 		return false, err
 	}
@@ -165,28 +172,54 @@ func EnsureSchema(db *sql.DB, address string, dir string) (bool, error) {
 	// 1. This is needed for referential integrity with other tables. Also,
 	// create a default profile.
 	if initial == 0 {
-		tx, err := db.Begin()
+		arch, err := osarch.ArchitectureGetLocalID()
 		if err != nil {
 			return false, err
 		}
-		stmt := `
-INSERT INTO nodes(id, name, address, schema, api_extensions) VALUES(1, 'none', '0.0.0.0', ?, ?)
-`
-		_, err = tx.Exec(stmt, SchemaVersion, apiExtensions)
-		if err != nil {
-			tx.Rollback()
-			return false, err
-		}
 
-		stmt = `
-INSERT INTO profiles (name, description) VALUES ('default', 'Default LXD profile')
+		err = query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
+			stmt := `
+INSERT INTO nodes(id, name, address, schema, api_extensions, arch, description) VALUES(1, 'none', '0.0.0.0', ?, ?, ?, '')
 `
-		_, err = tx.Exec(stmt)
-		if err != nil {
-			tx.Rollback()
-			return false, err
-		}
-		err = tx.Commit()
+			_, err = tx.Exec(stmt, SchemaVersion, apiExtensions, arch)
+			if err != nil {
+				return err
+			}
+
+			// Default project
+			var defaultProjectStmt strings.Builder
+			_, _ = defaultProjectStmt.WriteString("INSERT INTO projects (name, description) VALUES ('default', 'Default LXD project');")
+
+			for _, key := range ProjectFeatures {
+				_, _ = defaultProjectStmt.WriteString(fmt.Sprintf("INSERT INTO projects_config (project_id, key, value) VALUES (1, '%s', 'true');", key))
+			}
+
+			_, err = tx.Exec(defaultProjectStmt.String())
+			if err != nil {
+				return err
+			}
+
+			// Default profile
+			stmt = `
+INSERT INTO profiles (name, description, project_id) VALUES ('default', 'Default LXD profile', 1)
+`
+			_, err = tx.Exec(stmt)
+			if err != nil {
+				return err
+			}
+
+			// Default cluster group
+			stmt = `
+INSERT INTO cluster_groups (name, description) VALUES ('default', 'Default cluster group');
+INSERT INTO nodes_cluster_groups (node_id, group_id) VALUES(1, 1);
+`
+			_, err = tx.Exec(stmt)
+			if err != nil {
+				return err
+			}
+
+			return nil
+		})
 		if err != nil {
 			return false, err
 		}
@@ -212,7 +245,7 @@ func checkClusterIsUpgradable(tx *sql.Tx, target [2]int) error {
 	// Get the current versions in the nodes table.
 	versions, err := selectNodesVersions(tx)
 	if err != nil {
-		return errors.Wrap(err, "failed to fetch current nodes versions")
+		return fmt.Errorf("failed to fetch current nodes versions: %w", err)
 	}
 
 	for _, version := range versions {
@@ -220,6 +253,7 @@ func checkClusterIsUpgradable(tx *sql.Tx, target [2]int) error {
 		if err != nil {
 			return err
 		}
+
 		switch n {
 		case 0:
 			// Versions are equal, there's hope for the
@@ -237,7 +271,6 @@ func checkClusterIsUpgradable(tx *sql.Tx, target [2]int) error {
 			// since we need a greater version.
 			return fmt.Errorf("this node's version is behind, please upgrade")
 		default:
-			// Sanity.
 			panic("unexpected return value from compareVersions")
 		}
 	}
