@@ -1,23 +1,31 @@
 package cluster_test
 
 import (
+	"context"
 	"database/sql"
 	"fmt"
+	"io/ioutil"
+	"os"
+	"path/filepath"
 	"testing"
 
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+
 	"github.com/lxc/lxd/lxd/db/cluster"
 	"github.com/lxc/lxd/lxd/db/query"
+	"github.com/lxc/lxd/shared/osarch"
 	"github.com/lxc/lxd/shared/version"
-	"github.com/mpvl/subtest"
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
 )
 
 // If the node is not clustered, the schema updates works normally.
 func TestEnsureSchema_NoClustered(t *testing.T) {
+	dir, cleanup := newDir(t)
+	defer cleanup()
+	assert.NoError(t, os.Mkdir(filepath.Join(dir, "global"), 0711))
 	db := newDB(t)
 	addNode(t, db, "0.0.0.0", 1, 1)
-	ready, err := cluster.EnsureSchema(db, "1.2.3.4:666", "/unused/db/dir")
+	ready, err := cluster.EnsureSchema(db, "1.2.3.4:666", dir)
 	assert.True(t, ready)
 	assert.NoError(t, err)
 }
@@ -79,8 +87,9 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 			"nodes have inconsistent versions",
 		},
 	}
+
 	for _, c := range cases {
-		subtest.Run(t, c.title, func(t *testing.T) {
+		t.Run(c.title, func(t *testing.T) {
 			db := newDB(t)
 			c.setup(t, db)
 			ready, err := cluster.EnsureSchema(db, "1", "/unused/db/dir")
@@ -116,8 +125,9 @@ func TestEnsureSchema_UpdateNodeVersion(t *testing.T) {
 			true,
 		},
 	}
+
 	for _, c := range cases {
-		subtest.Run(t, fmt.Sprintf("%v", c.ready), func(t *testing.T) {
+		t.Run(fmt.Sprintf("%v", c.ready), func(t *testing.T) {
 			db := newDB(t)
 
 			// Add ourselves with an older schema version and API
@@ -157,12 +167,12 @@ CREATE TABLE schema (
 
 // Add a new node with the given address, schema version and number of api extensions.
 func addNode(t *testing.T, db *sql.DB, address string, schema int, apiExtensions int) {
-	err := query.Transaction(db, func(tx *sql.Tx) error {
+	err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 		stmt := `
-INSERT INTO nodes(name, address, schema, api_extensions) VALUES (?, ?, ?, ?)
+INSERT INTO nodes(name, address, schema, api_extensions, arch, description) VALUES (?, ?, ?, ?, ?, '')
 `
 		name := fmt.Sprintf("node at %s", address)
-		_, err := tx.Exec(stmt, name, address, schema, apiExtensions)
+		_, err := tx.Exec(stmt, name, address, schema, apiExtensions, osarch.ARCH_64BIT_INTEL_X86)
 		return err
 	})
 	require.NoError(t, err)
@@ -171,7 +181,7 @@ INSERT INTO nodes(name, address, schema, api_extensions) VALUES (?, ?, ?, ?)
 // Assert that the node with the given address has the given schema version and API
 // extensions number.
 func assertNode(t *testing.T, db *sql.DB, address string, schema int, apiExtensions int) {
-	err := query.Transaction(db, func(tx *sql.Tx) error {
+	err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 		where := "address=? AND schema=? AND api_extensions=?"
 		n, err := query.Count(tx, "nodes", where, address, schema, apiExtensions)
 		assert.Equal(t, 1, n, "node does not have expected version")
@@ -179,3 +189,22 @@ func assertNode(t *testing.T, db *sql.DB, address string, schema int, apiExtensi
 	})
 	require.NoError(t, err)
 }
+
+// Return a new temporary directory.
+func newDir(t *testing.T) (string, func()) {
+	t.Helper()
+
+	dir, err := ioutil.TempDir("", "dqlite-replication-test-")
+	assert.NoError(t, err)
+
+	cleanup := func() {
+		_, err := os.Stat(dir)
+		if err != nil {
+			assert.True(t, os.IsNotExist(err))
+		} else {
+			assert.NoError(t, os.RemoveAll(dir))
+		}
+	}
+
+	return dir, cleanup
+}
