@@ -17,13 +17,16 @@ func updateNodeVersion(tx *sql.Tx, address string, apiExtensions int) error {
 	if err != nil {
 		return err
 	}
+
 	n, err := result.RowsAffected()
 	if err != nil {
 		return err
 	}
+
 	if n != 1 {
 		return fmt.Errorf("updated %d rows instead of 1", n)
 	}
+
 	return nil
 }
 
@@ -38,19 +41,24 @@ func selectUnclusteredNodesCount(tx *sql.Tx) (int, error) {
 func selectNodesVersions(tx *sql.Tx) ([][2]int, error) {
 	versions := [][2]int{}
 
-	dest := func(i int) []interface{} {
+	dest := func(i int) []any {
 		versions = append(versions, [2]int{})
-		return []interface{}{&versions[i][0], &versions[i][1]}
+		return []any{&versions[i][0], &versions[i][1]}
 	}
 
-	stmt, err := tx.Prepare("SELECT schema, api_extensions FROM nodes")
+	stmt, err := tx.Prepare("SELECT schema, api_extensions FROM nodes WHERE state=0")
 	if err != nil {
-		return nil, err
+		// In order to make cluster updates work, let's check for "pending" as well as that's the column's previous name.
+		stmt, err = tx.Prepare("SELECT schema, api_extensions FROM nodes WHERE pending=0")
+		if err != nil {
+			return nil, err
+		}
 	}
-	defer stmt.Close()
+	defer func() { _ = stmt.Close() }()
 	err = query.SelectObjects(stmt, dest)
 	if err != nil {
 		return nil, err
 	}
+
 	return versions, nil
 }
