@@ -1,33 +1,34 @@
+//go:build linux && cgo && !agent
+
 package db
 
 import (
+	"context"
 	"database/sql"
+	"errors"
 	"fmt"
+	"os"
+	"path/filepath"
+	"regexp"
+	"sort"
+	"strings"
 	"sync"
 	"time"
 
-	"github.com/CanonicalLtd/go-dqlite"
-	"github.com/pkg/errors"
+	"github.com/canonical/go-dqlite/driver"
 
 	"github.com/lxc/lxd/lxd/db/cluster"
 	"github.com/lxc/lxd/lxd/db/node"
 	"github.com/lxc/lxd/lxd/db/query"
+	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
 )
 
-var (
-	// ErrAlreadyDefined hapens when the given entry already exists,
-	// for example a container.
-	ErrAlreadyDefined = fmt.Errorf("The container/snapshot already exists")
-
-	// ErrNoSuchObject is in the case of joins (and probably other) queries,
-	// we don't get back sql.ErrNoRows when no rows are returned, even though we do
-	// on selects without joins. Instead, you can use this error to
-	// propagate up and generate proper 404s to the client when something
-	// isn't found so we don't abuse sql.ErrNoRows any more than we
-	// already do.
-	ErrNoSuchObject = fmt.Errorf("No such object")
-)
+// DB represents access to LXD's global and local databases.
+type DB struct {
+	Node    *Node
+	Cluster *Cluster
+}
 
 // Node mediates access to LXD's data stored in the node-local SQLite database.
 type Node struct {
@@ -40,39 +41,19 @@ type Node struct {
 // The fresh hook parameter is used by the daemon to mark all known patch names
 // as applied when a brand new database is created.
 //
-// The legacyPatches parameter is used as a mean to apply the legacy V10, V11,
-// V15, V29 and V30 non-db updates during the database upgrade sequence, to
-// avoid any change in semantics wrt the old logic (see PR #3322).
-//
-// Return the newly created Node object, and a Dump of the pre-clustering data
-// if we've migrating to a cluster-aware version.
-func OpenNode(dir string, fresh func(*Node) error, legacyPatches map[int]*LegacyPatch) (*Node, *Dump, error) {
-	// When updating the node database schema we'll detect if we're
-	// transitioning to the dqlite-based database and dump all the data
-	// before purging the schema. This data will be then imported by the
-	// daemon into the dqlite database.
-	var dump *Dump
-
+// Return the newly created Node object.
+func OpenNode(dir string, fresh func(*Node) error) (*Node, error) {
 	db, err := node.Open(dir)
 	if err != nil {
-		return nil, nil, err
+		return nil, err
 	}
 
-	legacyHook := legacyPatchHook(db, legacyPatches)
-	hook := func(version int, tx *sql.Tx) error {
-		if version == node.UpdateFromPreClustering {
-			logger.Debug("Loading pre-clustering sqlite data")
-			var err error
-			dump, err = LoadPreClusteringData(tx)
-			if err != nil {
-				return err
-			}
-		}
-		return legacyHook(version, tx)
-	}
-	initial, err := node.EnsureSchema(db, dir, hook)
+	db.SetMaxOpenConns(1)
+	db.SetMaxIdleConns(1)
+
+	initial, err := node.EnsureSchema(db, dir)
 	if err != nil {
-		return nil, nil, err
+		return nil, err
 	}
 
 	node := &Node{
@@ -84,21 +65,17 @@ func OpenNode(dir string, fresh func(*Node) error, legacyPatches map[int]*Legacy
 		if fresh != nil {
 			err := fresh(node)
 			if err != nil {
-				return nil, nil, err
+				return nil, err
 			}
 		}
 	}
 
-	db.SetMaxOpenConns(1)
-	db.SetMaxIdleConns(1)
-
-	return node, dump, nil
+	return node, nil
 }
 
-// ForLegacyPatches is a aid for the hack in initializeDbObject, which sets
-// the db-related Deamon attributes upfront, to be backward compatible with the
-// legacy patches that need to interact with the database.
-func ForLegacyPatches(db *sql.DB) *Node {
+// DirectAccess is a bit of a hack which allows getting a database Node struct from any standard Go sql.DB.
+// This is primarily used to access the "db.bin" read-only copy of the database during startup.
+func DirectAccess(db *sql.DB) *Node {
 	return &Node{db: db}
 }
 
@@ -122,7 +99,7 @@ func (n *Node) Dir() string {
 // node-level database, otherwise they are rolled back.
 func (n *Node) Transaction(f func(*NodeTx) error) error {
 	nodeTx := &NodeTx{}
-	return query.Transaction(n.db, func(tx *sql.Tx) error {
+	return query.Transaction(context.TODO(), n.db, func(ctx context.Context, tx *sql.Tx) error {
 		nodeTx.tx = tx
 		return f(nodeTx)
 	})
@@ -133,17 +110,12 @@ func (n *Node) Close() error {
 	return n.db.Close()
 }
 
-// Begin a new transaction against the local database. Legacy method.
-func (n *Node) Begin() (*sql.Tx, error) {
-	return begin(n.db)
-}
-
 // Cluster mediates access to LXD's data stored in the cluster dqlite database.
 type Cluster struct {
-	db     *sql.DB // Handle to the cluster dqlite database, gated behind gRPC SQL.
-	nodeID int64   // Node ID of this LXD instance.
-	mu     sync.RWMutex
-	stmts  map[int]*sql.Stmt // Prepared statements by code.
+	db         *sql.DB // Handle to the cluster dqlite database, gated behind gRPC SQL.
+	nodeID     int64   // Node ID of this LXD instance.
+	mu         sync.RWMutex
+	closingCtx context.Context
 }
 
 // OpenCluster creates a new Cluster object for interacting with the dqlite
@@ -153,26 +125,33 @@ type Cluster struct {
 // - dialer: Function used to connect to the dqlite backend via gRPC SQL.
 // - address: Network address of this node (or empty string).
 // - dir: Base LXD database directory (e.g. /var/lib/lxd/database)
+// - timeout: Give up trying to open the database after this amount of time.
+// - dump: If not nil, a copy of 2.0 db data, for migrating to 3.0.
 //
 // The address and api parameters will be used to determine if the cluster
 // database matches our version, and possibly trigger a schema update. If the
 // schema update can't be performed right now, because some nodes are still
 // behind, an Upgrading error is returned.
-func OpenCluster(name string, store dqlite.ServerStore, address, dir string, timeout time.Duration, options ...dqlite.DriverOption) (*Cluster, error) {
+// Accepts a closingCtx context argument used to indicate when the daemon is shutting down.
+func OpenCluster(closingCtx context.Context, name string, store driver.NodeStore, address, dir string, timeout time.Duration, dump *Dump, options ...driver.Option) (*Cluster, error) {
 	db, err := cluster.Open(name, store, options...)
 	if err != nil {
-		return nil, errors.Wrap(err, "failed to open database")
+		return nil, fmt.Errorf("Failed to open database: %w", err)
 	}
 
+	db.SetMaxOpenConns(1)
+	db.SetMaxIdleConns(1)
+
 	// Test that the cluster database is operational. We wait up to the
 	// given timeout , in case there's no quorum of nodes online yet.
-	timer := time.After(timeout)
+	connectCtx, connectCancel := context.WithTimeout(closingCtx, timeout)
+	defer connectCancel()
 	for i := 0; ; i++ {
 		// Log initial attempts at debug level, but use warn
 		// level after the 5'th attempt (about 10 seconds).
 		// After the 15'th attempt (about 30 seconds), log
 		// only one attempt every 5.
-		logPriority := 1 // 0 is discard, 1 is Debug, 2 is Warn
+		logPriority := 1 // 0 is discard, 1 is Debug, 2 is Error
 		if i > 5 {
 			logPriority = 2
 			if i > 15 && !((i % 5) == 0) {
@@ -180,77 +159,130 @@ func OpenCluster(name string, store dqlite.ServerStore, address, dir string, tim
 			}
 		}
 
-		err = db.Ping()
-		if err == nil {
-			break
-		}
-
-		cause := errors.Cause(err)
-		if cause != dqlite.ErrNoAvailableLeader {
+		logger.Info("Connecting to global database")
+		pingCtx, pingCancel := context.WithTimeout(connectCtx, time.Second*5)
+		err = db.PingContext(pingCtx)
+		pingCancel()
+		logCtx := logger.Ctx{"err": err, "attempt": i}
+		if err != nil && !errors.Is(err, driver.ErrNoAvailableLeader) {
 			return nil, err
+		} else if err == nil {
+			logger.Info("Connected to global database")
+			break
 		}
 
 		switch logPriority {
 		case 1:
-			logger.Debugf("Failed connecting to global database (attempt %d): %v", i, err)
+			logger.Debug("Failed connecting to global database", logCtx)
 		case 2:
-			logger.Warnf("Failed connecting to global database (attempt %d): %v", i, err)
+			logger.Error("Failed connecting to global database", logCtx)
 		}
 
-		time.Sleep(2 * time.Second)
 		select {
-		case <-timer:
-			return nil, fmt.Errorf("failed to connect to cluster database")
+		case <-connectCtx.Done():
+			return nil, connectCtx.Err()
 		default:
+			time.Sleep(2 * time.Second)
+		}
+	}
+
+	// FIXME: https://github.com/canonical/dqlite/issues/163
+	_, err = db.Exec("PRAGMA cache_size=-50000")
+	if err != nil {
+		return nil, fmt.Errorf("Failed to set page cache size: %w", err)
+	}
+
+	if dump != nil {
+		logger.Infof("Migrating data from local to global database")
+		err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
+			return importPreClusteringData(tx, dump)
+		})
+		if err != nil {
+			// Restore the local sqlite3 backup and wipe the raft
+			// directory, so users can fix problems and retry.
+			path := filepath.Join(dir, "local.db")
+			copyErr := shared.FileCopy(path+".bak", path)
+			if copyErr != nil {
+				// Ignore errors here, there's not much we can do
+				logger.Errorf("Failed to restore local database: %v", copyErr)
+			}
+
+			rmErr := os.RemoveAll(filepath.Join(dir, "global"))
+			if rmErr != nil {
+				// Ignore errors here, there's not much we can do
+				logger.Errorf("Failed to cleanup global database: %v", rmErr)
+			}
+
+			return nil, fmt.Errorf("Failed to migrate data to global database: %w", err)
 		}
 	}
 
 	nodesVersionsMatch, err := cluster.EnsureSchema(db, address, dir)
 	if err != nil {
-		return nil, errors.Wrap(err, "failed to ensure schema")
+		return nil, fmt.Errorf("failed to ensure schema: %w", err)
 	}
 
-	db.SetMaxOpenConns(1)
-	db.SetMaxIdleConns(1)
-
-	stmts, err := cluster.PrepareStmts(db)
-	if err != nil {
-		return nil, errors.Wrap(err, "Failed to prepare statements")
-	}
-
-	cluster := &Cluster{
-		db:    db,
-		stmts: stmts,
-	}
-
-	// Figure out the ID of this node.
-	err = cluster.Transaction(func(tx *ClusterTx) error {
-		nodes, err := tx.Nodes()
-		if err != nil {
-			return errors.Wrap(err, "failed to fetch nodes")
+	if !nodesVersionsMatch {
+		cluster := &Cluster{
+			db:         db,
+			closingCtx: closingCtx,
 		}
+
+		return cluster, ErrSomeNodesAreBehind
+	}
+
+	stmts, err := cluster.PrepareStmts(db, false)
+	if err != nil {
+		return nil, fmt.Errorf("Failed to prepare statements: %w", err)
+	}
+
+	cluster.PreparedStmts = stmts
+
+	clusterDB := &Cluster{
+		db:         db,
+		closingCtx: closingCtx,
+	}
+
+	err = clusterDB.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
+		// Figure out the ID of this node.
+		nodes, err := tx.GetNodes()
+		if err != nil {
+			return fmt.Errorf("Failed to fetch nodes: %w", err)
+		}
+
+		nodeID := int64(-1)
 		if len(nodes) == 1 && nodes[0].Address == "0.0.0.0" {
 			// We're not clustered
-			cluster.NodeID(1)
-			return nil
-		}
-		for _, node := range nodes {
-			if node.Address == address {
-				cluster.nodeID = node.ID
-				return nil
+			nodeID = 1
+		} else {
+			for _, node := range nodes {
+				if node.Address == address {
+					nodeID = node.ID
+					break
+				}
 			}
 		}
-		return fmt.Errorf("no node registered with address %s", address)
+
+		if nodeID < 0 {
+			return fmt.Errorf("No node registered with address %s", address)
+		}
+
+		// Set the local member ID
+		clusterDB.NodeID(nodeID)
+
+		// Delete any operation tied to this member
+		err = cluster.DeleteOperations(ctx, tx.tx, nodeID)
+		if err != nil {
+			return err
+		}
+
+		return nil
 	})
 	if err != nil {
 		return nil, err
 	}
 
-	if !nodesVersionsMatch {
-		err = ErrSomeNodesAreBehind
-	}
-
-	return cluster, err
+	return clusterDB, err
 }
 
 // ErrSomeNodesAreBehind is returned by OpenCluster if some of the nodes in the
@@ -261,13 +293,30 @@ var ErrSomeNodesAreBehind = fmt.Errorf("some nodes are behind this node's versio
 // sets the db-related Deamon attributes upfront, to be backward compatible
 // with the legacy patches that need to interact with the database.
 func ForLocalInspection(db *sql.DB) *Cluster {
-	return &Cluster{db: db}
+	return &Cluster{
+		db:         db,
+		closingCtx: context.Background(),
+	}
 }
 
-// SetDefaultTimeout sets the default go-dqlite driver timeout.
-func (c *Cluster) SetDefaultTimeout(timeout time.Duration) {
-	driver := c.db.Driver().(*dqlite.Driver)
-	driver.SetContextTimeout(timeout)
+// ForLocalInspectionWithPreparedStmts is the same as ForLocalInspection but it
+// also prepares the statements used in auto-generated database code.
+func ForLocalInspectionWithPreparedStmts(db *sql.DB) (*Cluster, error) {
+	c := ForLocalInspection(db)
+
+	stmts, err := cluster.PrepareStmts(c.db, true)
+	if err != nil {
+		return nil, fmt.Errorf("Prepare database statements: %w", err)
+	}
+
+	cluster.PreparedStmts = stmts
+
+	return c, nil
+}
+
+// GetNodeID returns the current nodeID (0 if not set).
+func (c *Cluster) GetNodeID() int64 {
+	return c.nodeID
 }
 
 // Transaction creates a new ClusterTx object and transactionally executes the
@@ -277,10 +326,10 @@ func (c *Cluster) SetDefaultTimeout(timeout time.Duration) {
 //
 // If EnterExclusive has been called before, calling Transaction will block
 // until ExitExclusive has been called as well to release the lock.
-func (c *Cluster) Transaction(f func(*ClusterTx) error) error {
+func (c *Cluster) Transaction(ctx context.Context, f func(context.Context, *ClusterTx) error) error {
 	c.mu.RLock()
 	defer c.mu.RUnlock()
-	return c.transaction(f)
+	return c.transaction(ctx, f)
 }
 
 // EnterExclusive acquires a lock on the cluster db, so any successive call to
@@ -305,26 +354,44 @@ func (c *Cluster) EnterExclusive() error {
 
 // ExitExclusive runs the given transaction and then releases the lock acquired
 // with EnterExclusive.
-func (c *Cluster) ExitExclusive(f func(*ClusterTx) error) error {
+func (c *Cluster) ExitExclusive(ctx context.Context, f func(context.Context, *ClusterTx) error) error {
 	logger.Debug("Releasing exclusive lock on cluster db")
 	defer c.mu.Unlock()
-	return c.transaction(f)
+	return c.transaction(ctx, f)
 }
 
-func (c *Cluster) transaction(f func(*ClusterTx) error) error {
+func (c *Cluster) transaction(ctx context.Context, f func(context.Context, *ClusterTx) error) error {
 	clusterTx := &ClusterTx{
 		nodeID: c.nodeID,
-		stmts:  c.stmts,
 	}
 
-	return query.Retry(func() error {
-		return query.Transaction(c.db, func(tx *sql.Tx) error {
+	return c.retry(func() error {
+		txFunc := func(ctx context.Context, tx *sql.Tx) error {
 			clusterTx.tx = tx
-			return f(clusterTx)
-		})
+			return f(ctx, clusterTx)
+		}
+
+		err := query.Transaction(ctx, c.db, txFunc)
+		if errors.Is(err, context.DeadlineExceeded) {
+			// If the query timed out it likely means that the leader has abruptly become unreachable.
+			// Now that this query has been cancelled, a leader election should have taken place by now.
+			// So let's retry the transaction once more in case the global database is now available again.
+			logger.Warn("Transaction timed out. Retrying once", logger.Ctx{"member": c.nodeID, "err": err})
+			return query.Transaction(ctx, c.db, txFunc)
+		}
+
+		return err
 	})
 }
 
+func (c *Cluster) retry(f func() error) error {
+	if c.closingCtx.Err() != nil {
+		return f()
+	}
+
+	return query.Retry(f)
+}
+
 // NodeID sets the the node NodeID associated with this cluster instance. It's used for
 // backward-compatibility of all db-related APIs that were written before
 // clustering and don't accept a node NodeID, so in those cases we automatically
@@ -335,9 +402,10 @@ func (c *Cluster) NodeID(id int64) {
 
 // Close the database facade.
 func (c *Cluster) Close() error {
-	for _, stmt := range c.stmts {
-		stmt.Close()
+	for _, stmt := range cluster.PreparedStmts {
+		_ = stmt.Close()
 	}
+
 	return c.db.Close()
 }
 
@@ -356,41 +424,18 @@ func (c *Cluster) Begin() (*sql.Tx, error) {
 	return begin(c.db)
 }
 
-// UpdateSchemasDotGo updates the schema.go files in the local/ and cluster/
-// sub-packages.
-func UpdateSchemasDotGo() error {
-	err := node.SchemaDotGo()
-	if err != nil {
-		return fmt.Errorf("failed to update node schema.go: %v", err)
-	}
-	err = cluster.SchemaDotGo()
-	if err != nil {
-		return fmt.Errorf("failed to update cluster schema.go: %v", err)
-	}
-
-	return nil
-}
-
-func isNoMatchError(err error) bool {
-	if err == nil {
-		return false
-	}
-	if err.Error() == "sql: no rows in result set" {
-		return true
-	}
-	return false
-}
-
 func begin(db *sql.DB) (*sql.Tx, error) {
 	for i := 0; i < 1000; i++ {
 		tx, err := db.Begin()
 		if err == nil {
 			return tx, nil
 		}
+
 		if !query.IsRetriableError(err) {
 			logger.Debugf("DbBegin: error %q", err)
 			return nil, err
 		}
+
 		time.Sleep(30 * time.Millisecond)
 	}
 
@@ -405,30 +450,75 @@ func TxCommit(tx *sql.Tx) error {
 	if err == nil || err == sql.ErrTxDone { // Ignore duplicate commits/rollbacks
 		return nil
 	}
+
 	return err
 }
 
-func dbQueryRowScan(db *sql.DB, q string, args []interface{}, outargs []interface{}) error {
-	return query.Retry(func() error {
-		return query.Transaction(db, func(tx *sql.Tx) error {
+// DqliteLatestSegment returns the latest segment ID in the global database.
+func DqliteLatestSegment() (string, error) {
+	dir := shared.VarPath("database", "global")
+	file, err := os.Open(dir)
+	if err != nil {
+		return "", fmt.Errorf("Unable to open directory %s with error %v", dir, err)
+	}
+
+	defer func() { _ = file.Close() }()
+
+	fileNames, err := file.Readdirnames(0)
+	if err != nil {
+		return "", fmt.Errorf("Unable to read file names in directory %s with error %v", dir, err)
+	}
+
+	if len(fileNames) == 0 {
+		return "none", nil
+	}
+
+	sort.Strings(fileNames)
+
+	r, _ := regexp.Compile("^[0-9]*-[0-9]*$")
+
+	for i := range fileNames {
+		fileName := fileNames[len(fileNames)-1-i]
+		if r.MatchString(fileName) {
+			segment := strings.Split(fileName, "-")[1]
+			// Trim leading o's.
+			index := 0
+			for i, c := range segment {
+				index = i
+				if c != '0' {
+					break
+				}
+			}
+
+			return segment[index:], nil
+		}
+	}
+
+	return "none", nil
+}
+
+func dbQueryRowScan(c *Cluster, q string, args []any, outargs []any) error {
+	return c.retry(func() error {
+		return query.Transaction(context.TODO(), c.db, func(ctx context.Context, tx *sql.Tx) error {
 			return tx.QueryRow(q, args...).Scan(outargs...)
 		})
 	})
 }
 
-func doDbQueryScan(db *sql.DB, q string, args []interface{}, outargs []interface{}) ([][]interface{}, error) {
-	result := [][]interface{}{}
+func doDbQueryScan(c *Cluster, q string, args []any, outargs []any) ([][]any, error) {
+	result := [][]any{}
 
-	err := query.Retry(func() error {
-		return query.Transaction(db, func(tx *sql.Tx) error {
+	err := c.retry(func() error {
+		return query.Transaction(context.TODO(), c.db, func(ctx context.Context, tx *sql.Tx) error {
 			rows, err := tx.Query(q, args...)
 			if err != nil {
 				return err
 			}
-			defer rows.Close()
+
+			defer func() { _ = rows.Close() }()
 
 			for rows.Next() {
-				ptrargs := make([]interface{}, len(outargs))
+				ptrargs := make([]any, len(outargs))
 				for i := range outargs {
 					switch t := outargs[i].(type) {
 					case string:
@@ -440,6 +530,9 @@ func doDbQueryScan(db *sql.DB, q string, args []interface{}, outargs []interface
 					case int64:
 						integer := int64(0)
 						ptrargs[i] = &integer
+					case bool:
+						boolean := bool(false)
+						ptrargs[i] = &boolean
 					default:
 						return fmt.Errorf("Bad interface type: %s", t)
 					}
@@ -448,7 +541,8 @@ func doDbQueryScan(db *sql.DB, q string, args []interface{}, outargs []interface
 				if err != nil {
 					return err
 				}
-				newargs := make([]interface{}, len(outargs))
+
+				newargs := make([]any, len(outargs))
 				for i := range ptrargs {
 					switch t := outargs[i].(type) {
 					case string:
@@ -457,25 +551,28 @@ func doDbQueryScan(db *sql.DB, q string, args []interface{}, outargs []interface
 						newargs[i] = *ptrargs[i].(*int)
 					case int64:
 						newargs[i] = *ptrargs[i].(*int64)
+					case bool:
+						newargs[i] = *ptrargs[i].(*bool)
 					default:
 						return fmt.Errorf("Bad interface type: %s", t)
 					}
 				}
 				result = append(result, newargs)
 			}
+
 			err = rows.Err()
 			if err != nil {
 				return err
 			}
+
 			return nil
 		})
 	})
 	if err != nil {
-		return [][]interface{}{}, err
+		return [][]any{}, err
 	}
 
 	return result, nil
-
 }
 
 /*
@@ -486,18 +583,18 @@ func doDbQueryScan(db *sql.DB, q string, args []interface{}, outargs []interface
  *   arguments, i.e.
  *      var arg1 string
  *      var arg2 int
- *      outfmt := {}interface{}{arg1, arg2}
+ *      outfmt := {}any{arg1, arg2}
  *
  * The result will be an array (one per output row) of arrays (one per output argument)
  * of interfaces, containing pointers to the actual output arguments.
  */
-func queryScan(db *sql.DB, q string, inargs []interface{}, outfmt []interface{}) ([][]interface{}, error) {
-	return doDbQueryScan(db, q, inargs, outfmt)
+func queryScan(c *Cluster, q string, inargs []any, outfmt []any) ([][]any, error) {
+	return doDbQueryScan(c, q, inargs, outfmt)
 }
 
-func exec(db *sql.DB, q string, args ...interface{}) error {
-	err := query.Retry(func() error {
-		return query.Transaction(db, func(tx *sql.Tx) error {
+func exec(c *Cluster, q string, args ...any) error {
+	err := c.retry(func() error {
+		return query.Transaction(context.TODO(), c.db, func(ctx context.Context, tx *sql.Tx) error {
 			_, err := tx.Exec(q, args...)
 			return err
 		})
