@@ -1,13 +1,22 @@
+//go:build linux && cgo && !agent
+
 package db_test
 
 import (
+	"context"
 	"database/sql"
+	"net"
 	"testing"
+	"time"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/query"
+	"github.com/canonical/go-dqlite/driver"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/db/cluster"
+	"github.com/lxc/lxd/lxd/db/query"
+	"github.com/lxc/lxd/lxd/project"
 )
 
 func TestLoadPreClusteringData(t *testing.T) {
@@ -19,17 +28,18 @@ func TestLoadPreClusteringData(t *testing.T) {
 	// config
 	assert.Equal(t, []string{"id", "key", "value"}, dump.Schema["config"])
 	assert.Len(t, dump.Data["config"], 3)
-	rows := []interface{}{int64(1), []byte("core.https_address"), []byte("1.2.3.4:666")}
+	assert.Equal(t, "1.2.3.4:666", dump.Data["config"][0][2])
+	rows := []any{int64(1), "core.https_address", "1.2.3.4:666"}
 	assert.Equal(t, rows, dump.Data["config"][0])
-	rows = []interface{}{int64(2), []byte("core.trust_password"), []byte("sekret")}
+	rows = []any{int64(2), "core.trust_password", "sekret"}
 	assert.Equal(t, rows, dump.Data["config"][1])
-	rows = []interface{}{int64(3), []byte("maas.machine"), []byte("mymaas")}
+	rows = []any{int64(3), "maas.machine", "mymaas"}
 	assert.Equal(t, rows, dump.Data["config"][2])
 
 	// networks
 	assert.Equal(t, []string{"id", "name", "description"}, dump.Schema["networks"])
 	assert.Len(t, dump.Data["networks"], 1)
-	rows = []interface{}{int64(1), []byte("lxcbr0"), []byte("LXD bridge")}
+	rows = []any{int64(1), "lxcbr0", "LXD bridge"}
 	assert.Equal(t, rows, dump.Data["networks"][0])
 }
 
@@ -39,25 +49,34 @@ func TestImportPreClusteringData(t *testing.T) {
 	dump, err := db.LoadPreClusteringData(tx)
 	require.NoError(t, err)
 
-	cluster, cleanup := db.NewTestCluster(t)
+	dir, store, cleanup := db.NewTestDqliteServer(t)
 	defer cleanup()
 
-	err = cluster.ImportPreClusteringData(dump)
+	dial := func(ctx context.Context, address string) (net.Conn, error) {
+		return net.Dial("unix", address)
+	}
+
+	c, err := db.OpenCluster(context.Background(), "test.db", store, "1", dir, 5*time.Second, dump, driver.WithDialFunc(dial))
 	require.NoError(t, err)
+	defer func() { _ = c.Close() }()
 
 	// certificates
-	certs, err := cluster.CertificatesGet()
+	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		certs, err := cluster.GetCertificates(context.Background(), tx.Tx(), cluster.CertificateFilter{})
+		require.NoError(t, err)
+		assert.Len(t, certs, 1)
+		cert := certs[0]
+		assert.Equal(t, 1, cert.ID)
+		assert.Equal(t, "abcd:efgh", cert.Fingerprint)
+		assert.Equal(t, cluster.CertificateTypeClient, cert.Type)
+		assert.Equal(t, "foo", cert.Name)
+		assert.Equal(t, "FOO", cert.Certificate)
+		return nil
+	})
 	require.NoError(t, err)
-	assert.Len(t, certs, 1)
-	cert := certs[0]
-	assert.Equal(t, 1, cert.ID)
-	assert.Equal(t, "abcd:efgh", cert.Fingerprint)
-	assert.Equal(t, 1, cert.Type)
-	assert.Equal(t, "foo", cert.Name)
-	assert.Equal(t, "FOO", cert.Certificate)
 
 	// config
-	err = cluster.Transaction(func(tx *db.ClusterTx) error {
+	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		config, err := tx.Config()
 		require.NoError(t, err)
 		values := map[string]string{"core.trust_password": "sekret"}
@@ -67,10 +86,10 @@ func TestImportPreClusteringData(t *testing.T) {
 	require.NoError(t, err)
 
 	// networks
-	networks, err := cluster.Networks()
+	networks, err := c.GetNetworks(project.Default)
 	require.NoError(t, err)
 	assert.Equal(t, []string{"lxcbr0"}, networks)
-	id, network, err := cluster.NetworkGet("lxcbr0")
+	id, network, _, err := c.GetNetworkInAnyState(project.Default, "lxcbr0")
 	require.NoError(t, err)
 	assert.Equal(t, int64(1), id)
 	assert.Equal(t, "true", network.Config["ipv4.nat"])
@@ -78,10 +97,10 @@ func TestImportPreClusteringData(t *testing.T) {
 	assert.Equal(t, []string{"none"}, network.Locations)
 
 	// storage
-	pools, err := cluster.StoragePools()
+	pools, err := c.GetStoragePoolNames()
 	require.NoError(t, err)
 	assert.Equal(t, []string{"default"}, pools)
-	id, pool, err := cluster.StoragePoolGet("default")
+	id, pool, _, err := c.GetStoragePool("default")
 	require.NoError(t, err)
 	assert.Equal(t, int64(1), id)
 	assert.Equal(t, "/foo/bar", pool.Config["source"])
@@ -91,12 +110,12 @@ func TestImportPreClusteringData(t *testing.T) {
 	assert.Equal(t, "true", pool.Config["zfs.clone_copy"])
 	assert.Equal(t, "Created", pool.Status)
 	assert.Equal(t, []string{"none"}, pool.Locations)
-	volumes, err := cluster.StoragePoolNodeVolumesGet(id, []int{1})
+	volumes, err := c.GetLocalStoragePoolVolumes("default", id, []int{1})
 	require.NoError(t, err)
 	assert.Len(t, volumes, 1)
 	assert.Equal(t, "/foo/bar", volumes[0].Config["source"])
 
-	err = cluster.Transaction(func(tx *db.ClusterTx) error {
+	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// The zfs.clone_copy config got a NULL node_id, since it's cluster global.
 		config, err := query.SelectConfig(tx.Tx(), "storage_pools_config", "node_id IS NULL")
 		require.NoError(t, err)
@@ -118,10 +137,10 @@ func TestImportPreClusteringData(t *testing.T) {
 	require.NoError(t, err)
 
 	// profiles
-	profiles, err := cluster.Profiles()
+	profiles, err := c.GetProfileNames("default")
 	require.NoError(t, err)
 	assert.Equal(t, []string{"default", "users"}, profiles)
-	_, profile, err := cluster.ProfileGet("default")
+	_, profile, err := c.GetProfile("default", "default")
 	require.NoError(t, err)
 	assert.Equal(t, map[string]string{}, profile.Config)
 	assert.Equal(t,
@@ -135,7 +154,7 @@ func TestImportPreClusteringData(t *testing.T) {
 				"nictype": "bridged",
 				"parent":  "lxdbr0"}},
 		profile.Devices)
-	_, profile, err = cluster.ProfileGet("users")
+	_, profile, err = c.GetProfile("default", "users")
 	require.NoError(t, err)
 	assert.Equal(t,
 		map[string]string{
@@ -182,10 +201,12 @@ func newPreClusteringTx(t *testing.T) *sql.Tx {
 		"INSERT INTO storage_volumes VALUES (1, 'dev', 1, 1, '')",
 		"INSERT INTO storage_volumes_config VALUES(1, 1, 'source', '/foo/bar')",
 	}
+
 	for _, stmt := range stmts {
 		_, err := tx.Exec(stmt)
 		require.NoError(t, err)
 	}
+
 	return tx
 }
 
