@@ -1,19 +1,25 @@
+//go:build linux && cgo && !agent
+
 package db_test
 
 import (
+	"context"
 	"testing"
 
-	"github.com/lxc/lxd/lxd/db"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/project"
+	"github.com/lxc/lxd/lxd/response"
 )
 
-// The NetworksNodeConfigs method returns only node-specific config values.
-func TestNetworksNodeConfigs(t *testing.T) {
+// The GetNetworksLocalConfigs method returns only node-specific config values.
+func TestGetNetworksLocalConfigs(t *testing.T) {
 	cluster, cleanup := db.NewTestCluster(t)
 	defer cleanup()
 
-	_, err := cluster.NetworkCreate("lxdbr0", "", map[string]string{
+	_, err := cluster.CreateNetwork(project.Default, "lxdbr0", "", db.NetworkTypeBridge, map[string]string{
 		"dns.mode":                   "none",
 		"bridge.external_interfaces": "vlan0",
 	})
@@ -21,9 +27,9 @@ func TestNetworksNodeConfigs(t *testing.T) {
 
 	var config map[string]map[string]string
 
-	err = cluster.Transaction(func(tx *db.ClusterTx) error {
+	err = cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
-		config, err = tx.NetworksNodeConfig()
+		config, err = tx.GetNetworksLocalConfig()
 		return err
 	})
 	require.NoError(t, err)
@@ -33,25 +39,25 @@ func TestNetworksNodeConfigs(t *testing.T) {
 	})
 }
 
-func TestNetworkCreatePending(t *testing.T) {
+func TestCreatePendingNetwork(t *testing.T) {
 	tx, cleanup := db.NewTestClusterTx(t)
 	defer cleanup()
 
-	_, err := tx.NodeAdd("buzz", "1.2.3.4:666")
+	_, err := tx.CreateNode("buzz", "1.2.3.4:666")
 	require.NoError(t, err)
-	_, err = tx.NodeAdd("rusp", "5.6.7.8:666")
+	_, err = tx.CreateNode("rusp", "5.6.7.8:666")
 	require.NoError(t, err)
 
 	config := map[string]string{"bridge.external_interfaces": "foo"}
-	err = tx.NetworkCreatePending("buzz", "network1", config)
+	err = tx.CreatePendingNetwork("buzz", project.Default, "network1", db.NetworkTypeBridge, config)
 	require.NoError(t, err)
 
-	networkID, err := tx.NetworkID("network1")
+	networkID, err := tx.GetNetworkID(project.Default, "network1")
 	require.NoError(t, err)
 	assert.True(t, networkID > 0)
 
 	config = map[string]string{"bridge.external_interfaces": "bar"}
-	err = tx.NetworkCreatePending("rusp", "network1", config)
+	err = tx.CreatePendingNetwork("rusp", project.Default, "network1", db.NetworkTypeBridge, config)
 	require.NoError(t, err)
 
 	// The initial node (whose name is 'none' by default) is missing.
@@ -59,7 +65,7 @@ func TestNetworkCreatePending(t *testing.T) {
 	require.EqualError(t, err, "Network not defined on nodes: none")
 
 	config = map[string]string{"bridge.external_interfaces": "egg"}
-	err = tx.NetworkCreatePending("none", "network1", config)
+	err = tx.CreatePendingNetwork("none", project.Default, "network1", db.NetworkTypeBridge, config)
 	require.NoError(t, err)
 
 	// Now the storage is defined on all nodes.
@@ -77,13 +83,13 @@ func TestNetworksCreatePending_AlreadyDefined(t *testing.T) {
 	tx, cleanup := db.NewTestClusterTx(t)
 	defer cleanup()
 
-	_, err := tx.NodeAdd("buzz", "1.2.3.4:666")
+	_, err := tx.CreateNode("buzz", "1.2.3.4:666")
 	require.NoError(t, err)
 
-	err = tx.NetworkCreatePending("buzz", "network1", map[string]string{})
+	err = tx.CreatePendingNetwork("buzz", project.Default, "network1", db.NetworkTypeBridge, map[string]string{})
 	require.NoError(t, err)
 
-	err = tx.NetworkCreatePending("buzz", "network1", map[string]string{})
+	err = tx.CreatePendingNetwork("buzz", project.Default, "network1", db.NetworkTypeBridge, map[string]string{})
 	require.Equal(t, db.ErrAlreadyDefined, err)
 }
 
@@ -92,6 +98,6 @@ func TestNetworksCreatePending_NonExistingNode(t *testing.T) {
 	tx, cleanup := db.NewTestClusterTx(t)
 	defer cleanup()
 
-	err := tx.NetworkCreatePending("buzz", "network1", map[string]string{})
-	require.Equal(t, db.ErrNoSuchObject, err)
+	err := tx.CreatePendingNetwork("buzz", project.Default, "network1", db.NetworkTypeBridge, map[string]string{})
+	require.True(t, response.IsNotFoundError(err))
 }
