@@ -5,7 +5,6 @@ import (
 	"fmt"
 	"path/filepath"
 
-	"github.com/lxc/lxd/lxd/db/schema"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
 )
@@ -15,7 +14,7 @@ func Open(dir string) (*sql.DB, error) {
 	path := filepath.Join(dir, "local.db")
 	db, err := sqliteOpen(path)
 	if err != nil {
-		return nil, fmt.Errorf("cannot open node database: %v", err)
+		return nil, fmt.Errorf("cannot open node database: %w", err)
 	}
 
 	return db, nil
@@ -26,7 +25,7 @@ func Open(dir string) (*sql.DB, error) {
 //
 // Return the initial schema version found before starting the update, along
 // with any error occurred.
-func EnsureSchema(db *sql.DB, dir string, hook schema.Hook) (int, error) {
+func EnsureSchema(db *sql.DB, dir string) (int, error) {
 	backupDone := false
 
 	schema := Schema()
@@ -49,14 +48,6 @@ func EnsureSchema(db *sql.DB, dir string, hook schema.Hook) (int, error) {
 			logger.Debugf("Updating DB schema from %d to %d", version, version+1)
 		}
 
-		// Run the given hook only against actual update versions, not
-		// when a custom query file is passed (signaled by version == -1).
-		if hook != nil && version != -1 {
-			err := hook(version, tx)
-			if err != nil {
-			}
-		}
-
 		return nil
 	})
 	return schema.Ensure(db)
