@@ -8,9 +8,11 @@ import (
 	"strconv"
 	"strings"
 
+	"github.com/lxc/lxd/lxd/db/query"
 	"github.com/lxc/lxd/lxd/db/schema"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/units"
 )
 
 // Schema for the local database.
@@ -20,6 +22,11 @@ func Schema() *schema.Schema {
 	return schema
 }
 
+// FreshSchema returns the fresh schema definition of the local database.
+func FreshSchema() string {
+	return freshSchema
+}
+
 // SchemaDotGo refreshes the schema.go file in this package, using the updates
 // defined here.
 func SchemaDotGo() error {
@@ -85,6 +92,11 @@ var updates = map[int]schema.Update{
 	35: updateFromV34,
 	36: updateFromV35,
 	37: updateFromV36,
+	38: updateFromV37,
+	39: updateFromV38,
+	40: updateFromV39,
+	41: updateFromV40,
+	42: updateFromV41,
 }
 
 // UpdateFromPreClustering is the last schema version where clustering support
@@ -93,6 +105,116 @@ const UpdateFromPreClustering = 36
 
 // Schema updates begin here
 
+func updateFromV41(tx *sql.Tx) error {
+	stmt := `
+	ALTER TABLE raft_nodes ADD COLUMN name TEXT NOT NULL default "";
+	`
+	_, err := tx.Exec(stmt)
+	return err
+}
+
+func updateFromV40(tx *sql.Tx) error {
+	stmt := `
+CREATE TABLE certificates (
+	id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+	fingerprint TEXT NOT NULL,
+	type INTEGER NOT NULL,
+	name TEXT NOT NULL,
+	certificate TEXT NOT NULL,
+	UNIQUE (fingerprint)
+);
+`
+	_, err := tx.Exec(stmt)
+	return err
+}
+
+// Fix the address of the bootstrap node being set to "0" in the raft_nodes
+// table.
+func updateFromV39(tx *sql.Tx) error {
+	nodes := []struct {
+		ID      uint64
+		Address string
+	}{}
+	dest := func(i int) []any {
+		nodes = append(nodes, struct {
+			ID      uint64
+			Address string
+		}{})
+		return []any{&nodes[i].ID, &nodes[i].Address}
+	}
+
+	stmt, err := tx.Prepare("SELECT id, address FROM raft_nodes")
+	if err != nil {
+		return err
+	}
+
+	defer func() { _ = stmt.Close() }()
+
+	err = query.SelectObjects(stmt, dest)
+	if err != nil {
+		return fmt.Errorf("Failed to fetch raft nodes: %w", err)
+	}
+
+	if len(nodes) != 1 {
+		return nil
+	}
+
+	info := nodes[0]
+	if info.ID != 1 || info.Address != "0" {
+		return nil
+	}
+
+	config, err := query.SelectConfig(tx, "config", "")
+	if err != nil {
+		return err
+	}
+
+	address := config["cluster.https_address"]
+	if address != "" {
+		_, err := tx.Exec("UPDATE raft_nodes SET address=? WHERE id=1", address)
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+// Add role column to raft_nodes table. All existing entries will have role "0"
+// which means voter.
+func updateFromV38(tx *sql.Tx) error {
+	stmt := `
+ALTER TABLE raft_nodes ADD COLUMN role INTEGER NOT NULL DEFAULT 0;
+`
+	_, err := tx.Exec(stmt)
+	return err
+}
+
+// Copy core.https_address to cluster.https_address in case this node is
+// clustered.
+func updateFromV37(tx *sql.Tx) error {
+	count, err := query.Count(tx, "raft_nodes", "")
+	if err != nil {
+		return fmt.Errorf("Fetch count of Raft nodes: %w", err)
+	}
+
+	if count == 0 {
+		// This node is not clustered, nothing to do.
+		return nil
+	}
+
+	// Copy the core.https_address config.
+	_, err = tx.Exec(`
+INSERT INTO config (key, value)
+  SELECT 'cluster.https_address', value FROM config WHERE key = 'core.https_address'
+`)
+	if err != nil {
+		return fmt.Errorf("Insert cluster.https_address config: %w", err)
+	}
+
+	return nil
+}
+
 // Add a raft_nodes table to be used when running in clustered mode. It lists
 // the current nodes in the LXD cluster that are participating to the dqlite
 // database Raft cluster.
@@ -262,7 +384,7 @@ func updateFromV28(tx *sql.Tx) error {
 INSERT INTO profiles_devices (profile_id, name, type) SELECT id, "aadisable", 2 FROM profiles WHERE name="docker";
 INSERT INTO profiles_devices_config (profile_device_id, key, value) SELECT profiles_devices.id, "source", "/dev/null" FROM profiles_devices LEFT JOIN profiles WHERE profiles_devices.profile_id = profiles.id AND profiles.name = "docker" AND profiles_devices.name = "aadisable";
 INSERT INTO profiles_devices_config (profile_device_id, key, value) SELECT profiles_devices.id, "path", "/sys/module/apparmor/parameters/enabled" FROM profiles_devices LEFT JOIN profiles WHERE profiles_devices.profile_id = profiles.id AND profiles.name = "docker" AND profiles_devices.name = "aadisable";`
-	tx.Exec(stmt)
+	_, _ = tx.Exec(stmt)
 
 	return nil
 }
@@ -295,7 +417,7 @@ INSERT INTO profiles_config (profile_id, key, value) SELECT id, "security.nestin
 INSERT INTO profiles_config (profile_id, key, value) SELECT id, "linux.kernel_modules", "overlay, nf_nat" FROM profiles WHERE name="docker";
 INSERT INTO profiles_devices (profile_id, name, type) SELECT id, "fuse", "unix-char" FROM profiles WHERE name="docker";
 INSERT INTO profiles_devices_config (profile_device_id, key, value) SELECT profiles_devices.id, "path", "/dev/fuse" FROM profiles_devices LEFT JOIN profiles WHERE profiles_devices.profile_id = profiles.id AND profiles.name = "docker";`
-	tx.Exec(stmt)
+	_, _ = tx.Exec(stmt)
 
 	return nil
 }
@@ -356,7 +478,8 @@ func updateFromV18(tx *sql.Tx) error {
 	if err != nil {
 		return err
 	}
-	defer rows.Close()
+
+	defer func() { _ = rows.Close() }()
 
 	for rows.Next() {
 		err := rows.Scan(&id, &value)
@@ -375,9 +498,9 @@ func updateFromV18(tx *sql.Tx) error {
 		value += "B"
 
 		// Deal with completely broken values
-		_, err = shared.ParseByteSizeString(value)
+		_, err = units.ParseByteSizeString(value)
 		if err != nil {
-			logger.Debugf("Invalid container memory limit, id=%d value=%s, removing.", id, value)
+			logger.Debugf("Invalid container memory limit, id=%d value=%s, removing", id, value)
 			_, err = tx.Exec("DELETE FROM containers_config WHERE id=?;", id)
 			if err != nil {
 				return err
@@ -401,7 +524,8 @@ func updateFromV18(tx *sql.Tx) error {
 	if err != nil {
 		return err
 	}
-	defer rows.Close()
+
+	defer func() { _ = rows.Close() }()
 
 	for rows.Next() {
 		err := rows.Scan(&id, &value)
@@ -420,9 +544,9 @@ func updateFromV18(tx *sql.Tx) error {
 		value += "B"
 
 		// Deal with completely broken values
-		_, err = shared.ParseByteSizeString(value)
+		_, err = units.ParseByteSizeString(value)
 		if err != nil {
-			logger.Debugf("Invalid profile memory limit, id=%d value=%s, removing.", id, value)
+			logger.Debugf("Invalid profile memory limit, id=%d value=%s, removing", id, value)
 			_, err = tx.Exec("DELETE FROM profiles_config WHERE id=?;", id)
 			if err != nil {
 				return err
@@ -721,21 +845,25 @@ PRAGMA foreign_keys=ON; -- Make sure we turn integrity checks back on.`
 		return err
 	}
 
+	defer func() { _ = rows.Close() }()
+
 	var tablestodelete []string
 	var rowidtodelete []int
 
-	defer rows.Close()
 	for rows.Next() {
 		var tablename string
 		var rowid int
 		var targetname string
 		var keynumber int
 
-		rows.Scan(&tablename, &rowid, &targetname, &keynumber)
+		err := rows.Scan(&tablename, &rowid, &targetname, &keynumber)
+		if err != nil {
+			return err
+		}
+
 		tablestodelete = append(tablestodelete, tablename)
 		rowidtodelete = append(rowidtodelete, rowid)
 	}
-	rows.Close()
 
 	for i := range tablestodelete {
 		_, err = tx.Exec(fmt.Sprintf("DELETE FROM %s WHERE rowid = %d;", tablestodelete[i], rowidtodelete[i]))
@@ -744,7 +872,7 @@ PRAGMA foreign_keys=ON; -- Make sure we turn integrity checks back on.`
 		}
 	}
 
-	return err
+	return nil
 }
 
 func updateFromV5(tx *sql.Tx) error {
@@ -773,7 +901,7 @@ CREATE TABLE IF NOT EXISTS config (
 	passOut, err := os.Open(passfname)
 	oldPassword := ""
 	if err == nil {
-		defer passOut.Close()
+		defer func() { _ = passOut.Close() }()
 		buff := make([]byte, 96)
 		_, err = passOut.Read(buff)
 		if err != nil {
@@ -796,7 +924,7 @@ CREATE TABLE IF NOT EXISTS config (
 
 func updateFromV3(tx *sql.Tx) error {
 	// Attempt to create a default profile (but don't fail if already there)
-	tx.Exec("INSERT INTO profiles (name) VALUES (\"default\");")
+	_, _ = tx.Exec("INSERT INTO profiles (name) VALUES (\"default\");")
 
 	return nil
 }
