@@ -4,8 +4,6 @@ import (
 	"database/sql"
 	"fmt"
 	"strings"
-
-	"github.com/pkg/errors"
 )
 
 // SelectConfig executes a query statement against a "config" table, which must
@@ -13,7 +11,7 @@ import (
 // additional WHERE filters can be specified.
 //
 // Returns a map of key names to their associated values.
-func SelectConfig(tx *sql.Tx, table string, where string, args ...interface{}) (map[string]string, error) {
+func SelectConfig(tx *sql.Tx, table string, where string, args ...any) (map[string]string, error) {
 	query := fmt.Sprintf("SELECT key, value FROM %s", table)
 	if where != "" {
 		query += fmt.Sprintf(" WHERE %s", where)
@@ -23,7 +21,8 @@ func SelectConfig(tx *sql.Tx, table string, where string, args ...interface{}) (
 	if err != nil {
 		return nil, err
 	}
-	defer rows.Close()
+
+	defer func() { _ = rows.Close() }()
 
 	values := map[string]string{}
 	for rows.Next() {
@@ -57,17 +56,18 @@ func UpdateConfig(tx *sql.Tx, table string, values map[string]string) error {
 			deletes = append(deletes, key)
 			continue
 		}
+
 		changes[key] = value
 	}
 
 	err := upsertConfig(tx, table, changes)
 	if err != nil {
-		return errors.Wrap(err, "updating values failed")
+		return fmt.Errorf("updating values failed: %w", err)
 	}
 
 	err = deleteConfig(tx, table, deletes)
 	if err != nil {
-		return errors.Wrap(err, "deleting values failed")
+		return fmt.Errorf("deleting values failed: %w", err)
 	}
 
 	return nil
@@ -81,12 +81,13 @@ func upsertConfig(tx *sql.Tx, table string, values map[string]string) error {
 
 	query := fmt.Sprintf("INSERT OR REPLACE INTO %s (key, value) VALUES", table)
 	exprs := []string{}
-	params := []interface{}{}
+	params := []any{}
 	for key, value := range values {
 		exprs = append(exprs, "(?, ?)")
 		params = append(params, key)
 		params = append(params, value)
 	}
+
 	query += strings.Join(exprs, ",")
 	_, err := tx.Exec(query, params...)
 	return err
@@ -101,10 +102,11 @@ func deleteConfig(tx *sql.Tx, table string, keys []string) error {
 	}
 
 	query := fmt.Sprintf("DELETE FROM %s WHERE key IN %s", table, Params(n))
-	values := make([]interface{}, n)
+	values := make([]any, n)
 	for i, key := range keys {
 		values[i] = key
 	}
+
 	_, err := tx.Exec(query, values...)
 	return err
 }
