@@ -4,10 +4,10 @@ import (
 	"database/sql"
 	"testing"
 
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/mpvl/subtest"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/lxc/lxd/lxd/db/query"
 )
 
 // Exercise possible failure modes.
@@ -18,11 +18,12 @@ func TestSelectObjects_Error(t *testing.T) {
 		error string
 	}{
 		{
-			func(int) []interface{} { return make([]interface{}, 1) },
+			func(int) []any { return make([]any, 1) },
 			"SELECT id, name FROM test",
 			"sql: expected 2 destination arguments in Scan, not 1",
 		},
 	}
+
 	for _, c := range cases {
 		t.Run(c.query, func(t *testing.T) {
 			tx := newTxForObjects(t)
@@ -45,9 +46,9 @@ func TestSelectObjects(t *testing.T) {
 	}, 1)
 	object := objects[0]
 
-	dest := func(i int) []interface{} {
+	dest := func(i int) []any {
 		require.Equal(t, 0, i, "expected at most one row to be yielded")
-		return []interface{}{&object.ID, &object.Name}
+		return []any{&object.ID, &object.Name}
 	}
 
 	stmt, err := tx.Prepare("SELECT id, name FROM test WHERE name=?")
@@ -64,22 +65,23 @@ func TestSelectObjects(t *testing.T) {
 func TestUpsertObject_Error(t *testing.T) {
 	cases := []struct {
 		columns []string
-		values  []interface{}
+		values  []any
 		error   string
 	}{
 		{
 			[]string{},
-			[]interface{}{},
+			[]any{},
 			"columns length is zero",
 		},
 		{
 			[]string{"id"},
-			[]interface{}{2, "egg"},
+			[]any{2, "egg"},
 			"columns length does not match values length",
 		},
 	}
+
 	for _, c := range cases {
-		subtest.Run(t, c.error, func(t *testing.T) {
+		t.Run(c.error, func(t *testing.T) {
 			tx := newTxForObjects(t)
 			id, err := query.UpsertObject(tx, "foo", c.columns, c.values)
 			assert.Equal(t, int64(-1), id)
@@ -92,7 +94,7 @@ func TestUpsertObject_Error(t *testing.T) {
 func TestUpsertObject_Insert(t *testing.T) {
 	tx := newTxForObjects(t)
 
-	id, err := query.UpsertObject(tx, "test", []string{"name"}, []interface{}{"egg"})
+	id, err := query.UpsertObject(tx, "test", []string{"name"}, []any{"egg"})
 	require.NoError(t, err)
 	assert.Equal(t, int64(2), id)
 
@@ -102,9 +104,9 @@ func TestUpsertObject_Insert(t *testing.T) {
 	}, 1)
 	object := objects[0]
 
-	dest := func(i int) []interface{} {
+	dest := func(i int) []any {
 		require.Equal(t, 0, i, "expected at most one row to be yielded")
-		return []interface{}{&object.ID, &object.Name}
+		return []any{&object.ID, &object.Name}
 	}
 
 	stmt, err := tx.Prepare("SELECT id, name FROM test WHERE name=?")
@@ -121,7 +123,7 @@ func TestUpsertObject_Insert(t *testing.T) {
 func TestUpsertObject_Update(t *testing.T) {
 	tx := newTxForObjects(t)
 
-	id, err := query.UpsertObject(tx, "test", []string{"id", "name"}, []interface{}{1, "egg"})
+	id, err := query.UpsertObject(tx, "test", []string{"id", "name"}, []any{1, "egg"})
 	require.NoError(t, err)
 	assert.Equal(t, int64(1), id)
 
@@ -131,9 +133,9 @@ func TestUpsertObject_Update(t *testing.T) {
 	}, 1)
 	object := objects[0]
 
-	dest := func(i int) []interface{} {
+	dest := func(i int) []any {
 		require.Equal(t, 0, i, "expected at most one row to be yielded")
-		return []interface{}{&object.ID, &object.Name}
+		return []any{&object.ID, &object.Name}
 	}
 
 	stmt, err := tx.Prepare("SELECT id, name FROM test WHERE name=?")
