@@ -1,33 +1,36 @@
 package query_test
 
 import (
+	"context"
 	"database/sql"
 	"fmt"
 	"testing"
 
-	"github.com/lxc/lxd/lxd/db/query"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+
+	"github.com/lxc/lxd/lxd/db/query"
 )
 
 // Any error happening when beginning the transaction will be propagated.
 func TestTransaction_BeginError(t *testing.T) {
 	db := newDB(t)
-	db.Close()
+	err := db.Close()
+	require.NoError(t, err)
 
-	err := query.Transaction(db, func(*sql.Tx) error { return nil })
+	err = query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error { return nil })
 	assert.NotNil(t, err)
-	assert.Contains(t, err.Error(), "failed to begin transaction")
+	assert.Contains(t, err.Error(), "Failed to begin transaction")
 }
 
 // Any error happening when in the transaction function will cause a rollback.
 func TestTransaction_FunctionError(t *testing.T) {
 	db := newDB(t)
 
-	err := query.Transaction(db, func(tx *sql.Tx) error {
+	err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 		_, err := tx.Exec("CREATE TABLE test (id INTEGER)")
 		assert.NoError(t, err)
 		return fmt.Errorf("boom")
-
 	})
 	assert.EqualError(t, err, "boom")
 
