@@ -1,6 +1,7 @@
 package schema
 
 import (
+	"context"
 	"database/sql"
 	"fmt"
 	"sort"
@@ -8,7 +9,6 @@ import (
 
 	"github.com/lxc/lxd/lxd/db/query"
 	"github.com/lxc/lxd/shared"
-	"github.com/pkg/errors"
 )
 
 // Schema captures the schema of a database in terms of a series of ordered
@@ -60,7 +60,7 @@ func NewFromMap(versionsToUpdates map[int]Update) *Schema {
 	}
 
 	// Sort the versions,
-	sort.Sort(sort.IntSlice(versions))
+	sort.Ints(versions)
 
 	// Build the updates slice.
 	updates := []Update{}
@@ -138,10 +138,10 @@ func (s *Schema) File(path string) {
 func (s *Schema) Ensure(db *sql.DB) (int, error) {
 	var current int
 	aborted := false
-	err := query.Transaction(db, func(tx *sql.Tx) error {
+	err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 		err := execFromFile(tx, s.path, s.hook)
 		if err != nil {
-			return errors.Wrapf(err, "failed to execute queries from %s", s.path)
+			return fmt.Errorf("failed to execute queries from %s: %w", s.path, err)
 		}
 
 		err = ensureSchemaTableExists(tx)
@@ -162,6 +162,7 @@ func (s *Schema) Ensure(db *sql.DB) (int, error) {
 				aborted = true
 				return nil
 			}
+
 			if err != nil {
 				return err
 			}
@@ -172,7 +173,7 @@ func (s *Schema) Ensure(db *sql.DB) (int, error) {
 		if current == 0 && s.fresh != "" {
 			_, err = tx.Exec(s.fresh)
 			if err != nil {
-				return fmt.Errorf("cannot apply fresh schema: %v", err)
+				return fmt.Errorf("cannot apply fresh schema: %w", err)
 			}
 		} else {
 			err = ensureUpdatesAreApplied(tx, current, s.updates, s.hook)
@@ -186,9 +187,11 @@ func (s *Schema) Ensure(db *sql.DB) (int, error) {
 	if err != nil {
 		return -1, err
 	}
+
 	if aborted {
 		return current, ErrGracefulAbort
 	}
+
 	return current, nil
 }
 
@@ -200,17 +203,19 @@ func (s *Schema) Ensure(db *sql.DB) (int, error) {
 // error will be returned.
 func (s *Schema) Dump(db *sql.DB) (string, error) {
 	var statements []string
-	err := query.Transaction(db, func(tx *sql.Tx) error {
+	err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 		err := checkAllUpdatesAreApplied(tx, s.updates)
 		if err != nil {
 			return err
 		}
+
 		statements, err = selectTablesSQL(tx)
 		return err
 	})
 	if err != nil {
 		return "", err
 	}
+
 	for i, statement := range statements {
 		statements[i] = formatSQL(statement)
 	}
@@ -247,14 +252,14 @@ func (s *Schema) ExerciseUpdate(version int, hook func(*sql.DB)) (*sql.DB, error
 	// Create an in-memory database.
 	db, err := sql.Open("sqlite3", ":memory:?_foreign_keys=1")
 	if err != nil {
-		return nil, fmt.Errorf("failed to open memory database: %v", err)
+		return nil, fmt.Errorf("failed to open memory database: %w", err)
 	}
 
 	// Apply all updates to the given version, excluded.
 	trimmed := s.Trim(version - 1)
 	_, err = s.Ensure(db)
 	if err != nil {
-		return nil, fmt.Errorf("failed to apply previous updates: %v", err)
+		return nil, fmt.Errorf("failed to apply previous updates: %w", err)
 	}
 
 	// Execute the optional hook.
@@ -266,7 +271,7 @@ func (s *Schema) ExerciseUpdate(version int, hook func(*sql.DB)) (*sql.DB, error
 	s.Add(trimmed[0])
 	_, err = s.Ensure(db)
 	if err != nil {
-		return nil, fmt.Errorf("failed to apply given update: %v", err)
+		return nil, fmt.Errorf("failed to apply given update: %w", err)
 	}
 
 	return db, nil
@@ -276,12 +281,13 @@ func (s *Schema) ExerciseUpdate(version int, hook func(*sql.DB)) (*sql.DB, error
 func ensureSchemaTableExists(tx *sql.Tx) error {
 	exists, err := DoesSchemaTableExist(tx)
 	if err != nil {
-		return fmt.Errorf("failed to check if schema table is there: %v", err)
+		return fmt.Errorf("failed to check if schema table is there: %w", err)
 	}
+
 	if !exists {
 		err := createSchemaTable(tx)
 		if err != nil {
-			return fmt.Errorf("failed to create schema table: %v", err)
+			return fmt.Errorf("failed to create schema table: %w", err)
 		}
 	}
 	return nil
@@ -292,7 +298,7 @@ func ensureSchemaTableExists(tx *sql.Tx) error {
 func queryCurrentVersion(tx *sql.Tx) (int, error) {
 	versions, err := selectSchemaVersions(tx)
 	if err != nil {
-		return -1, fmt.Errorf("failed to fetch update versions: %v", err)
+		return -1, fmt.Errorf("failed to fetch update versions: %w", err)
 	}
 
 	// Fix bad upgrade code between 30 and 32
@@ -305,7 +311,25 @@ func queryCurrentVersion(tx *sql.Tx) (int, error) {
 
 		versions, err = selectSchemaVersions(tx)
 		if err != nil {
-			return -1, fmt.Errorf("failed to fetch update versions: %v", err)
+			return -1, fmt.Errorf("failed to fetch update versions: %w", err)
+		}
+	}
+
+	// Fix broken schema version between 37 and 38
+	if hasVersion(37) && !hasVersion(38) {
+		count, err := query.Count(tx, "config", "key = 'cluster.https_address'")
+		if err != nil {
+			return -1, fmt.Errorf("Failed to check if cluster.https_address is set: %w", err)
+		}
+
+		if count == 1 {
+			// Insert the missing version.
+			err := insertSchemaVersion(tx, 38)
+			if err != nil {
+				return -1, fmt.Errorf("Failed to insert missing schema version 38")
+			}
+
+			versions = append(versions, 38)
 		}
 	}
 
@@ -315,6 +339,7 @@ func queryCurrentVersion(tx *sql.Tx) (int, error) {
 		if err != nil {
 			return -1, err
 		}
+
 		current = versions[len(versions)-1] // Highest recorded version
 	}
 
@@ -345,8 +370,9 @@ func ensureUpdatesAreApplied(tx *sql.Tx, current int, updates []Update, hook Hoo
 		}
 		err := update(tx)
 		if err != nil {
-			return fmt.Errorf("failed to apply update %d: %v", current, err)
+			return fmt.Errorf("failed to apply update %d: %w", current, err)
 		}
+
 		current++
 
 		err = insertSchemaVersion(tx, current)
@@ -361,8 +387,7 @@ func ensureUpdatesAreApplied(tx *sql.Tx, current int, updates []Update, hook Hoo
 // Check that the given list of update version numbers doesn't have "holes",
 // that is each version equal the preceding version plus 1.
 func checkSchemaVersionsHaveNoHoles(versions []int) error {
-	// Sanity check that there are no "holes" in the recorded
-	// versions.
+	// Ensure that there are no "holes" in the recorded versions.
 	for i := range versions[:len(versions)-1] {
 		if versions[i+1] != versions[i]+1 {
 			return fmt.Errorf("Missing updates: %d to %d", versions[i], versions[i+1])
@@ -375,7 +400,7 @@ func checkSchemaVersionsHaveNoHoles(versions []int) error {
 func checkAllUpdatesAreApplied(tx *sql.Tx, updates []Update) error {
 	versions, err := selectSchemaVersions(tx)
 	if err != nil {
-		return fmt.Errorf("failed to fetch update versions: %v", err)
+		return fmt.Errorf("failed to fetch update versions: %w", err)
 	}
 
 	if len(versions) == 0 {
@@ -391,6 +416,7 @@ func checkAllUpdatesAreApplied(tx *sql.Tx, updates []Update) error {
 	if current != len(updates) {
 		return fmt.Errorf("update level is %d, expected %d", current, len(updates))
 	}
+
 	return nil
 }
 
@@ -406,7 +432,9 @@ func formatSQL(statement string) string {
 			// Let UNIQUE(x, y) constraints alone.
 			continue
 		}
+
 		lines[i] = strings.Replace(line, ", ", ",\n    ", -1)
 	}
+
 	return strings.Join(lines, "\n")
 }
