@@ -3,6 +3,7 @@ package schema_test
 import (
 	"database/sql"
 	"fmt"
+	"io/ioutil"
 	"os"
 	"testing"
 
@@ -14,6 +15,23 @@ import (
 	"github.com/lxc/lxd/shared"
 )
 
+// WriteTempFile creates a temp file with the specified content.
+func WriteTempFile(dir string, prefix string, content string) (string, error) {
+	f, err := ioutil.TempFile(dir, prefix)
+	if err != nil {
+		return "", err
+	}
+
+	defer func() { _ = f.Close() }()
+
+	_, err = f.WriteString(content)
+	if err != nil {
+		return "", err
+	}
+
+	return f.Name(), f.Close()
+}
+
 // Create a new Schema by specifying an explicit map from versions to Update
 // functions.
 func TestNewFromMap(t *testing.T) {
@@ -295,6 +313,7 @@ func TestSchema_Trim(t *testing.T) {
 		2: updateInsertValue,
 		3: updateAddColumn,
 	}
+
 	schema := schema.NewFromMap(updates)
 	trimmed := schema.Trim(2)
 	assert.Len(t, trimmed, 1)
@@ -352,9 +371,9 @@ func TestSchema_File_Garbage(t *testing.T) {
 	schema, db := newSchemaAndDB(t)
 	schema.Add(updateCreateTable)
 
-	path, err := shared.WriteTempFile("", "lxd-db-schema-", "SELECT FROM baz")
+	path, err := WriteTempFile("", "lxd-db-schema-", "SELECT FROM baz")
 	require.NoError(t, err)
-	defer os.Remove(path)
+	defer func() { _ = os.Remove(path) }()
 
 	schema.File(path)
 
@@ -372,12 +391,12 @@ func TestSchema_File(t *testing.T) {
 	// Add an update that would insert a value into a non-existing table.
 	schema.Add(updateInsertValue)
 
-	path, err := shared.WriteTempFile("", "lxd-db-schema-",
+	path, err := WriteTempFile("", "lxd-db-schema-",
 		`CREATE TABLE test (id INTEGER);
 INSERT INTO test VALUES (2);
 `)
 	require.NoError(t, err)
-	defer os.Remove(path)
+	defer func() { _ = os.Remove(path) }()
 
 	schema.File(path)
 
@@ -406,9 +425,9 @@ func TestSchema_File_Hook(t *testing.T) {
 
 	// Add a custom schema update query file that inserts a value into a
 	// non-existing table.
-	path, err := shared.WriteTempFile("", "lxd-db-schema-", "INSERT INTO test VALUES (2)")
+	path, err := WriteTempFile("", "lxd-db-schema-", "INSERT INTO test VALUES (2)")
 	require.NoError(t, err)
-	defer os.Remove(path)
+	defer func() { _ = os.Remove(path) }()
 
 	schema.File(path)
 
@@ -419,6 +438,7 @@ func TestSchema_File_Hook(t *testing.T) {
 			_, err := tx.Exec("CREATE TABLE test (id INTEGER)")
 			return err
 		}
+
 		return nil
 	})
 
