@@ -6,9 +6,18 @@ import (
 	"strings"
 
 	"github.com/juju/gomaasapi"
+
+	"github.com/lxc/lxd/lxd/project"
 )
 
-// Controller represents a MAAS server's machine functions
+// Instance is a MAAS specific instance interface.
+// This is used rather than instance.Instance to avoid import loops.
+type Instance interface {
+	Name() string
+	Project() string
+}
+
+// Controller represents a MAAS server's machine functions.
 type Controller struct {
 	url string
 
@@ -17,21 +26,21 @@ type Controller struct {
 	machine gomaasapi.Machine
 }
 
-// ContainerInterface represents a MAAS connected network interface on the container
+// ContainerInterface represents a MAAS connected network interface on the container.
 type ContainerInterface struct {
 	Name       string
 	MACAddress string
 	Subnets    []ContainerInterfaceSubnet
 }
 
-// ContainerInterfaceSubnet represents an interface's subscription to a MAAS subnet
+// ContainerInterfaceSubnet represents an interface's subscription to a MAAS subnet.
 type ContainerInterfaceSubnet struct {
 	Name    string
 	Address string
 }
 
 func parseInterfaces(interfaces []ContainerInterface) (map[string]ContainerInterface, error) {
-	// Sanity checks
+	// Quick checks.
 	if len(interfaces) == 0 {
 		return nil, fmt.Errorf("At least one interface must be provided")
 	}
@@ -58,7 +67,7 @@ func parseInterfaces(interfaces []ContainerInterface) (map[string]ContainerInter
 	return macInterfaces, nil
 }
 
-// NewController returns a new Controller using the specific MAAS server and machine
+// NewController returns a new Controller using the specific MAAS server and machine.
 func NewController(url string, key string, machine string) (*Controller, error) {
 	baseURL := fmt.Sprintf("%s/api/2.0/", url)
 
@@ -102,14 +111,28 @@ func NewController(url string, key string, machine string) (*Controller, error)
 	return &c, err
 }
 
-func (c *Controller) getDevice(name string) (gomaasapi.Device, error) {
-	devs, err := c.machine.Devices(gomaasapi.DevicesArgs{Hostname: []string{name}})
+func (c *Controller) getDomain(inst Instance) string {
+	fields := strings.Split(c.machine.FQDN(), ".")
+	domain := strings.Join(fields[1:], ".")
+
+	if inst.Project() == project.Default {
+		return domain
+	}
+
+	return fmt.Sprintf("%s.%s", inst.Project(), domain)
+}
+
+func (c *Controller) getDevice(name string, domain string) (gomaasapi.Device, error) {
+	devs, err := c.machine.Devices(gomaasapi.DevicesArgs{
+		Hostname: []string{name},
+		Domain:   domain,
+	})
 	if err != nil {
 		return nil, err
 	}
 
 	if len(devs) != 1 {
-		return nil, fmt.Errorf("Couldn't find the specified container: %s", name)
+		return nil, fmt.Errorf("Couldn't find the specified instance: %s", name)
 	}
 
 	return devs[0], nil
@@ -133,8 +156,8 @@ func (c *Controller) getSubnets() (map[string]gomaasapi.Subnet, error) {
 	return subnets, nil
 }
 
-// CreateContainer defines a new MAAS device for the controller
-func (c *Controller) CreateContainer(name string, interfaces []ContainerInterface) error {
+// CreateContainer defines a new MAAS device for the controller.
+func (c *Controller) CreateContainer(inst Instance, interfaces []ContainerInterface) error {
 	// Parse the provided interfaces
 	macInterfaces, err := parseInterfaces(interfaces)
 	if err != nil {
@@ -147,9 +170,28 @@ func (c *Controller) CreateContainer(name string, interfaces []ContainerInterfac
 		return err
 	}
 
+	// Validation
+	if len(interfaces) < 1 {
+		return fmt.Errorf("Empty list of MAAS interface provided")
+	}
+
+	for _, iface := range interfaces {
+		if len(iface.Subnets) < 1 {
+			return fmt.Errorf("Bad subnet provided for interface '%s'", iface.Name)
+		}
+
+		for _, subnet := range iface.Subnets {
+			_, ok := subnets[subnet.Name]
+			if !ok {
+				return fmt.Errorf("Subnet '%s' doesn't exist in MAAS", interfaces[0].Subnets[0].Name)
+			}
+		}
+	}
+
 	// Create the device and first interface
 	device, err := c.machine.CreateDevice(gomaasapi.CreateMachineDeviceArgs{
-		Hostname:      name,
+		Hostname:      inst.Name(),
+		Domain:        c.getDomain(inst),
 		InterfaceName: interfaces[0].Name,
 		MACAddress:    interfaces[0].MACAddress,
 		VLAN:          subnets[interfaces[0].Subnets[0].Name].VLAN(),
@@ -161,11 +203,11 @@ func (c *Controller) CreateContainer(name string, interfaces []ContainerInterfac
 	// Wipe the container entry if anything fails
 	success := false
 	defer func() {
-		if success == true {
+		if success {
 			return
 		}
 
-		c.DeleteContainer(name)
+		_ = c.DeleteContainer(inst)
 	}()
 
 	// Create the rest of the interfaces
@@ -181,7 +223,7 @@ func (c *Controller) CreateContainer(name string, interfaces []ContainerInterfac
 	}
 
 	// Get a fresh copy of the device
-	device, err = c.getDevice(name)
+	device, err = c.getDevice(inst.Name(), c.getDomain(inst))
 	if err != nil {
 		return err
 	}
@@ -211,9 +253,12 @@ func (c *Controller) CreateContainer(name string, interfaces []ContainerInterfac
 	return nil
 }
 
-// DefinedContainer returns true if the container is defined in MAAS
-func (c *Controller) DefinedContainer(name string) (bool, error) {
-	devs, err := c.machine.Devices(gomaasapi.DevicesArgs{Hostname: []string{name}})
+// DefinedContainer returns true if the container is defined in MAAS.
+func (c *Controller) DefinedContainer(inst Instance) (bool, error) {
+	devs, err := c.machine.Devices(gomaasapi.DevicesArgs{
+		Hostname: []string{inst.Name()},
+		Domain:   c.getDomain(inst),
+	})
 	if err != nil {
 		return false, err
 	}
@@ -225,8 +270,8 @@ func (c *Controller) DefinedContainer(name string) (bool, error) {
 	return false, nil
 }
 
-// UpdateContainer updates the MAAS device's interfaces with the new provided state
-func (c *Controller) UpdateContainer(name string, interfaces []ContainerInterface) error {
+// UpdateContainer updates the MAAS device's interfaces with the new provided state.
+func (c *Controller) UpdateContainer(inst Instance, interfaces []ContainerInterface) error {
 	// Parse the provided interfaces
 	macInterfaces, err := parseInterfaces(interfaces)
 	if err != nil {
@@ -239,11 +284,29 @@ func (c *Controller) UpdateContainer(name string, interfaces []ContainerInterfac
 		return err
 	}
 
-	device, err := c.getDevice(name)
+	device, err := c.getDevice(inst.Name(), c.getDomain(inst))
 	if err != nil {
 		return err
 	}
 
+	// Validation
+	if len(interfaces) < 1 {
+		return fmt.Errorf("Empty list of MAAS interface provided")
+	}
+
+	for _, iface := range interfaces {
+		if len(iface.Subnets) < 1 {
+			return fmt.Errorf("Bad subnet provided for interface '%s'", iface.Name)
+		}
+
+		for _, subnet := range iface.Subnets {
+			_, ok := subnets[subnet.Name]
+			if !ok {
+				return fmt.Errorf("Subnet '%s' doesn't exist in MAAS", interfaces[0].Subnets[0].Name)
+			}
+		}
+	}
+
 	// Iterate over existing interfaces, drop all removed ones and update existing ones
 	existingInterfaces := map[string]gomaasapi.Interface{}
 	for _, entry := range device.InterfaceSet() {
@@ -269,6 +332,7 @@ func (c *Controller) UpdateContainer(name string, interfaces []ContainerInterfac
 					if subnet.Address == "" || subnet.Address == link.IPAddress() {
 						found = true
 					}
+
 					break
 				}
 			}
@@ -344,9 +408,9 @@ func (c *Controller) UpdateContainer(name string, interfaces []ContainerInterfac
 	return nil
 }
 
-// RenameContainer renames the MAAS device for the container without releasing any allocation
-func (c *Controller) RenameContainer(name string, newName string) error {
-	device, err := c.getDevice(name)
+// RenameContainer renames the MAAS device for the container without releasing any allocation.
+func (c *Controller) RenameContainer(inst Instance, newName string) error {
+	device, err := c.getDevice(inst.Name(), c.getDomain(inst))
 	if err != nil {
 		return err
 	}
@@ -368,9 +432,9 @@ func (c *Controller) RenameContainer(name string, newName string) error {
 	return nil
 }
 
-// DeleteContainer removes the MAAS device for the container
-func (c *Controller) DeleteContainer(name string) error {
-	device, err := c.getDevice(name)
+// DeleteContainer removes the MAAS device for the container.
+func (c *Controller) DeleteContainer(inst Instance) error {
+	device, err := c.getDevice(inst.Name(), c.getDomain(inst))
 	if err != nil {
 		return err
 	}
