@@ -1,15 +1,18 @@
 package main
 
 import (
+	"context"
 	"database/sql"
 	"fmt"
 	"os"
 
-	"github.com/mattn/go-sqlite3"
+	sqlite3 "github.com/mattn/go-sqlite3"
 	"github.com/spf13/cobra"
 
 	"github.com/lxc/lxd/client"
 	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/instance"
+	"github.com/lxc/lxd/lxd/instance/instancetype"
 	"github.com/lxc/lxd/lxd/node"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/idmap"
@@ -31,8 +34,8 @@ func (c *cmdActivateifneeded) Command() *cobra.Command {
 	cmd.Long = `Description:
   Check if LXD should be started
 
-  This command will check if LXD has any auto-started containers,
-  containers which were running prior to LXD's last shutdown or if it's
+  This command will check if LXD has any auto-started instances,
+  instances which were running prior to LXD's last shutdown or if it's
   configured to listen on the network address.
 
   If at least one of those is true, then a connection will be attempted to the
@@ -49,18 +52,14 @@ func (c *cmdActivateifneeded) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf("This must be run as root")
 	}
 
-	// Don't start a full daemon, we just need DB access
-	d := DefaultDaemon()
+	// Don't start a full daemon, we just need database access
+	d := defaultDaemon()
 
-	// Check if either the local database or the legacy local database
-	// files exists.
+	// Check if either the local database files exists.
 	path := d.os.LocalDatabasePath()
 	if !shared.PathExists(d.os.LocalDatabasePath()) {
-		path = d.os.LegacyLocalDatabasePath()
-		if !shared.PathExists(path) {
-			logger.Debugf("No DB, so no need to start the daemon now")
-			return nil
-		}
+		logger.Debugf("No local database, so no need to start the daemon now")
+		return nil
 	}
 
 	// Open the database directly to avoid triggering any initialization
@@ -69,10 +68,11 @@ func (c *cmdActivateifneeded) Run(cmd *cobra.Command, args []string) error {
 	if err != nil {
 		return err
 	}
-	d.db = db.ForLegacyPatches(sqldb)
+
+	d.db.Node = db.DirectAccess(sqldb)
 
 	// Load the configured address from the database
-	address, err := node.HTTPSAddress(d.db)
+	address, err := node.HTTPSAddress(d.db.Node)
 	if err != nil {
 		return err
 	}
@@ -84,59 +84,83 @@ func (c *cmdActivateifneeded) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	// Load the idmap for unprivileged containers
+	// Load the idmap for unprivileged instances
 	d.os.IdmapSet, err = idmap.DefaultIdmapSet("", "")
 	if err != nil {
 		return err
 	}
 
-	// Look for auto-started or previously started containers
+	// Look for auto-started or previously started instances
 	path = d.os.GlobalDatabasePath()
 	if !shared.PathExists(path) {
-		path = d.os.LegacyGlobalDatabasePath()
-		if !shared.PathExists(path) {
-			logger.Debugf("No DB, so no need to start the daemon now")
-			return nil
-		}
+		logger.Debugf("No global database, so no need to start the daemon now")
+		return nil
 	}
+
 	sqldb, err = sql.Open("dqlite_direct_access", path+"?mode=ro")
 	if err != nil {
 		return err
 	}
 
-	d.cluster = db.ForLocalInspection(sqldb)
-	result, err := d.cluster.ContainersList(db.CTypeRegular)
+	defer func() { _ = sqldb.Close() }()
+
+	d.db.Cluster, err = db.ForLocalInspectionWithPreparedStmts(sqldb)
 	if err != nil {
 		return err
 	}
 
-	for _, name := range result {
-		c, err := containerLoadByName(d.State(), name)
-		if err != nil {
-			sqldb.Close()
-			return err
-		}
+	instances, err := instance.LoadNodeAll(d.State(), instancetype.Any)
+	if err != nil {
+		return err
+	}
 
-		config := c.ExpandedConfig()
+	for _, inst := range instances {
+		config := inst.ExpandedConfig()
 		lastState := config["volatile.last_state.power"]
 		autoStart := config["boot.autostart"]
 
-		if c.IsRunning() {
-			sqldb.Close()
-			logger.Debugf("Daemon has running containers, activating...")
+		if inst.IsRunning() {
+			logger.Debugf("Daemon has running instances, activating...")
 			_, err := lxd.ConnectLXDUnix("", nil)
 			return err
 		}
 
 		if lastState == "RUNNING" || lastState == "Running" || shared.IsTrue(autoStart) {
-			sqldb.Close()
-			logger.Debugf("Daemon has auto-started containers, activating...")
+			logger.Debugf("Daemon has auto-started instances, activating...")
+			_, err := lxd.ConnectLXDUnix("", nil)
+			return err
+		}
+
+		// Check for scheduled instance snapshots
+		if config["snapshots.schedule"] != "" {
+			logger.Debugf("Daemon has scheduled instance snapshots, activating...")
+			_, err := lxd.ConnectLXDUnix("", nil)
+			return err
+		}
+	}
+
+	// Check for scheduled volume snapshots
+	var volumes []db.StorageVolumeArgs
+	err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		volumes, err = tx.GetStoragePoolVolumesWithType(db.StoragePoolVolumeTypeCustom)
+		if err != nil {
+			return err
+		}
+
+		return nil
+	})
+	if err != nil {
+		return err
+	}
+
+	for _, vol := range volumes {
+		if vol.Config["snapshots.schedule"] != "" {
+			logger.Debugf("Daemon has scheduled volume snapshots, activating...")
 			_, err := lxd.ConnectLXDUnix("", nil)
 			return err
 		}
 	}
 
-	sqldb.Close()
 	logger.Debugf("No need to start the daemon now")
 	return nil
 }
