@@ -3,10 +3,11 @@ package node_test
 import (
 	"testing"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/node"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/node"
 )
 
 // The server configuration is initially empty.
@@ -17,7 +18,7 @@ func TestConfigLoad_Initial(t *testing.T) {
 	config, err := node.ConfigLoad(tx)
 
 	require.NoError(t, err)
-	assert.Equal(t, map[string]interface{}{}, config.Dump())
+	assert.Equal(t, map[string]any{}, config.Dump())
 }
 
 // If the database contains invalid keys, they are ignored.
@@ -34,7 +35,7 @@ func TestConfigLoad_IgnoreInvalidKeys(t *testing.T) {
 	config, err := node.ConfigLoad(tx)
 
 	require.NoError(t, err)
-	values := map[string]interface{}{"core.https_address": "127.0.0.1:666"}
+	values := map[string]any{"core.https_address": "127.0.0.1:666"}
 	assert.Equal(t, values, config.Dump())
 }
 
@@ -46,7 +47,7 @@ func TestConfigLoad_Triggers(t *testing.T) {
 	config, err := node.ConfigLoad(tx)
 
 	require.NoError(t, err)
-	assert.Equal(t, map[string]interface{}{}, config.Dump())
+	assert.Equal(t, map[string]any{}, config.Dump())
 }
 
 // If some previously set values are missing from the ones passed to Replace(),
@@ -58,11 +59,11 @@ func TestConfig_ReplaceDeleteValues(t *testing.T) {
 	config, err := node.ConfigLoad(tx)
 	require.NoError(t, err)
 
-	changed, err := config.Replace(map[string]interface{}{"core.https_address": "127.0.0.1:666"})
+	changed, err := config.Replace(map[string]any{"core.https_address": "127.0.0.1:666"})
 	assert.NoError(t, err)
 	assert.Equal(t, map[string]string{"core.https_address": "127.0.0.1:666"}, changed)
 
-	changed, err = config.Replace(map[string]interface{}{})
+	changed, err = config.Replace(map[string]any{})
 	assert.NoError(t, err)
 	assert.Equal(t, map[string]string{"core.https_address": ""}, changed)
 
@@ -82,10 +83,10 @@ func TestConfig_PatchKeepsValues(t *testing.T) {
 	config, err := node.ConfigLoad(tx)
 	require.NoError(t, err)
 
-	_, err = config.Replace(map[string]interface{}{"core.https_address": "127.0.0.1:666"})
+	_, err = config.Replace(map[string]any{"core.https_address": "127.0.0.1:666"})
 	assert.NoError(t, err)
 
-	_, err = config.Patch(map[string]interface{}{})
+	_, err = config.Patch(map[string]any{})
 	assert.NoError(t, err)
 
 	assert.Equal(t, "127.0.0.1:666", config.HTTPSAddress())
@@ -108,7 +109,7 @@ func TestHTTPSAddress(t *testing.T) {
 	err = nodeDB.Transaction(func(tx *db.NodeTx) error {
 		config, err := node.ConfigLoad(tx)
 		require.NoError(t, err)
-		_, err = config.Replace(map[string]interface{}{"core.https_address": "127.0.0.1:666"})
+		_, err = config.Replace(map[string]any{"core.https_address": "127.0.0.1:666"})
 		require.NoError(t, err)
 		return nil
 	})
@@ -118,3 +119,27 @@ func TestHTTPSAddress(t *testing.T) {
 	require.NoError(t, err)
 	assert.Equal(t, "127.0.0.1:666", address)
 }
+
+// The cluster.https_address config key is fetched from the db with a new
+// transaction.
+func TestClusterAddress(t *testing.T) {
+	nodeDB, cleanup := db.NewTestNode(t)
+	defer cleanup()
+
+	address, err := node.ClusterAddress(nodeDB)
+	require.NoError(t, err)
+	assert.Equal(t, "", address)
+
+	err = nodeDB.Transaction(func(tx *db.NodeTx) error {
+		config, err := node.ConfigLoad(tx)
+		require.NoError(t, err)
+		_, err = config.Replace(map[string]any{"cluster.https_address": "127.0.0.1:666"})
+		require.NoError(t, err)
+		return nil
+	})
+	require.NoError(t, err)
+
+	address, err = node.ClusterAddress(nodeDB)
+	require.NoError(t, err)
+	assert.Equal(t, "127.0.0.1:666", address)
+}
