@@ -1,7 +1,10 @@
+//go:build linux && cgo
+
 package idmap
 
 import (
 	"bufio"
+	"encoding/json"
 	"fmt"
 	"os"
 	"os/user"
@@ -12,10 +15,10 @@ import (
 	"strconv"
 	"strings"
 	"sync/atomic"
+	"syscall"
 
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
-	"github.com/pkg/errors"
 )
 
 const VFS3FscapsUnsupported int32 = 0
@@ -23,6 +26,7 @@ const VFS3FscapsSupported int32 = 1
 const VFS3FscapsUnknown int32 = -1
 
 var VFS3Fscaps int32 = VFS3FscapsUnknown
+var ErrNoUserMap = fmt.Errorf("No map found for user")
 
 type IdRange struct {
 	Isuid   bool
@@ -35,18 +39,6 @@ func (i *IdRange) Contains(id int64) bool {
 	return id >= i.Startid && id <= i.Endid
 }
 
-/*
- * One entry in id mapping set - a single range of either
- * uid or gid mappings.
- */
-type IdmapEntry struct {
-	Isuid    bool
-	Isgid    bool
-	Hostid   int64 // id as seen on the host - i.e. 100000
-	Nsid     int64 // id as seen in the ns - i.e. 0
-	Maprange int64
-}
-
 func (e *IdmapEntry) ToLxcString() []string {
 	if e.Isuid && e.Isgid {
 		return []string{
@@ -107,6 +99,47 @@ func (e *IdmapEntry) Intersects(i IdmapEntry) bool {
 	return false
 }
 
+// HostIDsCoveredBy returns whether or not the entry is covered by the supplied host UID and GID ID maps.
+// If e.Isuid is true then host IDs must be covered by an entry in allowedHostUIDs, and if e.Isgid is true then
+// host IDs must be covered by an entry in allowedHostGIDs.
+func (e *IdmapEntry) HostIDsCoveredBy(allowedHostUIDs []IdmapEntry, allowedHostGIDs []IdmapEntry) bool {
+	if !e.Isuid && !e.Isgid {
+		return false // This is an invalid idmap entry.
+	}
+
+	isUIDAllowed := false
+
+	if e.Isuid {
+		for _, allowedIDMap := range allowedHostUIDs {
+			if !allowedIDMap.Isuid {
+				continue
+			}
+
+			if e.Hostid >= allowedIDMap.Hostid && (e.Hostid+e.Maprange) <= (allowedIDMap.Hostid+allowedIDMap.Maprange) {
+				isUIDAllowed = true
+				break
+			}
+		}
+	}
+
+	isGIDAllowed := false
+
+	if e.Isgid {
+		for _, allowedIDMap := range allowedHostGIDs {
+			if !allowedIDMap.Isgid {
+				continue
+			}
+
+			if e.Hostid >= allowedIDMap.Hostid && (e.Hostid+e.Maprange) <= (allowedIDMap.Hostid+allowedIDMap.Maprange) {
+				isGIDAllowed = true
+				break
+			}
+		}
+	}
+
+	return e.Isuid == isUIDAllowed && e.Isgid == isGIDAllowed
+}
+
 func (e *IdmapEntry) Usable() error {
 	kernelIdmap, err := CurrentIdmapSet()
 	if err != nil {
@@ -183,18 +216,21 @@ func (e *IdmapEntry) parse(s string) error {
 	if err != nil {
 		return err
 	}
+
 	e.Nsid = int64(nsid)
 
 	hostid, err := strconv.ParseUint(split[2], 10, 32)
 	if err != nil {
 		return err
 	}
+
 	e.Hostid = int64(hostid)
 
 	maprange, err := strconv.ParseUint(split[3], 10, 32)
 	if err != nil {
 		return err
 	}
+
 	e.Maprange = int64(maprange)
 
 	// wraparound
@@ -207,7 +243,7 @@ func (e *IdmapEntry) parse(s string) error {
 
 /*
  * Shift a uid from the host into the container
- * I.e. 0 -> 1000 -> 101000
+ * I.e. 0 -> 1000 -> 101000.
  */
 func (e *IdmapEntry) shift_into_ns(id int64) (int64, error) {
 	if id < e.Nsid || id >= e.Nsid+e.Maprange {
@@ -220,7 +256,7 @@ func (e *IdmapEntry) shift_into_ns(id int64) (int64, error) {
 
 /*
  * Shift a uid from the container back to the host
- * I.e. 101000 -> 1000
+ * I.e. 101000 -> 1000.
  */
 func (e *IdmapEntry) shift_from_ns(id int64) (int64, error) {
 	if id < e.Hostid || id >= e.Hostid+e.Maprange {
@@ -245,7 +281,7 @@ func (s ByHostid) Less(i, j int) bool {
 	return s[i].Hostid < s[j].Hostid
 }
 
-/* taken from http://blog.golang.org/slices (which is under BSD licence) */
+/* taken from http://blog.golang.org/slices (which is under BSD licence). */
 func Extend(slice []IdmapEntry, element IdmapEntry) []IdmapEntry {
 	n := len(slice)
 	if n == cap(slice) {
@@ -255,13 +291,38 @@ func Extend(slice []IdmapEntry, element IdmapEntry) []IdmapEntry {
 		copy(newSlice, slice)
 		slice = newSlice
 	}
+
 	slice = slice[0 : n+1]
 	slice[n] = element
 	return slice
 }
 
-type IdmapSet struct {
-	Idmap []IdmapEntry
+func (m *IdmapSet) Equals(other *IdmapSet) bool {
+	// Get comparable maps
+	expandSortIdmap := func(input *IdmapSet) IdmapSet {
+		if input == nil {
+			input = &IdmapSet{}
+		}
+
+		newEntries := []IdmapEntry{}
+
+		for _, entry := range input.Idmap {
+			if entry.Isuid && entry.Isgid {
+				newEntries = append(newEntries, IdmapEntry{true, false, entry.Hostid, entry.Nsid, entry.Maprange})
+				newEntries = append(newEntries, IdmapEntry{false, true, entry.Hostid, entry.Nsid, entry.Maprange})
+			} else {
+				newEntries = append(newEntries, entry)
+			}
+		}
+
+		output := IdmapSet{Idmap: newEntries}
+		sort.Sort(output)
+
+		return output
+	}
+
+	// Actually compare
+	return reflect.DeepEqual(expandSortIdmap(m), expandSortIdmap(other))
 }
 
 func (m IdmapSet) Len() int {
@@ -274,11 +335,11 @@ func (m IdmapSet) Swap(i, j int) {
 
 func (m IdmapSet) Less(i, j int) bool {
 	if m.Idmap[i].Isuid != m.Idmap[j].Isuid {
-		return m.Idmap[i].Isuid == true
+		return m.Idmap[i].Isuid
 	}
 
 	if m.Idmap[i].Isgid != m.Idmap[j].Isgid {
-		return m.Idmap[i].Isgid == true
+		return m.Idmap[i].Isgid
 	}
 
 	return m.Idmap[i].Nsid < m.Idmap[j].Nsid
@@ -322,6 +383,7 @@ func (m IdmapSet) ValidRanges() ([]*IdRange, error) {
 	if err != nil {
 		return nil, err
 	}
+
 	sort.Sort(idmap)
 
 	for _, mapEntry := range idmap.Idmap {
@@ -392,6 +454,7 @@ func (m *IdmapSet) AddSafe(i IdmapEntry) error {
 		if lower.Maprange > 0 {
 			result = append(result, lower)
 		}
+
 		result = append(result, i)
 		if upper.Maprange > 0 {
 			result = append(result, upper)
@@ -415,18 +478,57 @@ func (m IdmapSet) ToLxcString() []string {
 			}
 		}
 	}
+
 	return lines
 }
 
+func (m IdmapSet) ToUidMappings() []syscall.SysProcIDMap {
+	mapping := []syscall.SysProcIDMap{}
+
+	for _, e := range m.Idmap {
+		if !e.Isuid {
+			continue
+		}
+
+		mapping = append(mapping, syscall.SysProcIDMap{
+			ContainerID: int(e.Nsid),
+			HostID:      int(e.Hostid),
+			Size:        int(e.Maprange),
+		})
+	}
+
+	return mapping
+}
+
+func (m IdmapSet) ToGidMappings() []syscall.SysProcIDMap {
+	mapping := []syscall.SysProcIDMap{}
+
+	for _, e := range m.Idmap {
+		if !e.Isgid {
+			continue
+		}
+
+		mapping = append(mapping, syscall.SysProcIDMap{
+			ContainerID: int(e.Nsid),
+			HostID:      int(e.Hostid),
+			Size:        int(e.Maprange),
+		})
+	}
+
+	return mapping
+}
+
 func (m IdmapSet) Append(s string) (IdmapSet, error) {
 	e := IdmapEntry{}
 	err := e.parse(s)
 	if err != nil {
 		return m, err
 	}
+
 	if m.Intersects(e) {
 		return m, fmt.Errorf("Conflicting id mapping")
 	}
+
 	m.Idmap = Extend(m.Idmap, e)
 	return m, nil
 }
@@ -489,8 +591,9 @@ func (set *IdmapSet) doUidshiftIntoContainer(dir string, testmode bool, how stri
 	tmp := filepath.Dir(dir)
 	tmp, err := filepath.EvalSymlinks(tmp)
 	if err != nil {
-		return errors.Wrap(err, "Expand symlinks")
+		return fmt.Errorf("Expand symlinks: %w", err)
 	}
+
 	dir = filepath.Join(tmp, filepath.Base(dir))
 	dir = strings.TrimRight(dir, "/")
 
@@ -604,7 +707,7 @@ func (set *IdmapSet) ShiftFile(p string) error {
 }
 
 /*
- * get a uid or gid mapping from /etc/subxid
+ * get a uid or gid mapping from /etc/subxid.
  */
 func getFromShadow(fname string, username string) ([][]int64, error) {
 	entries := [][]int64{}
@@ -613,7 +716,8 @@ func getFromShadow(fname string, username string) ([][]int64, error) {
 	if err != nil {
 		return nil, err
 	}
-	defer f.Close()
+
+	defer func() { _ = f.Close() }()
 
 	scanner := bufio.NewScanner(f)
 	for scanner.Scan() {
@@ -647,14 +751,14 @@ func getFromShadow(fname string, username string) ([][]int64, error) {
 	}
 
 	if len(entries) == 0 {
-		return nil, fmt.Errorf("User %q has no %ss", username, path.Base(fname))
+		return nil, ErrNoUserMap
 	}
 
 	return entries, nil
 }
 
 /*
- * get a uid or gid mapping from /proc/self/{g,u}id_map
+ * get a uid or gid mapping from /proc/self/{g,u}id_map.
  */
 func getFromProc(fname string) ([][]int64, error) {
 	entries := [][]int64{}
@@ -663,7 +767,8 @@ func getFromProc(fname string) ([][]int64, error) {
 	if err != nil {
 		return nil, err
 	}
-	defer f.Close()
+
+	defer func() { _ = f.Close() }()
 
 	scanner := bufio.NewScanner(f)
 	for scanner.Scan() {
@@ -708,7 +813,7 @@ func getFromProc(fname string) ([][]int64, error) {
 }
 
 /*
- * Create a new default idmap
+ * Create a new default idmap.
  */
 func DefaultIdmapSet(rootfs string, username string) (*IdmapSet, error) {
 	idmapset := new(IdmapSet)
@@ -729,6 +834,11 @@ func DefaultIdmapSet(rootfs string, username string) (*IdmapSet, error) {
 		// Parse the shadow uidmap
 		entries, err := getFromShadow(subuidPath, username)
 		if err != nil {
+			if username == "root" && err == ErrNoUserMap {
+				// No root map available, figure out a default map
+				return kernelDefaultMap()
+			}
+
 			return nil, err
 		}
 
@@ -748,6 +858,11 @@ func DefaultIdmapSet(rootfs string, username string) (*IdmapSet, error) {
 		// Parse the shadow gidmap
 		entries, err = getFromShadow(subgidPath, username)
 		if err != nil {
+			if username == "root" && err == ErrNoUserMap {
+				// No root map available, figure out a default map
+				return kernelDefaultMap()
+			}
+
 			return nil, err
 		}
 
@@ -768,10 +883,19 @@ func DefaultIdmapSet(rootfs string, username string) (*IdmapSet, error) {
 	}
 
 	// No shadow available, figure out a default map
+	return kernelDefaultMap()
+}
+
+func kernelDefaultMap() (*IdmapSet, error) {
+	idmapset := new(IdmapSet)
+
 	kernelMap, err := CurrentIdmapSet()
 	if err != nil {
 		// Hardcoded fallback map
-		e := IdmapEntry{Isuid: true, Isgid: true, Nsid: 0, Hostid: 1000000, Maprange: 1000000000}
+		e := IdmapEntry{Isuid: true, Isgid: false, Nsid: 0, Hostid: 1000000, Maprange: 1000000000}
+		idmapset.Idmap = Extend(idmapset.Idmap, e)
+
+		e = IdmapEntry{Isuid: false, Isgid: true, Nsid: 0, Hostid: 1000000, Maprange: 1000000000}
 		idmapset.Idmap = Extend(idmapset.Idmap, e)
 		return idmapset, nil
 	}
@@ -789,7 +913,10 @@ func DefaultIdmapSet(rootfs string, username string) (*IdmapSet, error) {
 
 	if reflect.DeepEqual(kernelRanges, fullKernelRanges) {
 		// Hardcoded fallback map
-		e := IdmapEntry{Isuid: true, Isgid: true, Nsid: 0, Hostid: 1000000, Maprange: 1000000000}
+		e := IdmapEntry{Isuid: true, Isgid: false, Nsid: 0, Hostid: 1000000, Maprange: 1000000000}
+		idmapset.Idmap = Extend(idmapset.Idmap, e)
+
+		e = IdmapEntry{Isuid: false, Isgid: true, Nsid: 0, Hostid: 1000000, Maprange: 1000000000}
 		idmapset.Idmap = Extend(idmapset.Idmap, e)
 		return idmapset, nil
 	}
@@ -858,7 +985,7 @@ func DefaultIdmapSet(rootfs string, username string) (*IdmapSet, error) {
 }
 
 /*
- * Create an idmap of the current allocation
+ * Create an idmap of the current allocation.
  */
 func CurrentIdmapSet() (*IdmapSet, error) {
 	idmapset := new(IdmapSet)
@@ -899,3 +1026,74 @@ func CurrentIdmapSet() (*IdmapSet, error) {
 
 	return idmapset, nil
 }
+
+// JSONUnmarshal unmarshals an IDMAP encoded as JSON.
+func JSONUnmarshal(idmapJSON string) (*IdmapSet, error) {
+	lastIdmap := new(IdmapSet)
+	err := json.Unmarshal([]byte(idmapJSON), &lastIdmap.Idmap)
+	if err != nil {
+		return nil, err
+	}
+
+	if len(lastIdmap.Idmap) == 0 {
+		return nil, nil
+	}
+
+	return lastIdmap, nil
+}
+
+// JSONMarshal marshals an IDMAP to JSON string.
+func JSONMarshal(idmapSet *IdmapSet) (string, error) {
+	idmapBytes, err := json.Marshal(idmapSet.Idmap)
+	if err != nil {
+		return "", err
+	}
+
+	return string(idmapBytes), nil
+}
+
+// GetIdmapSet reads the uid/gid allocation.
+func GetIdmapSet() *IdmapSet {
+	idmapSet, err := DefaultIdmapSet("", "")
+	if err != nil {
+		logger.Warn("Error reading default uid/gid map", map[string]any{"err": err.Error()})
+		logger.Warnf("Only privileged containers will be able to run")
+		idmapSet = nil
+	} else {
+		kernelIdmapSet, err := CurrentIdmapSet()
+		if err == nil {
+			logger.Infof("Kernel uid/gid map:")
+			for _, lxcmap := range kernelIdmapSet.ToLxcString() {
+				logger.Infof(fmt.Sprintf(" - %s", lxcmap))
+			}
+		}
+
+		if len(idmapSet.Idmap) == 0 {
+			logger.Warnf("No available uid/gid map could be found")
+			logger.Warnf("Only privileged containers will be able to run")
+			idmapSet = nil
+		} else {
+			logger.Infof("Configured LXD uid/gid map:")
+			for _, lxcmap := range idmapSet.Idmap {
+				suffix := ""
+
+				if lxcmap.Usable() != nil {
+					suffix = " (unusable)"
+				}
+
+				for _, lxcEntry := range lxcmap.ToLxcString() {
+					logger.Infof(" - %s%s", lxcEntry, suffix)
+				}
+			}
+
+			err = idmapSet.Usable()
+			if err != nil {
+				logger.Warnf("One or more uid/gid map entry isn't usable (typically due to nesting)")
+				logger.Warnf("Only privileged containers will be able to run")
+				idmapSet = nil
+			}
+		}
+	}
+
+	return idmapSet
+}
