@@ -1,6 +1,7 @@
 package shared
 
 import (
+	"context"
 	"crypto/tls"
 	"crypto/x509"
 	"encoding/pem"
@@ -9,17 +10,22 @@ import (
 	"io/ioutil"
 	"net"
 	"net/http"
-	"strconv"
+	"os"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/logger"
 )
 
-func RFC3493Dialer(network, address string) (net.Conn, error) {
+// connectErrorPrefix used as prefix to error returned from RFC3493Dialer.
+const connectErrorPrefix = "Unable to connect to"
+
+// RFC3493Dialer connects to the specified server and returns the connection.
+// If the connection cannot be established then an error with the connectErrorPrefix is returned.
+func RFC3493Dialer(context context.Context, network string, address string) (net.Conn, error) {
 	host, port, err := net.SplitHostPort(address)
 	if err != nil {
 		return nil, err
@@ -29,49 +35,60 @@ func RFC3493Dialer(network, address string) (net.Conn, error) {
 	if err != nil {
 		return nil, err
 	}
+
+	var errs []error
 	for _, a := range addrs {
 		c, err := net.DialTimeout(network, net.JoinHostPort(a, port), 10*time.Second)
 		if err != nil {
+			errs = append(errs, err)
 			continue
 		}
-		if tc, ok := c.(*net.TCPConn); ok {
-			tc.SetKeepAlive(true)
-			tc.SetKeepAlivePeriod(3 * time.Second)
+
+		tc, ok := c.(*net.TCPConn)
+		if ok {
+			_ = tc.SetKeepAlive(true)
+			_ = tc.SetKeepAlivePeriod(3 * time.Second)
 		}
-		return c, err
+
+		return c, nil
 	}
-	return nil, fmt.Errorf("Unable to connect to: " + address)
+
+	return nil, fmt.Errorf("%s: %s (%v)", connectErrorPrefix, address, errs)
+}
+
+// IsConnectionError returns true if the given error is due to the dialer not being able to connect to the target
+// LXD server.
+func IsConnectionError(err error) bool {
+	// FIXME: unfortunately the LXD client currently does not provide a way to differentiate between errors.
+	return strings.Contains(err.Error(), connectErrorPrefix)
 }
 
 // InitTLSConfig returns a tls.Config populated with default encryption
 // parameters. This is used as baseline config for both client and server
 // certificates used by LXD.
 func InitTLSConfig() *tls.Config {
-	return &tls.Config{
-		MinVersion: tls.VersionTLS12,
-		CipherSuites: []uint16{
-			tls.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
-			tls.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
-			tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
-			tls.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
-			tls.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
-			tls.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
-			tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
-			tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
-		},
-		PreferServerCipherSuites: true,
+	config := &tls.Config{}
+
+	// Restrict to TLS 1.3 unless LXD_INSECURE_TLS is set.
+	if IsFalseOrEmpty(os.Getenv("LXD_INSECURE_TLS")) {
+		config.MinVersion = tls.VersionTLS13
+	} else {
+		config.MinVersion = tls.VersionTLS12
 	}
+
+	return config
 }
 
 func finalizeTLSConfig(tlsConfig *tls.Config, tlsRemoteCert *x509.Certificate) {
+	// Setup RootCA
+	if tlsConfig.RootCAs == nil {
+		tlsConfig.RootCAs, _ = systemCertPool()
+	}
+
 	// Trusted certificates
 	if tlsRemoteCert != nil {
-		caCertPool := tlsConfig.RootCAs
-		if caCertPool == nil {
-			caCertPool, _ = x509.SystemCertPool()
-			if caCertPool == nil {
-				caCertPool = x509.NewCertPool()
-			}
+		if tlsConfig.RootCAs == nil {
+			tlsConfig.RootCAs = x509.NewCertPool()
 		}
 
 		// Make it a valid RootCA
@@ -79,8 +96,7 @@ func finalizeTLSConfig(tlsConfig *tls.Config, tlsRemoteCert *x509.Certificate) {
 		tlsRemoteCert.KeyUsage = x509.KeyUsageCertSign
 
 		// Setup the pool
-		caCertPool.AddCert(tlsRemoteCert)
-		tlsConfig.RootCAs = caCertPool
+		tlsConfig.RootCAs.AddCert(tlsRemoteCert)
 
 		// Set the ServerName
 		if tlsRemoteCert.DNSNames != nil {
@@ -180,20 +196,13 @@ func WebsocketSendStream(conn *websocket.Conn, r io.Reader, bufferSize int) chan
 				break
 			}
 
-			w, err := conn.NextWriter(websocket.BinaryMessage)
+			err := conn.WriteMessage(websocket.BinaryMessage, buf)
 			if err != nil {
-				logger.Debugf("Got error getting next writer %s", err)
-				break
-			}
-
-			_, err = w.Write(buf)
-			w.Close()
-			if err != nil {
-				logger.Debugf("Got err writing %s", err)
+				logger.Debug("Got err writing", logger.Ctx{"err": err})
 				break
 			}
 		}
-		conn.WriteMessage(websocket.TextMessage, []byte{})
+		_ = conn.WriteMessage(websocket.TextMessage, []byte{})
 		ch <- true
 	}(conn, r)
 
@@ -207,23 +216,23 @@ func WebsocketRecvStream(w io.Writer, conn *websocket.Conn) chan bool {
 		for {
 			mt, r, err := conn.NextReader()
 			if mt == websocket.CloseMessage {
-				logger.Debugf("Got close message for reader")
+				logger.Debug("WebsocketRecvStream got close message for reader")
 				break
 			}
 
 			if mt == websocket.TextMessage {
-				logger.Debugf("got message barrier")
+				logger.Debug("WebsocketRecvStream got message barrier")
 				break
 			}
 
 			if err != nil {
-				logger.Debugf("Got error getting next reader %s, %s", err, w)
+				logger.Debug("WebsocketRecvStream got error getting next reader", logger.Ctx{"err": err})
 				break
 			}
 
 			buf, err := ioutil.ReadAll(r)
 			if err != nil {
-				logger.Debugf("Got error writing to writer %s", err)
+				logger.Debug("WebsocketRecvStream got error writing to writer", logger.Ctx{"err": err})
 				break
 			}
 
@@ -233,11 +242,12 @@ func WebsocketRecvStream(w io.Writer, conn *websocket.Conn) chan bool {
 
 			i, err := w.Write(buf)
 			if i != len(buf) {
-				logger.Debugf("Didn't write all of buf")
+				logger.Debug("WebsocketRecvStream didn't write all of buf")
 				break
 			}
+
 			if err != nil {
-				logger.Debugf("Error writing buf %s", err)
+				logger.Debug("WebsocketRecvStream error writing buf", logger.Ctx{"err": err})
 				break
 			}
 		}
@@ -247,8 +257,9 @@ func WebsocketRecvStream(w io.Writer, conn *websocket.Conn) chan bool {
 	return ch
 }
 
-func WebsocketProxy(source *websocket.Conn, target *websocket.Conn) chan bool {
-	forward := func(in *websocket.Conn, out *websocket.Conn, ch chan bool) {
+func WebsocketProxy(source *websocket.Conn, target *websocket.Conn) chan struct{} {
+	// Forwarder between two websockets, closes channel upon disconnection.
+	forward := func(in *websocket.Conn, out *websocket.Conn, ch chan struct{}) {
 		for {
 			mt, r, err := in.NextReader()
 			if err != nil {
@@ -261,32 +272,34 @@ func WebsocketProxy(source *websocket.Conn, target *websocket.Conn) chan bool {
 			}
 
 			_, err = io.Copy(w, r)
-			w.Close()
+			_ = w.Close()
 			if err != nil {
 				break
 			}
 		}
 
-		ch <- true
+		close(ch)
 	}
 
-	chSend := make(chan bool)
+	// Spawn forwarders in both directions.
+	chSend := make(chan struct{})
 	go forward(source, target, chSend)
 
-	chRecv := make(chan bool)
+	chRecv := make(chan struct{})
 	go forward(target, source, chRecv)
 
-	ch := make(chan bool)
+	// Close main channel and disconnect upon completion of either forwarder.
+	ch := make(chan struct{})
 	go func() {
 		select {
 		case <-chSend:
 		case <-chRecv:
 		}
 
-		source.Close()
-		target.Close()
+		_ = source.Close()
+		_ = target.Close()
 
-		ch <- true
+		close(ch)
 	}()
 
 	return ch
@@ -301,66 +314,128 @@ func defaultReader(conn *websocket.Conn, r io.ReadCloser, readDone chan<- bool)
 	for {
 		buf, ok := <-in
 		if !ok {
-			r.Close()
-			logger.Debugf("sending write barrier")
-			conn.WriteMessage(websocket.TextMessage, []byte{})
+			_ = r.Close()
+			logger.Debug("Sending write barrier")
+			_ = conn.WriteMessage(websocket.TextMessage, []byte{})
 			readDone <- true
 			return
 		}
-		w, err := conn.NextWriter(websocket.BinaryMessage)
-		if err != nil {
-			logger.Debugf("Got error getting next writer %s", err)
-			break
-		}
 
-		_, err = w.Write(buf)
-		w.Close()
+		err := conn.WriteMessage(websocket.BinaryMessage, buf)
 		if err != nil {
-			logger.Debugf("Got err writing %s", err)
+			logger.Debug("Got err writing", logger.Ctx{"err": err})
 			break
 		}
 	}
 	closeMsg := websocket.FormatCloseMessage(websocket.CloseNormalClosure, "")
-	conn.WriteMessage(websocket.CloseMessage, closeMsg)
+	_ = conn.WriteMessage(websocket.CloseMessage, closeMsg)
 	readDone <- true
-	r.Close()
+	_ = r.Close()
 }
 
-func defaultWriter(conn *websocket.Conn, w io.WriteCloser, writeDone chan<- bool) {
+func DefaultWriter(conn *websocket.Conn, w io.WriteCloser, writeDone chan<- bool) {
 	for {
 		mt, r, err := conn.NextReader()
 		if err != nil {
-			logger.Debugf("Got error getting next reader %s", err)
+			logger.Debug("DefaultWriter got error getting next reader", logger.Ctx{"err": err})
 			break
 		}
 
 		if mt == websocket.CloseMessage {
-			logger.Debugf("Got close message for reader")
+			logger.Debug("DefaultWriter got close message for reader")
 			break
 		}
 
 		if mt == websocket.TextMessage {
-			logger.Debugf("Got message barrier, resetting stream")
+			logger.Debug("DefaultWriter got message barrier, resetting stream")
 			break
 		}
 
 		buf, err := ioutil.ReadAll(r)
 		if err != nil {
-			logger.Debugf("Got error writing to writer %s", err)
+			logger.Debug("DefaultWriter got error writing to writer", logger.Ctx{"err": err})
 			break
 		}
+
 		i, err := w.Write(buf)
 		if i != len(buf) {
-			logger.Debugf("Didn't write all of buf")
+			logger.Debug("DefaultWriter didn't write all of buf")
 			break
 		}
+
 		if err != nil {
-			logger.Debugf("Error writing buf %s", err)
+			logger.Debug("DefaultWriter error writing buf", logger.Ctx{"err": err})
 			break
 		}
 	}
 	writeDone <- true
-	w.Close()
+	_ = w.Close()
+}
+
+// WebsocketIO is a wrapper implementing ReadWriteCloser on top of websocket.
+type WebsocketIO struct {
+	Conn   *websocket.Conn
+	reader io.Reader
+	mu     sync.Mutex
+}
+
+func (w *WebsocketIO) Read(p []byte) (n int, err error) {
+	// First read from this message
+	if w.reader == nil {
+		var mt int
+
+		mt, w.reader, err = w.Conn.NextReader()
+		if err != nil {
+			return 0, err
+		}
+
+		if mt == websocket.CloseMessage {
+			return 0, io.EOF
+		}
+
+		if mt == websocket.TextMessage {
+			return 0, io.EOF
+		}
+	}
+
+	// Perform the read itself
+	n, err = w.reader.Read(p)
+	if err == io.EOF {
+		// At the end of the message, reset reader
+		w.reader = nil
+		return n, nil
+	}
+
+	if err != nil {
+		return 0, err
+	}
+
+	return n, nil
+}
+
+func (w *WebsocketIO) Write(p []byte) (n int, err error) {
+	w.mu.Lock()
+	defer w.mu.Unlock()
+	wr, err := w.Conn.NextWriter(websocket.BinaryMessage)
+	if err != nil {
+		return -1, err
+	}
+
+	n, err = wr.Write(p)
+	if err != nil {
+		return -1, err
+	}
+
+	return n, wr.Close()
+}
+
+// Close sends a control message indicating the stream is finished, but it does not actually close
+// the socket.
+func (w *WebsocketIO) Close() error {
+	w.mu.Lock()
+	defer w.mu.Unlock()
+	// Target expects to get a control message indicating stream is finished.
+	return w.Conn.WriteMessage(websocket.TextMessage, []byte{})
 }
 
 // WebsocketMirror allows mirroring a reader to a websocket and taking the
@@ -382,7 +457,7 @@ func WebsocketMirror(conn *websocket.Conn, w io.WriteCloser, r io.ReadCloser, Re
 
 	WriteFunc := Writer
 	if WriteFunc == nil {
-		WriteFunc = defaultWriter
+		WriteFunc = DefaultWriter
 	}
 
 	go ReadFunc(conn, r, readDone)
@@ -395,36 +470,32 @@ func WebsocketConsoleMirror(conn *websocket.Conn, w io.WriteCloser, r io.ReadClo
 	readDone := make(chan bool, 1)
 	writeDone := make(chan bool, 1)
 
-	go defaultWriter(conn, w, writeDone)
+	go DefaultWriter(conn, w, writeDone)
 
 	go func(conn *websocket.Conn, r io.ReadCloser) {
 		in := ReaderToChannel(r, -1)
 		for {
 			buf, ok := <-in
 			if !ok {
-				r.Close()
-				logger.Debugf("sending write barrier")
-				conn.WriteMessage(websocket.TextMessage, []byte{})
+				_ = r.Close()
+				logger.Debugf("Sending write barrier")
+				_ = conn.WriteMessage(websocket.BinaryMessage, []byte("\r"))
+				_ = conn.WriteMessage(websocket.TextMessage, []byte{})
 				readDone <- true
 				return
 			}
-			w, err := conn.NextWriter(websocket.BinaryMessage)
-			if err != nil {
-				logger.Debugf("Got error getting next writer %s", err)
-				break
-			}
 
-			_, err = w.Write(buf)
-			w.Close()
+			err := conn.WriteMessage(websocket.BinaryMessage, buf)
 			if err != nil {
 				logger.Debugf("Got err writing %s", err)
 				break
 			}
 		}
+
 		closeMsg := websocket.FormatCloseMessage(websocket.CloseNormalClosure, "")
-		conn.WriteMessage(websocket.CloseMessage, closeMsg)
+		_ = conn.WriteMessage(websocket.CloseMessage, closeMsg)
 		readDone <- true
-		r.Close()
+		_ = r.Close()
 	}(conn, r)
 
 	return readDone, writeDone
@@ -434,7 +505,7 @@ var WebsocketUpgrader = websocket.Upgrader{
 	CheckOrigin: func(r *http.Request) bool { return true },
 }
 
-// AllocatePort asks the kernel for a free open port that is ready to use
+// AllocatePort asks the kernel for a free open port that is ready to use.
 func AllocatePort() (int, error) {
 	addr, err := net.ResolveTCPAddr("tcp", "localhost:0")
 	if err != nil {
@@ -445,54 +516,6 @@ func AllocatePort() (int, error) {
 	if err != nil {
 		return -1, err
 	}
-	defer l.Close()
-	return l.Addr().(*net.TCPAddr).Port, nil
-}
 
-func NetworkGetCounters(ifName string) api.NetworkStateCounters {
-	counters := api.NetworkStateCounters{}
-	// Get counters
-	content, err := ioutil.ReadFile("/proc/net/dev")
-	if err == nil {
-		for _, line := range strings.Split(string(content), "\n") {
-			fields := strings.Fields(line)
-
-			if len(fields) != 17 {
-				continue
-			}
-
-			intName := strings.TrimSuffix(fields[0], ":")
-			if intName != ifName {
-				continue
-			}
-
-			rxBytes, err := strconv.ParseInt(fields[1], 10, 64)
-			if err != nil {
-				continue
-			}
-
-			rxPackets, err := strconv.ParseInt(fields[2], 10, 64)
-			if err != nil {
-				continue
-			}
-
-			txBytes, err := strconv.ParseInt(fields[9], 10, 64)
-			if err != nil {
-				continue
-			}
-
-			txPackets, err := strconv.ParseInt(fields[10], 10, 64)
-			if err != nil {
-				continue
-			}
-
-			counters.BytesSent = txBytes
-			counters.BytesReceived = rxBytes
-			counters.PacketsSent = txPackets
-			counters.PacketsReceived = rxPackets
-			break
-		}
-	}
-
-	return counters
+	return l.Addr().(*net.TCPAddr).Port, l.Close()
 }
