@@ -14,6 +14,10 @@ const (
 	ARCH_64BIT_POWERPC_BIG_ENDIAN    = 6
 	ARCH_64BIT_POWERPC_LITTLE_ENDIAN = 7
 	ARCH_64BIT_S390_BIG_ENDIAN       = 8
+	ARCH_32BIT_MIPS                  = 9
+	ARCH_64BIT_MIPS                  = 10
+	ARCH_32BIT_RISCV_LITTLE_ENDIAN   = 11
+	ARCH_64BIT_RISCV_LITTLE_ENDIAN   = 12
 )
 
 var architectureNames = map[int]string{
@@ -25,16 +29,24 @@ var architectureNames = map[int]string{
 	ARCH_64BIT_POWERPC_BIG_ENDIAN:    "ppc64",
 	ARCH_64BIT_POWERPC_LITTLE_ENDIAN: "ppc64le",
 	ARCH_64BIT_S390_BIG_ENDIAN:       "s390x",
+	ARCH_32BIT_MIPS:                  "mips",
+	ARCH_64BIT_MIPS:                  "mips64",
+	ARCH_32BIT_RISCV_LITTLE_ENDIAN:   "riscv32",
+	ARCH_64BIT_RISCV_LITTLE_ENDIAN:   "riscv64",
 }
 
 var architectureAliases = map[int][]string{
-	ARCH_32BIT_INTEL_X86:             []string{"i386"},
-	ARCH_64BIT_INTEL_X86:             []string{"amd64"},
-	ARCH_32BIT_ARMV7_LITTLE_ENDIAN:   []string{"armel", "armhf"},
-	ARCH_64BIT_ARMV8_LITTLE_ENDIAN:   []string{"arm64"},
-	ARCH_32BIT_POWERPC_BIG_ENDIAN:    []string{"powerpc"},
-	ARCH_64BIT_POWERPC_BIG_ENDIAN:    []string{"powerpc64"},
-	ARCH_64BIT_POWERPC_LITTLE_ENDIAN: []string{"ppc64el"},
+	ARCH_32BIT_INTEL_X86:             {"i386", "i586", "386", "x86", "generic_32"},
+	ARCH_64BIT_INTEL_X86:             {"amd64", "generic_64"},
+	ARCH_32BIT_ARMV7_LITTLE_ENDIAN:   {"armel", "armhf", "arm", "armhfp", "armv7a_hardfp", "armv7", "armv7a_vfpv3_hardfp"},
+	ARCH_64BIT_ARMV8_LITTLE_ENDIAN:   {"arm64", "arm64_generic"},
+	ARCH_32BIT_POWERPC_BIG_ENDIAN:    {"powerpc"},
+	ARCH_64BIT_POWERPC_BIG_ENDIAN:    {"powerpc64", "ppc64"},
+	ARCH_64BIT_POWERPC_LITTLE_ENDIAN: {"ppc64el"},
+	ARCH_32BIT_MIPS:                  {"mipsel"},
+	ARCH_64BIT_MIPS:                  {"mips64el"},
+	ARCH_32BIT_RISCV_LITTLE_ENDIAN:   {},
+	ARCH_64BIT_RISCV_LITTLE_ENDIAN:   {},
 }
 
 var architecturePersonalities = map[int]string{
@@ -46,17 +58,25 @@ var architecturePersonalities = map[int]string{
 	ARCH_64BIT_POWERPC_BIG_ENDIAN:    "linux64",
 	ARCH_64BIT_POWERPC_LITTLE_ENDIAN: "linux64",
 	ARCH_64BIT_S390_BIG_ENDIAN:       "linux64",
+	ARCH_32BIT_MIPS:                  "linux32",
+	ARCH_64BIT_MIPS:                  "linux64",
+	ARCH_32BIT_RISCV_LITTLE_ENDIAN:   "linux32",
+	ARCH_64BIT_RISCV_LITTLE_ENDIAN:   "linux64",
 }
 
 var architectureSupportedPersonalities = map[int][]int{
-	ARCH_32BIT_INTEL_X86:             []int{},
-	ARCH_64BIT_INTEL_X86:             []int{ARCH_32BIT_INTEL_X86},
-	ARCH_32BIT_ARMV7_LITTLE_ENDIAN:   []int{},
-	ARCH_64BIT_ARMV8_LITTLE_ENDIAN:   []int{ARCH_32BIT_ARMV7_LITTLE_ENDIAN},
-	ARCH_32BIT_POWERPC_BIG_ENDIAN:    []int{},
-	ARCH_64BIT_POWERPC_BIG_ENDIAN:    []int{ARCH_32BIT_POWERPC_BIG_ENDIAN},
-	ARCH_64BIT_POWERPC_LITTLE_ENDIAN: []int{},
-	ARCH_64BIT_S390_BIG_ENDIAN:       []int{},
+	ARCH_32BIT_INTEL_X86:             {},
+	ARCH_64BIT_INTEL_X86:             {ARCH_32BIT_INTEL_X86},
+	ARCH_32BIT_ARMV7_LITTLE_ENDIAN:   {},
+	ARCH_64BIT_ARMV8_LITTLE_ENDIAN:   {ARCH_32BIT_ARMV7_LITTLE_ENDIAN},
+	ARCH_32BIT_POWERPC_BIG_ENDIAN:    {},
+	ARCH_64BIT_POWERPC_BIG_ENDIAN:    {ARCH_32BIT_POWERPC_BIG_ENDIAN},
+	ARCH_64BIT_POWERPC_LITTLE_ENDIAN: {},
+	ARCH_64BIT_S390_BIG_ENDIAN:       {},
+	ARCH_32BIT_MIPS:                  {},
+	ARCH_64BIT_MIPS:                  {ARCH_32BIT_MIPS},
+	ARCH_32BIT_RISCV_LITTLE_ENDIAN:   {},
+	ARCH_64BIT_RISCV_LITTLE_ENDIAN:   {},
 }
 
 const ArchitectureDefault = "x86_64"
@@ -85,7 +105,7 @@ func ArchitectureId(arch string) (int, error) {
 		}
 	}
 
-	return 0, fmt.Errorf("Architecture isn't supported: %s", arch)
+	return ARCH_UNKNOWN, fmt.Errorf("Architecture isn't supported: %s", arch)
 }
 
 func ArchitecturePersonality(arch int) (string, error) {
@@ -105,3 +125,28 @@ func ArchitecturePersonalities(arch int) ([]int, error) {
 
 	return []int{}, fmt.Errorf("Architecture isn't supported: %d", arch)
 }
+
+// ArchitectureGetLocalID returns the local hardware architecture ID.
+func ArchitectureGetLocalID() (int, error) {
+	name, err := ArchitectureGetLocal()
+	if err != nil {
+		return -1, err
+	}
+
+	id, err := ArchitectureId(name)
+	if err != nil {
+		return -1, err
+	}
+
+	return id, nil
+}
+
+// SupportedArchitectures returns the list of all supported architectures.
+func SupportedArchitectures() []string {
+	result := []string{}
+	for _, archName := range architectureNames {
+		result = append(result, archName)
+	}
+
+	return result
+}
