@@ -3,12 +3,10 @@ package shared
 import (
 	"bufio"
 	"bytes"
+	"context"
 	"crypto/rand"
-	"crypto/sha256"
-	"crypto/sha512"
 	"encoding/gob"
 	"encoding/hex"
-	"encoding/json"
 	"fmt"
 	"hash"
 	"io"
@@ -26,13 +24,18 @@ import (
 	"strings"
 	"time"
 
+	"github.com/flosch/pongo2"
+
+	"github.com/lxc/lxd/lxd/revert"
 	"github.com/lxc/lxd/shared/cancel"
 	"github.com/lxc/lxd/shared/ioprogress"
-	"github.com/pkg/errors"
+	"github.com/lxc/lxd/shared/units"
 )
 
 const SnapshotDelimiter = "/"
-const DefaultPort = "8443"
+const HTTPSDefaultPort = 8443
+const HTTPDefaultPort = 8080
+const HTTPSMetricsDefaultPort = 9100
 
 // URLEncode encodes a path and query parameters to a URL.
 func URLEncode(path string, query map[string]string) (string, error) {
@@ -45,6 +48,7 @@ func URLEncode(path string, query map[string]string) (string, error) {
 	for key, value := range query {
 		params.Add(key, value)
 	}
+
 	u.RawQuery = params.Encode()
 	return u.String(), nil
 }
@@ -65,6 +69,7 @@ func PathExists(name string) bool {
 	if err != nil && os.IsNotExist(err) {
 		return false
 	}
+
 	return true
 }
 
@@ -74,7 +79,8 @@ func PathIsEmpty(path string) (bool, error) {
 	if err != nil {
 		return false, err
 	}
-	defer f.Close()
+
+	defer func() { _ = f.Close() }()
 
 	// read in ONLY one file
 	_, err = f.Readdir(1)
@@ -83,6 +89,7 @@ func PathIsEmpty(path string) (bool, error) {
 	if err == io.EOF {
 		return true, nil
 	}
+
 	return false, err
 }
 
@@ -92,6 +99,7 @@ func IsDir(name string) bool {
 	if err != nil {
 		return false
 	}
+
 	return stat.IsDir()
 }
 
@@ -102,12 +110,65 @@ func IsUnixSocket(path string) bool {
 	if err != nil {
 		return false
 	}
+
 	return (stat.Mode() & os.ModeSocket) == os.ModeSocket
 }
 
+// HostPathFollow takes a valid path (from HostPath) and resolves it
+// all the way to its target or to the last which can be resolved.
+func HostPathFollow(path string) string {
+	// Ignore empty paths
+	if len(path) == 0 {
+		return path
+	}
+
+	// Don't prefix stdin/stdout
+	if path == "-" {
+		return path
+	}
+
+	// Check if we're running in a snap package.
+	if !InSnap() {
+		return path
+	}
+
+	// Handle relative paths
+	if path[0] != os.PathSeparator {
+		// Use the cwd of the parent as snap-confine alters our own cwd on launch
+		ppid := os.Getppid()
+		if ppid < 1 {
+			return path
+		}
+
+		pwd, err := os.Readlink(fmt.Sprintf("/proc/%d/cwd", ppid))
+		if err != nil {
+			return path
+		}
+
+		path = filepath.Clean(strings.Join([]string{pwd, path}, string(os.PathSeparator)))
+	}
+
+	// Rely on "readlink -m" to do the right thing.
+	path = HostPath(path)
+	for {
+		target, err := RunCommand("readlink", "-m", path)
+		if err != nil {
+			return path
+		}
+
+		target = strings.TrimSpace(target)
+
+		if path == HostPath(target) {
+			return path
+		}
+
+		path = HostPath(target)
+	}
+}
+
 // HostPath returns the host path for the provided path
 // On a normal system, this does nothing
-// When inside of a snap environment, returns the real path
+// When inside of a snap environment, returns the real path.
 func HostPath(path string) string {
 	// Ignore empty paths
 	if len(path) == 0 {
@@ -120,9 +181,7 @@ func HostPath(path string) string {
 	}
 
 	// Check if we're running in a snap package
-	snap := os.Getenv("SNAP")
-	snapName := os.Getenv("SNAP_NAME")
-	if snap == "" || snapName != "lxd" {
+	if !InSnap() {
 		return path
 	}
 
@@ -173,6 +232,7 @@ func CachePath(path ...string) string {
 	if varDir != "" {
 		logDir = filepath.Join(varDir, "cache")
 	}
+
 	items := []string{logDir}
 	items = append(items, path...)
 	return filepath.Join(items...)
@@ -186,6 +246,7 @@ func LogPath(path ...string) string {
 	if varDir != "" {
 		logDir = filepath.Join(varDir, "logs")
 	}
+
 	items := []string{logDir}
 	items = append(items, path...)
 	return filepath.Join(items...)
@@ -231,15 +292,6 @@ func ParseLXDFileHeaders(headers http.Header) (uid int64, gid int64, mode int, t
 	return uid, gid, mode, type_, write
 }
 
-func ReadToJSON(r io.Reader, req interface{}) error {
-	buf, err := ioutil.ReadAll(r)
-	if err != nil {
-		return err
-	}
-
-	return json.Unmarshal(buf, req)
-}
-
 func ReaderToChannel(r io.Reader, bufferSize int) <-chan []byte {
 	if bufferSize <= 128*1024 {
 		bufferSize = 128 * 1024
@@ -287,12 +339,6 @@ func RandomCryptoString() (string, error) {
 	return hex.EncodeToString(buf), nil
 }
 
-func SplitExt(fpath string) (string, string) {
-	b := path.Base(fpath)
-	ext := path.Ext(fpath)
-	return b[:len(b)-len(ext)], ext
-}
-
 func AtoiEmptyDefault(s string, def int) (int, error) {
 	if s == "" {
 		return def, nil
@@ -307,6 +353,7 @@ func ReadStdin() ([]byte, error) {
 	if err != nil {
 		return nil, err
 	}
+
 	return line, nil
 }
 
@@ -327,6 +374,34 @@ func WriteAll(w io.Writer, data []byte) error {
 	}
 }
 
+// QuotaWriter returns an error once a given write quota gets exceeded.
+type QuotaWriter struct {
+	writer io.Writer
+	quota  int64
+	n      int64
+}
+
+// NewQuotaWriter returns a new QuotaWriter wrapping the given writer.
+//
+// If the given quota is negative, then no quota is applied.
+func NewQuotaWriter(writer io.Writer, quota int64) *QuotaWriter {
+	return &QuotaWriter{
+		writer: writer,
+		quota:  quota,
+	}
+}
+
+// Write implements the Writer interface.
+func (w *QuotaWriter) Write(p []byte) (n int, err error) {
+	if w.quota >= 0 {
+		w.n += int64(len(p))
+		if w.n > w.quota {
+			return 0, fmt.Errorf("reached %d bytes, exceeding quota of %d", w.n, w.quota)
+		}
+	}
+	return w.writer.Write(p)
+}
+
 // FileMove tries to move a file by using os.Rename,
 // if that fails it tries to copy the file and remove the source.
 func FileMove(oldPath string, newPath string) error {
@@ -340,23 +415,51 @@ func FileMove(oldPath string, newPath string) error {
 		return err
 	}
 
-	os.Remove(oldPath)
+	_ = os.Remove(oldPath)
 
 	return nil
 }
 
 // FileCopy copies a file, overwriting the target if it exists.
 func FileCopy(source string, dest string) error {
+	fi, err := os.Lstat(source)
+	if err != nil {
+		return err
+	}
+
+	_, uid, gid := GetOwnerMode(fi)
+
+	if fi.Mode()&os.ModeSymlink != 0 {
+		target, err := os.Readlink(source)
+		if err != nil {
+			return err
+		}
+
+		if PathExists(dest) {
+			err = os.Remove(dest)
+			if err != nil {
+				return err
+			}
+		}
+
+		err = os.Symlink(target, dest)
+		if err != nil {
+			return err
+		}
+
+		if runtime.GOOS != "windows" {
+			return os.Lchown(dest, uid, gid)
+		}
+
+		return nil
+	}
+
 	s, err := os.Open(source)
 	if err != nil {
 		return err
 	}
-	defer s.Close()
 
-	fi, err := s.Stat()
-	if err != nil {
-		return err
-	}
+	defer func() { _ = s.Close() }()
 
 	d, err := os.Create(dest)
 	if err != nil {
@@ -369,7 +472,6 @@ func FileCopy(source string, dest string) error {
 			return err
 		}
 	}
-	defer d.Close()
 
 	_, err = io.Copy(d, s)
 	if err != nil {
@@ -378,11 +480,13 @@ func FileCopy(source string, dest string) error {
 
 	/* chown not supported on windows */
 	if runtime.GOOS != "windows" {
-		_, uid, gid := GetOwnerMode(fi)
-		return d.Chown(uid, gid)
+		err = d.Chown(uid, gid)
+		if err != nil {
+			return err
+		}
 	}
 
-	return nil
+	return d.Close()
 }
 
 // DirCopy copies a directory recursively, overwriting the target if it exists.
@@ -390,7 +494,7 @@ func DirCopy(source string, dest string) error {
 	// Get info about source.
 	info, err := os.Stat(source)
 	if err != nil {
-		return errors.Wrapf(err, "failed to get source directory info")
+		return fmt.Errorf("failed to get source directory info: %w", err)
 	}
 
 	if !info.IsDir() {
@@ -401,39 +505,37 @@ func DirCopy(source string, dest string) error {
 	if PathExists(dest) {
 		err := os.RemoveAll(dest)
 		if err != nil {
-			return errors.Wrapf(err, "failed to remove destination directory %s", dest)
+			return fmt.Errorf("failed to remove destination directory %s: %w", dest, err)
 		}
 	}
 
 	// Create dest.
 	err = os.MkdirAll(dest, info.Mode())
 	if err != nil {
-		return errors.Wrapf(err, "failed to create destination directory %s", dest)
+		return fmt.Errorf("failed to create destination directory %s: %w", dest, err)
 	}
 
 	// Copy all files.
 	entries, err := ioutil.ReadDir(source)
 	if err != nil {
-		return errors.Wrapf(err, "failed to read source directory %s", source)
+		return fmt.Errorf("failed to read source directory %s: %w", source, err)
 	}
 
 	for _, entry := range entries {
-
 		sourcePath := filepath.Join(source, entry.Name())
 		destPath := filepath.Join(dest, entry.Name())
 
 		if entry.IsDir() {
 			err := DirCopy(sourcePath, destPath)
 			if err != nil {
-				return errors.Wrapf(err, "failed to copy sub-directory from %s to %s", sourcePath, destPath)
+				return fmt.Errorf("failed to copy sub-directory from %s to %s: %w", sourcePath, destPath, err)
 			}
 		} else {
 			err := FileCopy(sourcePath, destPath)
 			if err != nil {
-				return errors.Wrapf(err, "failed to copy file from %s to %s", sourcePath, destPath)
+				return fmt.Errorf("failed to copy file from %s to %s: %w", sourcePath, destPath, err)
 			}
 		}
-
 	}
 
 	return nil
@@ -456,10 +558,6 @@ func IsSnapshot(name string) bool {
 	return strings.Contains(name, SnapshotDelimiter)
 }
 
-func ExtractSnapshotName(name string) string {
-	return strings.SplitN(name, SnapshotDelimiter, 2)[1]
-}
-
 func MkdirAllOwner(path string, perm os.FileMode, uid int, gid int) error {
 	// This function is a slightly modified version of MkdirAll from the Go standard library.
 	// https://golang.org/src/os/path.go?s=488:535#L9
@@ -470,6 +568,7 @@ func MkdirAllOwner(path string, perm os.FileMode, uid int, gid int) error {
 		if dir.IsDir() {
 			return nil
 		}
+
 		return fmt.Errorf("path exists but isn't a directory")
 	}
 
@@ -507,11 +606,20 @@ func MkdirAllOwner(path string, perm os.FileMode, uid int, gid int) error {
 		if err1 == nil && dir.IsDir() {
 			return nil
 		}
+
 		return err
 	}
+
 	return nil
 }
 
+// HasKey returns true if map has key.
+func HasKey[K comparable, V any](key K, m map[K]V) bool {
+	_, found := m[key]
+
+	return found
+}
+
 func StringInSlice(key string, list []string) bool {
 	for _, entry := range list {
 		if entry == key {
@@ -521,6 +629,38 @@ func StringInSlice(key string, list []string) bool {
 	return false
 }
 
+// RemoveElementsFromStringSlice returns a slice equivalent to removing the given elements from the given list.
+// Elements not present in the list are ignored.
+func RemoveElementsFromStringSlice(list []string, elements ...string) []string {
+	for i := len(elements) - 1; i >= 0; i-- {
+		element := elements[i]
+		match := false
+		for j := len(list) - 1; j >= 0; j-- {
+			if element == list[j] {
+				match = true
+				list = append(list[:j], list[j+1:]...)
+				break
+			}
+		}
+
+		if match {
+			elements = append(elements[:i], elements[i+1:]...)
+		}
+	}
+
+	return list
+}
+
+// StringHasPrefix returns true if value has one of the supplied prefixes.
+func StringHasPrefix(value string, prefixes ...string) bool {
+	for _, prefix := range prefixes {
+		if strings.HasPrefix(value, prefix) {
+			return true
+		}
+	}
+	return false
+}
+
 func IntInSlice(key int, list []int) bool {
 	for _, entry := range list {
 		if entry == key {
@@ -539,26 +679,49 @@ func Int64InSlice(key int64, list []int64) bool {
 	return false
 }
 
-func IsTrue(value string) bool {
-	if StringInSlice(strings.ToLower(value), []string{"true", "1", "yes", "on"}) {
-		return true
+func Uint64InSlice(key uint64, list []uint64) bool {
+	for _, entry := range list {
+		if entry == key {
+			return true
+		}
 	}
-
 	return false
 }
 
-func IsUnixDev(path string) bool {
-	stat, err := os.Stat(path)
-	if err != nil {
-		return false
+// IsTrue returns true if value is "true", "1", "yes" or "on" (case insensitive).
+func IsTrue(value string) bool {
+	return StringInSlice(strings.ToLower(value), []string{"true", "1", "yes", "on"})
+}
 
+// IsTrueOrEmpty returns true if value is empty or if IsTrue() returns true.
+func IsTrueOrEmpty(value string) bool {
+	return value == "" || IsTrue(value)
+}
+
+// IsFalse returns true if value is "false", "0", "no" or "off" (case insensitive).
+func IsFalse(value string) bool {
+	return StringInSlice(strings.ToLower(value), []string{"false", "0", "no", "off"})
+}
+
+// IsFalseOrEmpty returns true if value is empty or if IsFalse() returns true.
+func IsFalseOrEmpty(value string) bool {
+	return value == "" || IsFalse(value)
+}
+
+func IsUserConfig(key string) bool {
+	return strings.HasPrefix(key, "user.")
+}
+
+// StringMapHasStringKey returns true if any of the supplied keys are present in the map.
+func StringMapHasStringKey(m map[string]string, keys ...string) bool {
+	for _, k := range keys {
+		_, ok := m[k]
+		if ok {
+			return true
+		}
 	}
 
-	if (stat.Mode() & os.ModeDevice) == 0 {
-		return false
-	}
-
-	return true
+	return false
 }
 
 func IsBlockdev(fm os.FileMode) bool {
@@ -575,25 +738,18 @@ func IsBlockdevPath(pathName string) bool {
 	return ((fm&os.ModeDevice != 0) && (fm&os.ModeCharDevice == 0))
 }
 
-func BlockFsDetect(dev string) (string, error) {
-	out, err := RunCommand("blkid", "-s", "TYPE", "-o", "value", dev)
-	if err != nil {
-		return "", err
-	}
-
-	return strings.TrimSpace(out), nil
-}
-
 // DeepCopy copies src to dest by using encoding/gob so its not that fast.
-func DeepCopy(src, dest interface{}) error {
+func DeepCopy(src, dest any) error {
 	buff := new(bytes.Buffer)
 	enc := gob.NewEncoder(buff)
 	dec := gob.NewDecoder(buff)
-	if err := enc.Encode(src); err != nil {
+	err := enc.Encode(src)
+	if err != nil {
 		return err
 	}
 
-	if err := dec.Decode(dest); err != nil {
+	err = dec.Decode(dest)
+	if err != nil {
 		return err
 	}
 
@@ -605,7 +761,8 @@ func RunningInUserNS() bool {
 	if err != nil {
 		return false
 	}
-	defer file.Close()
+
+	defer func() { _ = file.Close() }()
 
 	buf := bufio.NewReader(file)
 	l, _, err := buf.ReadLine()
@@ -615,43 +772,15 @@ func RunningInUserNS() bool {
 
 	line := string(l)
 	var a, b, c int64
-	fmt.Sscanf(line, "%d %d %d", &a, &b, &c)
+	_, _ = fmt.Sscanf(line, "%d %d %d", &a, &b, &c)
 	if a == 0 && b == 0 && c == 4294967295 {
 		return false
 	}
-	return true
-}
-
-func ValidHostname(name string) bool {
-	// Validate length
-	if len(name) < 1 || len(name) > 63 {
-		return false
-	}
-
-	// Validate first character
-	if strings.HasPrefix(name, "-") {
-		return false
-	}
-
-	if _, err := strconv.Atoi(string(name[0])); err == nil {
-		return false
-	}
-
-	// Validate last character
-	if strings.HasSuffix(name, "-") {
-		return false
-	}
-
-	// Validate the character set
-	match, _ := regexp.MatchString("^[-a-zA-Z0-9]*$", name)
-	if !match {
-		return false
-	}
 
 	return true
 }
 
-// Spawn the editor with a temporary YAML file for editing configs
+// Spawn the editor with a temporary YAML file for editing configs.
 func TextEditor(inPath string, inContent []byte) ([]byte, error) {
 	var f *os.File
 	var err error
@@ -682,19 +811,36 @@ func TextEditor(inPath string, inContent []byte) ([]byte, error) {
 			return []byte{}, err
 		}
 
+		revert := revert.New()
+		defer revert.Fail()
+		revert.Add(func() {
+			_ = f.Close()
+			_ = os.Remove(f.Name())
+		})
+
 		err = os.Chmod(f.Name(), 0600)
 		if err != nil {
-			f.Close()
-			os.Remove(f.Name())
 			return []byte{}, err
 		}
 
-		f.Write(inContent)
-		f.Close()
+		_, err = f.Write(inContent)
+		if err != nil {
+			return []byte{}, err
+		}
+
+		err = f.Close()
+		if err != nil {
+			return []byte{}, err
+		}
 
 		path = fmt.Sprintf("%s.yaml", f.Name())
-		os.Rename(f.Name(), path)
-		defer os.Remove(path)
+		err = os.Rename(f.Name(), path)
+		if err != nil {
+			return []byte{}, err
+		}
+
+		revert.Success()
+		revert.Add(func() { _ = os.Remove(path) })
 	} else {
 		path = inPath
 	}
@@ -717,8 +863,8 @@ func TextEditor(inPath string, inContent []byte) ([]byte, error) {
 	return content, nil
 }
 
-func ParseMetadata(metadata interface{}) (map[string]interface{}, error) {
-	newMetadata := make(map[string]interface{})
+func ParseMetadata(metadata any) (map[string]any, error) {
+	newMetadata := make(map[string]any)
 	s := reflect.ValueOf(metadata)
 	if !s.IsValid() {
 		return nil, nil
@@ -729,6 +875,7 @@ func ParseMetadata(metadata interface{}) (map[string]interface{}, error) {
 			if k.Kind() != reflect.String {
 				return nil, fmt.Errorf("Invalid metadata provided (key isn't a string)")
 			}
+
 			newMetadata[k.String()] = s.MapIndex(k).Interface()
 		}
 	} else if s.Kind() == reflect.Ptr && !s.Elem().IsValid() {
@@ -740,160 +887,6 @@ func ParseMetadata(metadata interface{}) (map[string]interface{}, error) {
 	return newMetadata, nil
 }
 
-// Parse a size string in bytes (e.g. 200kB or 5GB) into the number of bytes it
-// represents. Supports suffixes up to EB. "" == 0.
-func ParseByteSizeString(input string) (int64, error) {
-	// Empty input
-	if input == "" {
-		return 0, nil
-	}
-
-	// Find where the suffix begins
-	suffixLen := 0
-	for i, chr := range []byte(input) {
-		_, err := strconv.Atoi(string([]byte{chr}))
-		if err != nil {
-			suffixLen = len(input) - i
-			break
-		}
-	}
-
-	if suffixLen == len(input) {
-		return -1, fmt.Errorf("Invalid value: %s", input)
-	}
-
-	// Extract the suffix
-	suffix := input[len(input)-suffixLen:]
-
-	// Extract the value
-	value := input[0 : len(input)-suffixLen]
-	valueInt, err := strconv.ParseInt(value, 10, 64)
-	if err != nil {
-		return -1, fmt.Errorf("Invalid integer: %s", input)
-	}
-
-	// Figure out the multiplicator
-	multiplicator := int64(0)
-	switch suffix {
-	case "", "B", " bytes":
-		multiplicator = 1
-	case "kB":
-		multiplicator = 1000
-	case "MB":
-		multiplicator = 1000 * 1000
-	case "GB":
-		multiplicator = 1000 * 1000 * 1000
-	case "TB":
-		multiplicator = 1000 * 1000 * 1000 * 1000
-	case "PB":
-		multiplicator = 1000 * 1000 * 1000 * 1000 * 1000
-	case "EB":
-		multiplicator = 1000 * 1000 * 1000 * 1000 * 1000 * 1000
-	case "KiB":
-		multiplicator = 1024
-	case "MiB":
-		multiplicator = 1024 * 1024
-	case "GiB":
-		multiplicator = 1024 * 1024 * 1024
-	case "TiB":
-		multiplicator = 1024 * 1024 * 1024 * 1024
-	case "PiB":
-		multiplicator = 1024 * 1024 * 1024 * 1024 * 1024
-	case "EiB":
-		multiplicator = 1024 * 1024 * 1024 * 1024 * 1024 * 1024
-	default:
-		return -1, fmt.Errorf("Invalid value: %s", input)
-	}
-
-	return valueInt * multiplicator, nil
-}
-
-// Parse a size string in bits (e.g. 200kbit or 5Gbit) into the number of bits
-// it represents. Supports suffixes up to Ebit. "" == 0.
-func ParseBitSizeString(input string) (int64, error) {
-	// Empty input
-	if input == "" {
-		return 0, nil
-	}
-
-	// Find where the suffix begins
-	suffixLen := 0
-	for i, chr := range []byte(input) {
-		_, err := strconv.Atoi(string([]byte{chr}))
-		if err != nil {
-			suffixLen = len(input) - i
-			break
-		}
-	}
-
-	if suffixLen == len(input) {
-		return -1, fmt.Errorf("Invalid value: %s", input)
-	}
-
-	// Extract the suffix
-	suffix := input[len(input)-suffixLen:]
-
-	// Extract the value
-	value := input[0 : len(input)-suffixLen]
-	valueInt, err := strconv.ParseInt(value, 10, 64)
-	if err != nil {
-		return -1, fmt.Errorf("Invalid integer: %s", input)
-	}
-
-	// Figure out the multiplicator
-	multiplicator := int64(0)
-	switch suffix {
-	case "", "bit":
-		multiplicator = 1
-	case "kbit":
-		multiplicator = 1000
-	case "Mbit":
-		multiplicator = 1000 * 1000
-	case "Gbit":
-		multiplicator = 1000 * 1000 * 1000
-	case "Tbit":
-		multiplicator = 1000 * 1000 * 1000 * 1000
-	case "Pbit":
-		multiplicator = 1000 * 1000 * 1000 * 1000 * 1000
-	case "Ebit":
-		multiplicator = 1000 * 1000 * 1000 * 1000 * 1000 * 1000
-	case "Kibit":
-		multiplicator = 1024
-	case "Mibit":
-		multiplicator = 1024 * 1024
-	case "Gibit":
-		multiplicator = 1024 * 1024 * 1024
-	case "Tibit":
-		multiplicator = 1024 * 1024 * 1024 * 1024
-	case "Pibit":
-		multiplicator = 1024 * 1024 * 1024 * 1024 * 1024
-	case "Eibit":
-		multiplicator = 1024 * 1024 * 1024 * 1024 * 1024 * 1024
-
-	default:
-		return -1, fmt.Errorf("Unsupported suffix: %s", suffix)
-	}
-
-	return valueInt * multiplicator, nil
-}
-
-func GetByteSizeString(input int64, precision uint) string {
-	if input < 1000 {
-		return fmt.Sprintf("%dB", input)
-	}
-
-	value := float64(input)
-
-	for _, unit := range []string{"kB", "MB", "GB", "TB", "PB", "EB"} {
-		value = value / 1000
-		if value < 1000 {
-			return fmt.Sprintf("%.*f%s", precision, value, unit)
-		}
-	}
-
-	return fmt.Sprintf("%.*fEB", precision, value)
-}
-
 // RemoveDuplicatesFromString removes all duplicates of the string 'sep'
 // from the specified string 's'.  Leading and trailing occurrences of sep
 // are NOT removed (duplicate leading/trailing are).  Performs poorly if
@@ -903,33 +896,115 @@ func RemoveDuplicatesFromString(s string, sep string) string {
 	for s = strings.Replace(s, dup, sep, -1); strings.Contains(s, dup); s = strings.Replace(s, dup, sep, -1) {
 
 	}
+
 	return s
 }
 
+// RunError is the error from the RunCommand family of functions.
 type RunError struct {
-	msg string
-	Err error
+	cmd    string
+	args   []string
+	err    error
+	stdout *bytes.Buffer
+	stderr *bytes.Buffer
 }
 
 func (e RunError) Error() string {
-	return e.msg
-}
-
-func RunCommand(name string, arg ...string) (string, error) {
-	output, err := exec.Command(name, arg...).CombinedOutput()
-	if err != nil {
-		err := RunError{
-			msg: fmt.Sprintf("Failed to run: %s %s: %s", name, strings.Join(arg, " "), strings.TrimSpace(string(output))),
-			Err: err,
-		}
-		return string(output), err
+	if e.stderr.Len() == 0 {
+		return fmt.Sprintf("Failed to run: %s %s: %v", e.cmd, strings.Join(e.args, " "), e.err)
 	}
 
-	return string(output), nil
+	return fmt.Sprintf("Failed to run: %s %s: %v (%s)", e.cmd, strings.Join(e.args, " "), e.err, strings.TrimSpace(e.stderr.String()))
 }
 
-func RunCommandWithFds(stdin io.Reader, stdout io.Writer, name string, arg ...string) error {
-	cmd := exec.Command(name, arg...)
+func (e RunError) Unwrap() error {
+	return e.err
+}
+
+// StdOut returns the stdout buffer.
+func (e RunError) StdOut() *bytes.Buffer {
+	return e.stdout
+}
+
+// StdErr returns the stdout buffer.
+func (e RunError) StdErr() *bytes.Buffer {
+	return e.stderr
+}
+
+// NewRunError returns new RunError.
+func NewRunError(cmd string, args []string, err error, stdout *bytes.Buffer, stderr *bytes.Buffer) error {
+	return RunError{
+		cmd:    cmd,
+		args:   args,
+		err:    err,
+		stdout: stdout,
+		stderr: stderr,
+	}
+}
+
+// RunCommandSplit runs a command with a supplied environment and optional arguments and returns the
+// resulting stdout and stderr output as separate variables. If the supplied environment is nil then
+// the default environment is used. If the command fails to start or returns a non-zero exit code
+// then an error is returned containing the output of stderr too.
+func RunCommandSplit(ctx context.Context, env []string, filesInherit []*os.File, name string, arg ...string) (string, string, error) {
+	cmd := exec.CommandContext(ctx, name, arg...)
+
+	if env != nil {
+		cmd.Env = env
+	}
+
+	if filesInherit != nil {
+		cmd.ExtraFiles = filesInherit
+	}
+
+	var stdout bytes.Buffer
+	var stderr bytes.Buffer
+	cmd.Stdout = &stdout
+	cmd.Stderr = &stderr
+
+	err := cmd.Run()
+	if err != nil {
+		return stdout.String(), stderr.String(), NewRunError(name, arg, err, &stdout, &stderr)
+	}
+
+	return stdout.String(), stderr.String(), nil
+}
+
+// RunCommandContext runs a command with optional arguments and returns stdout. If the command fails to
+// start or returns a non-zero exit code then an error is returned containing the output of stderr.
+func RunCommandContext(ctx context.Context, name string, arg ...string) (string, error) {
+	stdout, _, err := RunCommandSplit(ctx, nil, nil, name, arg...)
+	return stdout, err
+}
+
+// RunCommand runs a command with optional arguments and returns stdout. If the command fails to
+// start or returns a non-zero exit code then an error is returned containing the output of stderr.
+// Deprecated: Use RunCommandContext.
+func RunCommand(name string, arg ...string) (string, error) {
+	stdout, _, err := RunCommandSplit(context.TODO(), nil, nil, name, arg...)
+	return stdout, err
+}
+
+// RunCommandInheritFds runs a command with optional arguments and passes a set
+// of file descriptors to the newly created process, returning stdout. If the
+// command fails to start or returns a non-zero exit code then an error is
+// returned containing the output of stderr.
+func RunCommandInheritFds(ctx context.Context, filesInherit []*os.File, name string, arg ...string) (string, error) {
+	stdout, _, err := RunCommandSplit(ctx, nil, filesInherit, name, arg...)
+	return stdout, err
+}
+
+// RunCommandCLocale runs a command with a LANG=C.UTF-8 environment set with optional arguments and
+// returns stdout. If the command fails to start or returns a non-zero exit code then an error is
+// returned containing the output of stderr.
+func RunCommandCLocale(name string, arg ...string) (string, error) {
+	stdout, _, err := RunCommandSplit(context.TODO(), append(os.Environ(), "LANG=C.UTF-8"), nil, name, arg...)
+	return stdout, err
+}
+
+// RunCommandWithFds runs a command with supplied file descriptors.
+func RunCommandWithFds(ctx context.Context, stdin io.Reader, stdout io.Writer, name string, arg ...string) error {
+	cmd := exec.CommandContext(ctx, name, arg...)
 
 	if stdin != nil {
 		cmd.Stdin = stdin
@@ -944,18 +1019,14 @@ func RunCommandWithFds(stdin io.Reader, stdout io.Writer, name string, arg ...st
 
 	err := cmd.Run()
 	if err != nil {
-		err := RunError{
-			msg: fmt.Sprintf("Failed to run: %s %s: %s", name, strings.Join(arg, " "),
-				strings.TrimSpace(buffer.String())),
-			Err: err,
-		}
-
-		return err
+		return NewRunError(name, arg, err, nil, &buffer)
 	}
 
 	return nil
 }
 
+// TryRunCommand runs the specified command up to 20 times with a 500ms delay between each call
+// until it runs without an error. If after 20 times it is still failing then returns the error.
 func TryRunCommand(name string, arg ...string) (string, error) {
 	var err error
 	var output string
@@ -984,21 +1055,9 @@ func TimeIsSet(ts time.Time) bool {
 	return true
 }
 
-// WriteTempFile creates a temp file with the specified content
-func WriteTempFile(dir string, prefix string, content string) (string, error) {
-	f, err := ioutil.TempFile(dir, prefix)
-	if err != nil {
-		return "", err
-	}
-	defer f.Close()
-
-	_, err = f.WriteString(content)
-	return f.Name(), err
-}
-
 // EscapePathFstab escapes a path fstab-style.
 // This ensures that getmntent_r() and friends can correctly parse stuff like
-// /some/wacky path with spaces /some/wacky target with spaces
+// /some/wacky path with spaces /some/wacky target with spaces.
 func EscapePathFstab(path string) string {
 	r := strings.NewReplacer(
 		" ", "\\040",
@@ -1008,20 +1067,45 @@ func EscapePathFstab(path string) string {
 	return r.Replace(path)
 }
 
-func DownloadFileSha256(httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.Canceler, filename string, url string, hash string, target io.WriteSeeker) (int64, error) {
-	return downloadFileSha(httpClient, useragent, progress, canceler, filename, url, hash, sha256.New(), target)
+func SetProgressMetadata(metadata map[string]any, stage, displayPrefix string, percent, processed, speed int64) {
+	progress := make(map[string]string)
+	// stage, percent, speed sent for API callers.
+	progress["stage"] = stage
+	if processed > 0 {
+		progress["processed"] = strconv.FormatInt(processed, 10)
+	}
+
+	if percent > 0 {
+		progress["percent"] = strconv.FormatInt(percent, 10)
+	}
+
+	progress["speed"] = strconv.FormatInt(speed, 10)
+	metadata["progress"] = progress
+
+	// <stage>_progress with formatted text sent for lxc cli.
+	if percent > 0 {
+		metadata[stage+"_progress"] = fmt.Sprintf("%s: %d%% (%s/s)", displayPrefix, percent, units.GetByteSizeString(speed, 2))
+	} else if processed > 0 {
+		metadata[stage+"_progress"] = fmt.Sprintf("%s: %s (%s/s)", displayPrefix, units.GetByteSizeString(processed, 2), units.GetByteSizeString(speed, 2))
+	} else {
+		metadata[stage+"_progress"] = fmt.Sprintf("%s: %s/s", displayPrefix, units.GetByteSizeString(speed, 2))
+	}
 }
 
-func DownloadFileSha512(httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.Canceler, filename string, url string, hash string, target io.WriteSeeker) (int64, error) {
-	return downloadFileSha(httpClient, useragent, progress, canceler, filename, url, hash, sha512.New(), target)
-}
-
-func downloadFileSha(httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.Canceler, filename string, url string, hash string, sha hash.Hash, target io.WriteSeeker) (int64, error) {
+func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.HTTPRequestCanceller, filename string, url string, hash string, hashFunc hash.Hash, target io.WriteSeeker) (int64, error) {
 	// Always seek to the beginning
-	target.Seek(0, 0)
+	_, _ = target.Seek(0, 0)
+
+	var req *http.Request
+	var err error
 
 	// Prepare the download request
-	req, err := http.NewRequest("GET", url, nil)
+	if ctx != nil {
+		req, err = http.NewRequestWithContext(ctx, "GET", url, nil)
+	} else {
+		req, err = http.NewRequest("GET", url, nil)
+	}
+
 	if err != nil {
 		return -1, err
 	}
@@ -1035,7 +1119,8 @@ func downloadFileSha(httpClient *http.Client, useragent string, progress func(pr
 	if err != nil {
 		return -1, err
 	}
-	defer r.Body.Close()
+
+	defer func() { _ = r.Body.Close() }()
 	defer close(doneCh)
 
 	if r.StatusCode != http.StatusOK {
@@ -1051,35 +1136,52 @@ func downloadFileSha(httpClient *http.Client, useragent string, progress func(pr
 				Length: r.ContentLength,
 				Handler: func(percent int64, speed int64) {
 					if filename != "" {
-						progress(ioprogress.ProgressData{Text: fmt.Sprintf("%s: %d%% (%s/s)", filename, percent, GetByteSizeString(speed, 2))})
+						progress(ioprogress.ProgressData{Text: fmt.Sprintf("%s: %d%% (%s/s)", filename, percent, units.GetByteSizeString(speed, 2))})
 					} else {
-						progress(ioprogress.ProgressData{Text: fmt.Sprintf("%d%% (%s/s)", percent, GetByteSizeString(speed, 2))})
+						progress(ioprogress.ProgressData{Text: fmt.Sprintf("%d%% (%s/s)", percent, units.GetByteSizeString(speed, 2))})
 					}
 				},
 			},
 		}
 	}
 
-	size, err := io.Copy(io.MultiWriter(target, sha), body)
-	if err != nil {
-		return -1, err
-	}
+	var size int64
 
-	result := fmt.Sprintf("%x", sha.Sum(nil))
-	if result != hash {
-		return -1, fmt.Errorf("Hash mismatch for %s: %s != %s", url, result, hash)
+	if hashFunc != nil {
+		size, err = io.Copy(io.MultiWriter(target, hashFunc), body)
+		if err != nil {
+			return -1, err
+		}
+
+		result := fmt.Sprintf("%x", hashFunc.Sum(nil))
+		if result != hash {
+			return -1, fmt.Errorf("Hash mismatch for %s: %s != %s", url, result, hash)
+		}
+	} else {
+		size, err = io.Copy(target, body)
+		if err != nil {
+			return -1, err
+		}
 	}
 
 	return size, nil
 }
 
 func ParseNumberFromFile(file string) (int64, error) {
-	buf, err := ioutil.ReadFile(file)
+	f, err := os.Open(file)
 	if err != nil {
 		return int64(0), err
 	}
 
-	str := strings.TrimSpace(string(buf))
+	defer func() { _ = f.Close() }()
+
+	buf := make([]byte, 4096)
+	n, err := f.Read(buf)
+	if err != nil {
+		return int64(0), err
+	}
+
+	str := strings.TrimSpace(string(buf[0:n]))
 	nr, err := strconv.Atoi(str)
 	if err != nil {
 		return int64(0), err
@@ -1104,3 +1206,112 @@ func (r *ReadSeeker) Read(p []byte) (n int, err error) {
 func (r *ReadSeeker) Seek(offset int64, whence int) (int64, error) {
 	return r.Seeker.Seek(offset, whence)
 }
+
+// RenderTemplate renders a pongo2 template.
+func RenderTemplate(template string, ctx pongo2.Context) (string, error) {
+	// Load template from string
+	tpl, err := pongo2.FromString("{% autoescape off %}" + template + "{% endautoescape %}")
+	if err != nil {
+		return "", err
+	}
+
+	// Get rendered template
+	ret, err := tpl.Execute(ctx)
+	if err != nil {
+		return ret, err
+	}
+
+	// Looks like we're nesting templates so run pongo again
+	if strings.Contains(ret, "{{") || strings.Contains(ret, "{%") {
+		return RenderTemplate(ret, ctx)
+	}
+
+	return ret, err
+}
+
+func GetSnapshotExpiry(refDate time.Time, s string) (time.Time, error) {
+	expr := strings.TrimSpace(s)
+
+	if expr == "" {
+		return time.Time{}, nil
+	}
+
+	re := regexp.MustCompile(`^(\d+)(M|H|d|w|m|y)$`)
+	expiry := map[string]int{
+		"M": 0,
+		"H": 0,
+		"d": 0,
+		"w": 0,
+		"m": 0,
+		"y": 0,
+	}
+
+	values := strings.Split(expr, " ")
+
+	if len(values) == 0 {
+		return time.Time{}, nil
+	}
+
+	for _, value := range values {
+		fields := re.FindStringSubmatch(value)
+		if fields == nil {
+			return time.Time{}, fmt.Errorf("Invalid expiry expression")
+		}
+
+		if expiry[fields[2]] > 0 {
+			// We don't allow fields to be set multiple times
+			return time.Time{}, fmt.Errorf("Invalid expiry expression")
+		}
+
+		val, err := strconv.Atoi(fields[1])
+		if err != nil {
+			return time.Time{}, err
+		}
+
+		expiry[fields[2]] = val
+	}
+
+	t := refDate.AddDate(expiry["y"], expiry["m"], expiry["d"]+expiry["w"]*7).Add(
+		time.Hour*time.Duration(expiry["H"]) + time.Minute*time.Duration(expiry["M"]))
+
+	return t, nil
+}
+
+// InSnap returns true if we're running inside the LXD snap.
+func InSnap() bool {
+	// Detect the snap.
+	_, snapPath := os.LookupEnv("SNAP")
+	snapName := os.Getenv("SNAP_NAME")
+	if snapPath && snapName == "lxd" {
+		return true
+	}
+
+	return false
+}
+
+// JoinUrlPath return the join of the input urls/paths sanitized.
+func JoinUrls(baseUrl, p string) (string, error) {
+	u, err := url.Parse(baseUrl)
+	if err != nil {
+		return "", err
+	}
+
+	u.Path = path.Join(u.Path, p)
+	return u.String(), nil
+}
+
+// SplitNTrimSpace returns result of strings.SplitN() and then strings.TrimSpace() on each element.
+// Accepts nilIfEmpty argument which if true, will return nil slice if s is empty (after trimming space).
+func SplitNTrimSpace(s string, sep string, n int, nilIfEmpty bool) []string {
+	if nilIfEmpty && strings.TrimSpace(s) == "" {
+		return nil
+	}
+
+	parts := strings.SplitN(s, sep, n)
+
+	for i, v := range parts {
+		parts[i] = strings.TrimSpace(v)
+	}
+
+	return parts
+}
