@@ -6,11 +6,12 @@ lxc() {
 
 lxc_remote() {
     set +x
-    # shellcheck disable=SC2039
+    # shellcheck disable=SC2039,3043
     local injected cmd arg
 
     injected=0
-    cmd=$(which lxc)
+    # find the path to lxc binary, not the shell wrapper function
+    cmd=$(unset -f lxc; command -v lxc)
 
     # shellcheck disable=SC2048,SC2068
     for arg in "$@"; do
@@ -44,7 +45,7 @@ gen_cert() {
     [ -f "${LXD_CONF}/${1}.crt" ] && return
     mv "${LXD_CONF}/client.crt" "${LXD_CONF}/client.crt.bak"
     mv "${LXD_CONF}/client.key" "${LXD_CONF}/client.key.bak"
-    echo y | lxc_remote remote add "$(uuidgen)" https://0.0.0.0 || true
+    echo y | lxc_remote remote add "remote-placeholder-$$" https://0.0.0.0 || true
     mv "${LXD_CONF}/client.crt" "${LXD_CONF}/${1}.crt"
     mv "${LXD_CONF}/client.key" "${LXD_CONF}/${1}.key"
     mv "${LXD_CONF}/client.crt.bak" "${LXD_CONF}/client.crt"
