@@ -5,7 +5,7 @@ spawn_lxd() {
     # LXD_DIR is local here because since $(lxc) is actually a function, it
     # overwrites the environment and we would lose LXD_DIR's value otherwise.
 
-    # shellcheck disable=2039
+    # shellcheck disable=2039,3043
     local LXD_DIR lxddir lxd_backend
 
     lxddir=${1}
@@ -26,15 +26,6 @@ spawn_lxd() {
         exit 1
     fi
 
-    # Copy pre generated Certs (either the default one or an alternate one)
-    if [ "${LXD_ALT_CERT}" = "" ]; then
-        cp deps/server.crt "${lxddir}"
-        cp deps/server.key "${lxddir}"
-    else
-        cp deps/server-alt.crt "${lxddir}"/server.crt
-        cp deps/server-alt.key "${lxddir}"/server.key
-    fi
-
     # setup storage
     "$lxd_backend"_setup "${lxddir}"
     echo "$lxd_backend" > "${lxddir}/lxd.backend"
@@ -55,8 +46,8 @@ spawn_lxd() {
     echo "${lxddir}" >> "${TEST_DIR}/daemons"
     echo "==> Spawned LXD (PID is ${LXD_PID})"
 
-    echo "==> Confirming lxd is responsive"
-    LXD_DIR="${lxddir}" lxd waitready --timeout=300
+    echo "==> Confirming lxd is responsive (PID is ${LXD_PID})"
+    LXD_DIR="${lxddir}" lxd waitready --timeout=300 || (echo "Killing PID ${LXD_PID}" ; kill -9 "${LXD_PID}" ; false)
 
     if [ "${LXD_NETNS}" = "" ]; then
         echo "==> Binding to network"
@@ -92,7 +83,7 @@ respawn_lxd() {
     # LXD_DIR is local here because since $(lxc) is actually a function, it
     # overwrites the environment and we would lose LXD_DIR's value otherwise.
 
-    # shellcheck disable=2039
+    # shellcheck disable=2039,3043
     local LXD_DIR
 
     lxddir=${1}
@@ -114,8 +105,12 @@ respawn_lxd() {
     echo "==> Spawned LXD (PID is ${LXD_PID})"
 
     if [ "${wait}" = true ]; then
-        echo "==> Confirming lxd is responsive"
-        LXD_DIR="${lxddir}" lxd waitready --timeout=300
+        echo "==> Confirming lxd is responsive (PID is ${LXD_PID})"
+        LXD_DIR="${lxddir}" lxd waitready --timeout=300 || (echo "Killing PID ${LXD_PID}" ; kill -9 "${LXD_PID}" ; false)
+    fi
+
+    if [ -n "${DEBUG:-}" ]; then
+        set -x
     fi
 }
 
@@ -123,7 +118,7 @@ kill_lxd() {
     # LXD_DIR is local here because since $(lxc) is actually a function, it
     # overwrites the environment and we would lose LXD_DIR's value otherwise.
 
-    # shellcheck disable=2039
+    # shellcheck disable=2039,3043
     local LXD_DIR daemon_dir daemon_pid check_leftovers lxd_backend
 
     daemon_dir=${1}
@@ -137,41 +132,41 @@ kill_lxd() {
     daemon_pid=$(cat "${daemon_dir}/lxd.pid")
     check_leftovers="false"
     lxd_backend=$(storage_backend "$daemon_dir")
-    echo "==> Killing LXD at ${daemon_dir}"
+    echo "==> Killing LXD at ${daemon_dir} (${daemon_pid})"
 
     if [ -e "${daemon_dir}/unix.socket" ]; then
         # Delete all containers
         echo "==> Deleting all containers"
-        for container in $(lxc list --fast --force-local | tail -n+3 | grep "^| " | cut -d' ' -f2); do
-            lxc delete "${container}" --force-local -f || true
+        for container in $(timeout -k 2 2 lxc list --force-local --format csv --columns n); do
+            timeout -k 10 10 lxc delete "${container}" --force-local -f || true
         done
 
         # Delete all images
         echo "==> Deleting all images"
-        for image in $(lxc image list --force-local | tail -n+3 | grep "^| " | cut -d'|' -f3 | sed "s/^ //g"); do
-            lxc image delete "${image}" --force-local || true
-        done
-
-        # Delete all networks
-        echo "==> Deleting all networks"
-        for network in $(lxc network list --force-local | grep YES | grep "^| " | cut -d' ' -f2); do
-            lxc network delete "${network}" --force-local || true
+        for image in $(timeout -k 2 2 lxc image list --force-local --format csv --columns f); do
+            timeout -k 10 10 lxc image delete "${image}" --force-local || true
         done
 
         # Delete all profiles
         echo "==> Deleting all profiles"
-        for profile in $(lxc profile list --force-local | tail -n+3 | grep "^| " | cut -d' ' -f2); do
-            lxc profile delete "${profile}" --force-local || true
+        for profile in $(timeout -k 2 2 lxc profile list --force-local --format csv | cut -d, -f1); do
+            timeout -k 10 10 lxc profile delete "${profile}" --force-local || true
+        done
+
+        # Delete all networks
+        echo "==> Deleting all networks"
+        for network in $(timeout -k 2 2 lxc network list --force-local --format csv | grep -F ',YES,' | cut -d, -f1); do
+            timeout -k 10 10 lxc network delete "${network}" --force-local || true
         done
 
         # Clear config of the default profile since the profile itself cannot
         # be deleted.
         echo "==> Clearing config of default profile"
-        printf 'config: {}\ndevices: {}' | lxc profile edit default
+        printf 'config: {}\ndevices: {}' | timeout -k 5 5 lxc profile edit default
 
         echo "==> Deleting all storage pools"
-        for storage in $(lxc storage list --force-local | tail -n+3 | grep "^| " | cut -d' ' -f2); do
-            lxc storage delete "${storage}" --force-local || true
+        for storage in $(timeout -k 2 2 lxc storage list --force-local --format csv | cut -d, -f1); do
+            timeout -k 20 20 lxc storage delete "${storage}" --force-local || true
         done
 
         echo "==> Checking for locked DB tables"
@@ -180,7 +175,9 @@ kill_lxd() {
         done
 
         # Kill the daemon
-        lxd shutdown || kill -9 "${daemon_pid}" 2>/dev/null || true
+        timeout -k 30 30 lxd shutdown || kill -9 "${daemon_pid}" 2>/dev/null || true
+
+        sleep 2
 
         # Cleanup shmounts (needed due to the forceful kill)
         find "${daemon_dir}" -name shmounts -exec "umount" "-l" "{}" \; >/dev/null 2>&1 || true
@@ -201,7 +198,7 @@ kill_lxd() {
         rm -f "${daemon_dir}/containers/lxc-monitord.log"
 
         # Support AppArmor policy cache directory
-        if apparmor_parser --help | grep -q -- --'print-cache.dir'; then
+        if apparmor_parser --help | grep -q -- '--print-cache.dir'; then
           apparmor_cache_dir="$(apparmor_parser -L "${daemon_dir}"/security/apparmor/cache --print-cache-dir)"
         else
           apparmor_cache_dir="${daemon_dir}/security/apparmor/cache"
@@ -219,11 +216,11 @@ kill_lxd() {
         check_empty "${daemon_dir}/snapshots/"
 
         echo "==> Checking for leftover DB entries"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "containers"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "containers_config"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "containers_devices"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "containers_devices_config"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "containers_profiles"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "instances"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "instances_config"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "instances_devices"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "instances_devices_config"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "instances_profiles"
         check_empty_table "${daemon_dir}/database/global/db.bin" "images"
         check_empty_table "${daemon_dir}/database/global/db.bin" "images_aliases"
         check_empty_table "${daemon_dir}/database/global/db.bin" "images_properties"
@@ -246,7 +243,6 @@ kill_lxd() {
     "$lxd_backend"_teardown "${daemon_dir}"
 
     # Wipe the daemon directory
-    sleep 2
     wipe "${daemon_dir}"
 
     # Remove the daemon from the list
@@ -257,25 +253,25 @@ shutdown_lxd() {
     # LXD_DIR is local here because since $(lxc) is actually a function, it
     # overwrites the environment and we would lose LXD_DIR's value otherwise.
 
-    # shellcheck disable=2039
+    # shellcheck disable=2039,3043
     local LXD_DIR
 
     daemon_dir=${1}
     # shellcheck disable=2034
     LXD_DIR=${daemon_dir}
     daemon_pid=$(cat "${daemon_dir}/lxd.pid")
-    echo "==> Killing LXD at ${daemon_dir}"
+    echo "==> Shutting down LXD at ${daemon_dir} (${daemon_pid})"
 
-    # Kill the daemon
+    # Shutting down the daemon
     lxd shutdown || kill -9 "${daemon_pid}" 2>/dev/null || true
 
     # Wait for any cleanup activity that might be happening right
     # after the websocket is closed.
-    sleep 2
+    sleep 0.5
 }
 
 wait_for() {
-    # shellcheck disable=SC2039
+    # shellcheck disable=SC2039,3043
     local addr op
 
     addr=${1}
@@ -285,14 +281,14 @@ wait_for() {
 }
 
 wipe() {
-    if which btrfs >/dev/null 2>&1; then
+    if command -v btrfs >/dev/null 2>&1; then
         rm -Rf "${1}" 2>/dev/null || true
         if [ -d "${1}" ]; then
             find "${1}" | tac | xargs btrfs subvolume delete >/dev/null 2>&1 || true
         fi
     fi
 
-    # shellcheck disable=SC2039
+    # shellcheck disable=SC2039,3043
     local pid
     # shellcheck disable=SC2009
     ps aux | grep lxc-monitord | grep "${1}" | awk '{print $2}' | while read -r pid; do
@@ -308,7 +304,7 @@ wipe() {
 
 # Kill and cleanup LXD instances and related resources
 cleanup_lxds() {
-    # shellcheck disable=SC2039
+    # shellcheck disable=SC2039,3043
     local test_dir daemon_dir
     test_dir="$1"
 
