@@ -1,8 +1,8 @@
 # Test setup helper functions.
 
 ensure_has_localhost_remote() {
-    # shellcheck disable=SC2039
-    local addr=${1}
+    # shellcheck disable=SC2039,3043
+    local addr="${1}"
     if ! lxc remote list | grep -q "localhost"; then
         lxc remote add localhost "https://${addr}" --accept-certificate --password foo
     fi
@@ -23,7 +23,8 @@ ensure_import_testimage() {
                 exit 1
             fi
 
-            deps/import-busybox --alias testimage
+            project=$(lxc project list | grep '(current)' | cut -d' ' -f2)
+            deps/import-busybox --alias testimage --project "$project"
         fi
     fi
 }
