@@ -1,10 +1,8 @@
 package main
 
 import (
-	"github.com/juju/schema"
-
 	"gopkg.in/juju/environschema.v1"
-	"gopkg.in/macaroon-bakery.v2/httpbakery/form"
+	"gopkg.in/macaroon-bakery.v3/httpbakery/form"
 )
 
 var schemaResponse = form.SchemaResponse{
@@ -24,13 +22,3 @@ var schemaFields = environschema.Fields{
 		Secret:      true,
 	},
 }
-
-var fieldsChecker = schema.FieldMap(validateSchema(schemaFields))
-
-func validateSchema(fields environschema.Fields) (schema.Fields, schema.Defaults) {
-	f, d, err := fields.ValidationSchema()
-	if err != nil {
-		panic(err)
-	}
-	return f, d
-}
