@@ -1,5 +1,5 @@
 test_basic_usage() {
-  # shellcheck disable=2039
+  # shellcheck disable=2039,3043
   local lxd_backend
   lxd_backend=$(storage_backend "$LXD_DIR")
 
@@ -48,8 +48,8 @@ test_basic_usage() {
   # Re-import the image
   mv "${LXD_DIR}/${sum}.tar.xz" "${LXD_DIR}/testimage.tar.xz"
   lxc image import "${LXD_DIR}/testimage.tar.xz" --alias testimage user.foo=bar --public
-  lxc image show testimage | grep -q "user.foo: bar"
-  lxc image show testimage | grep -q "public: true"
+  lxc image show testimage | grep -qF "user.foo: bar"
+  lxc image show testimage | grep -qF "public: true"
   lxc image delete testimage
   lxc image import "${LXD_DIR}/testimage.tar.xz" --alias testimage
   rm "${LXD_DIR}/testimage.tar.xz"
@@ -101,10 +101,21 @@ test_basic_usage() {
   lxc list --format json | jq '.[]|select(.name="foo")' | grep '"name": "foo"'
 
   # Test list with --columns and --fast
-  ! lxc list --columns=nsp --fast
+  ! lxc list --columns=nsp --fast || false
+
+  # Check volatile.apply_template is correct.
+  lxc config get foo volatile.apply_template | grep create
+
+  # Start the instance to clear apply_template.
+  lxc start foo
+  lxc stop foo -f
 
   # Test container rename
   lxc move foo bar
+
+  # Check volatile.apply_template is altered during rename.
+  lxc config get bar volatile.apply_template | grep rename
+
   lxc list | grep -v foo
   lxc list | grep bar
 
@@ -126,7 +137,7 @@ test_basic_usage() {
   # Test unprivileged container publish
   lxc publish bar --alias=foo-image prop1=val1
   lxc image show foo-image | grep val1
-  curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep "/1.0/images/" && false
+  curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/" && false
   lxc image delete foo-image
 
   # Test container publish with existing alias
@@ -143,9 +154,12 @@ test_basic_usage() {
   # Test image compression on publish
   lxc publish bar --alias=foo-image-compressed --compression=bzip2 prop=val1
   lxc image show foo-image-compressed | grep val1
-  curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep "/1.0/images/" && false
+  curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/" && false
   lxc image delete foo-image-compressed
 
+  # Test compression options
+  lxc publish bar --alias=foo-image-compressed --compression="gzip --rsyncable" prop=val1
+  lxc image delete foo-image-compressed
 
   # Test privileged container publish
   lxc profile create priv
@@ -153,7 +167,7 @@ test_basic_usage() {
   lxc init testimage barpriv -p default -p priv
   lxc publish barpriv --alias=foo-image prop1=val1
   lxc image show foo-image | grep val1
-  curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep "/1.0/images/" && false
+  curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/" && false
   lxc image delete foo-image
   lxc delete barpriv
   lxc profile delete priv
@@ -173,16 +187,16 @@ test_basic_usage() {
 
   # Test public images
   lxc publish --public bar --alias=foo-image2
-  curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep "/1.0/images/"
+  curl -k -s --cert "${LXD_CONF}/client3.crt" --key "${LXD_CONF}/client3.key" -X GET "https://${LXD_ADDR}/1.0/images" | grep -F "/1.0/images/"
   lxc image delete foo-image2
 
   # Test invalid container names
-  ! lxc init testimage -abc
-  ! lxc init testimage abc-
-  ! lxc init testimage 1234
-  ! lxc init testimage 12test
-  ! lxc init testimage a_b_c
-  ! lxc init testimage aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
+  ! lxc init testimage -abc || false
+  ! lxc init testimage abc- || false
+  ! lxc init testimage 1234 || false
+  ! lxc init testimage 12test || false
+  ! lxc init testimage a_b_c || false
+  ! lxc init testimage aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa || false
 
   # Test snapshot publish
   lxc snapshot bar
@@ -230,7 +244,7 @@ test_basic_usage() {
 
   # Test randomly named container creation
   lxc launch testimage
-  RDNAME=$(lxc list | tail -n2 | grep ^\| | awk '{print $2}')
+  RDNAME=$(lxc list --format csv --columns n)
   lxc delete -f "${RDNAME}"
 
   # Test "nonetype" container creation
@@ -254,50 +268,81 @@ test_basic_usage() {
     # shellcheck disable=SC2030
     LXD_DIR=${LXD_ACTIVATION_DIR}
     ensure_import_testimage
-    lxd activateifneeded --debug 2>&1 | grep -q "Daemon has core.https_address set, activating..."
+    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has core.https_address set, activating..."
     lxc config unset core.https_address --force-local
-    lxd activateifneeded --debug 2>&1 | grep -q -v "activating..."
+    lxd activateifneeded --debug 2>&1 | grep -qF -v "activating..."
     lxc init testimage autostart --force-local
-    lxd activateifneeded --debug 2>&1 | grep -q -v "activating..."
+    lxd activateifneeded --debug 2>&1 | grep -qF -v "activating..."
     lxc config set autostart boot.autostart true --force-local
 
     # Restart the daemon, this forces the global database to be dumped to disk.
     shutdown_lxd "${LXD_DIR}"
     respawn_lxd "${LXD_DIR}" true
-    lxc stop --force autostart
+    lxc stop --force autostart --force-local
 
-    lxd activateifneeded --debug 2>&1 | grep -q "Daemon has auto-started containers, activating..."
+    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has auto-started instances, activating..."
 
     lxc config unset autostart boot.autostart --force-local
-    lxd activateifneeded --debug 2>&1 | grep -q -v "activating..."
+    lxd activateifneeded --debug 2>&1 | grep -qF -v "activating..."
 
     lxc start autostart --force-local
-    PID=$(lxc info autostart --force-local | grep ^Pid | awk '{print $2}')
+    PID=$(lxc info autostart --force-local | awk '/^PID:/ {print $2}')
     shutdown_lxd "${LXD_DIR}"
     [ -d "/proc/${PID}" ] && false
 
-    lxd activateifneeded --debug 2>&1 | grep -q "Daemon has auto-started containers, activating..."
+    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has auto-started instances, activating..."
 
     # shellcheck disable=SC2031
     respawn_lxd "${LXD_DIR}" true
 
     lxc list --force-local autostart | grep -q RUNNING
 
+    # Check for scheduled instance snapshots
+    lxc stop --force autostart --force-local
+    lxc config set autostart snapshots.schedule "* * * * *" --force-local
+    shutdown_lxd "${LXD_DIR}"
+    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has scheduled instance snapshots, activating..."
+
+    # shellcheck disable=SC2031
+    respawn_lxd "${LXD_DIR}" true
+
+    lxc config unset autostart snapshots.schedule --force-local
+
+    # Check for scheduled volume snapshots
+    storage_pool="lxdtest-$(basename "${LXD_DIR}")"
+
+    lxc storage volume create "${storage_pool}" vol --force-local
+
+    shutdown_lxd "${LXD_DIR}"
+    lxd activateifneeded --debug 2>&1 | grep -qF -v "activating..."
+
+    # shellcheck disable=SC2031
+    respawn_lxd "${LXD_DIR}" true
+
+    lxc storage volume set "${storage_pool}" vol snapshots.schedule="* * * * *" --force-local
+
+    shutdown_lxd "${LXD_DIR}"
+    lxd activateifneeded --debug 2>&1 | grep -qF "Daemon has scheduled volume snapshots, activating..."
+
+    # shellcheck disable=SC2031
+    respawn_lxd "${LXD_DIR}" true
+
     lxc delete autostart --force --force-local
+    lxc storage volume delete "${storage_pool}" vol --force-local
   )
-  # shellcheck disable=SC2031
+  # shellcheck disable=SC2031,2269
   LXD_DIR=${LXD_DIR}
   kill_lxd "${LXD_ACTIVATION_DIR}"
 
   # Create and start a container
   lxc launch testimage foo
   lxc list | grep foo | grep RUNNING
-  lxc stop foo --force  # stop is hanging
+  lxc stop foo --force
 
   # cycle it a few times
   lxc start foo
   mac1=$(lxc exec foo cat /sys/class/net/eth0/address)
-  lxc stop foo --force # stop is hanging
+  lxc stop foo --force
   lxc start foo
   mac2=$(lxc exec foo cat /sys/class/net/eth0/address)
 
@@ -320,13 +365,23 @@ test_basic_usage() {
   lxc list last-used-at-test  --format json | jq -r '.[].last_used_at' | grep -v '1970-01-01T00:00:00Z'
   lxc delete last-used-at-test --force
 
+  # Test user, group and cwd
+  lxc exec foo -- mkdir /blah
+  [ "$(lxc exec foo --user 1000 -- id -u)" = "1000" ] || false
+  [ "$(lxc exec foo --group 1000 -- id -g)" = "1000" ] || false
+  [ "$(lxc exec foo --cwd /blah -- pwd)" = "/blah" ] || false
+
+  [ "$(lxc exec foo --user 1234 --group 5678 --cwd /blah -- id -u)" = "1234" ] || false
+  [ "$(lxc exec foo --user 1234 --group 5678 --cwd /blah -- id -g)" = "5678" ] || false
+  [ "$(lxc exec foo --user 1234 --group 5678 --cwd /blah -- pwd)" = "/blah" ] || false
+
   # check that we can set the environment
   lxc exec foo pwd | grep /root
   lxc exec --env BEST_BAND=meshuggah foo env | grep meshuggah
   lxc exec foo ip link show | grep eth0
 
   # check that we can get the return code for a non- wait-for-websocket exec
-  op=$(my_curl -X POST "https://${LXD_ADDR}/1.0/containers/foo/exec" -d '{"command": ["sleep", "1"], "environment": {}, "wait-for-websocket": false, "interactive": false}' | jq -r .operation)
+  op=$(my_curl -X POST "https://${LXD_ADDR}/1.0/containers/foo/exec" -d '{"command": ["echo", "test"], "environment": {}, "wait-for-websocket": false, "interactive": false}' | jq -r .operation)
   [ "$(my_curl "https://${LXD_ADDR}${op}/wait" | jq -r .metadata.metadata.return)" != "null" ]
 
   # test file transfer
@@ -365,7 +420,7 @@ test_basic_usage() {
   fi
 
   lxc launch testimage deleterunning
-  my_curl -X DELETE "https://${LXD_ADDR}/1.0/containers/deleterunning" | grep "container is running"
+  my_curl -X DELETE "https://${LXD_ADDR}/1.0/containers/deleterunning" | grep "Instance is running"
   lxc delete deleterunning -f
 
   # cleanup
@@ -385,13 +440,13 @@ test_basic_usage() {
 
     if [ "${MAJOR}" -gt "1" ] || { [ "${MAJOR}" = "1" ] && [ "${MINOR}" -ge "2" ]; }; then
       aa_namespace="lxd-lxd-apparmor-test_<$(echo "${LXD_DIR}" | sed -e 's/\//-/g' -e 's/^.//')>"
-      aa-status | grep -q ":${aa_namespace}:unconfined" || aa-status | grep -q ":${aa_namespace}://unconfined"
+      aa-status | grep -q ":${aa_namespace}:unconfined" || aa-status | grep -qF ":${aa_namespace}://unconfined"
       lxc stop lxd-apparmor-test --force
-      ! aa-status | grep -q ":${aa_namespace}:" || false
+      ! aa-status | grep -qF ":${aa_namespace}:" || false
     else
       aa-status | grep "lxd-lxd-apparmor-test_<${LXD_DIR}>"
       lxc stop lxd-apparmor-test --force
-      ! aa-status | grep -q "lxd-lxd-apparmor-test_<${LXD_DIR}>" || false
+      ! aa-status | grep -qF "lxd-lxd-apparmor-test_<${LXD_DIR}>" || false
     fi
     lxc delete lxd-apparmor-test
     [ ! -f "${LXD_DIR}/security/apparmor/profiles/lxd-lxd-apparmor-test" ]
@@ -400,20 +455,20 @@ test_basic_usage() {
   fi
 
   lxc launch testimage lxd-seccomp-test
-  init=$(lxc info lxd-seccomp-test | grep Pid | cut -f2 -d" ")
-  [ "$(grep Seccomp "/proc/${init}/status" | cut -f2)" -eq "2" ]
+  init=$(lxc info lxd-seccomp-test | awk '/^PID:/ {print $2}')
+  [ "$(awk '/^Seccomp:/ {print $2}' "/proc/${init}/status")" -eq "2" ]
   lxc stop --force lxd-seccomp-test
-  lxc config set lxd-seccomp-test security.syscalls.blacklist_default false
+  lxc config set lxd-seccomp-test security.syscalls.deny_default false
   lxc start lxd-seccomp-test
-  init=$(lxc info lxd-seccomp-test | grep Pid | cut -f2 -d" ")
-  [ "$(grep Seccomp "/proc/${init}/status" | cut -f2)" -eq "0" ]
+  init=$(lxc info lxd-seccomp-test | awk '/^PID:/ {print $2}')
+  [ "$(awk '/^Seccomp:/ {print $2}' "/proc/${init}/status")" -eq "0" ]
   lxc delete --force lxd-seccomp-test
 
   # make sure that privileged containers are not world-readable
   lxc profile create unconfined
   lxc profile set unconfined security.privileged true
   lxc init testimage foo2 -p unconfined -s "lxdtest-$(basename "${LXD_DIR}")"
-  [ "$(stat -L -c "%a" "${LXD_DIR}/containers/foo2")" = "700" ]
+  [ "$(stat -L -c "%a" "${LXD_DIR}/containers/foo2")" = "100" ]
   lxc delete foo2
   lxc profile delete unconfined
 
@@ -458,19 +513,22 @@ test_basic_usage() {
 
   # Ephemeral
   lxc launch testimage foo -e
-
-  OLD_INIT=$(lxc info foo | grep ^Pid)
-  lxc exec foo reboot || true
+  OLD_INIT=$(lxc info foo | awk '/^PID:/ {print $2}')
 
   REBOOTED="false"
 
   # shellcheck disable=SC2034
-  for i in $(seq 20); do
-    NEW_INIT=$(lxc info foo | grep ^Pid || true)
+  for i in $(seq 60); do
+    NEW_INIT=$(lxc info foo | awk '/^PID:/ {print $2}' || true)
 
-    if [ -n "${NEW_INIT}" ] && [ "${OLD_INIT}" != "${NEW_INIT}" ]; then
-      REBOOTED="true"
-      break
+    # If init process is running, check if is old or new process.
+    if [ -n "${NEW_INIT}" ]; then
+      if [ "${OLD_INIT}" != "${NEW_INIT}" ]; then
+        REBOOTED="true"
+        break
+      else
+        lxc exec foo reboot || true  # Signal to running old init processs to reboot if not rebooted yet.
+      fi
     fi
 
     sleep 0.5
@@ -478,14 +536,18 @@ test_basic_usage() {
 
   [ "${REBOOTED}" = "true" ]
 
-  # Workaround for LXC bug which causes LXD to double-start containers
-  # on reboot
-  sleep 2
+  lxc publish foo --alias foo --force
+  lxc image delete foo
 
-  lxc stop foo --force || true
+  lxc restart -f foo
+  lxc stop foo --force
   ! lxc list | grep -q foo || false
 
   # Test renaming/deletion of the default profile
-  ! lxc profile rename default foobar
-  ! lxc profile delete default
+  ! lxc profile rename default foobar || false
+  ! lxc profile delete default || false
+
+  lxc init testimage c1
+  ! lxc config device override c1 root pool bla || false
+  lxc rm -f c1
 }
